@@ -1,4 +1,4 @@
-from .datatypes import *
+from .datatypes import Array, Float, Int, Pointer, SampleHandle, String, Struct, TextureHandle
 
 class Sound(Struct):
 	def __init__(self, filename=""):
@@ -7,13 +7,13 @@ class Sound(Struct):
 		self.filename = String(filename)
 
 class SoundSet(Struct):
-	def __init__(self, name="", files=[]):
+	def __init__(self, name="", files=()):
 		Struct.__init__(self, "CDataSoundset")
 		self.name = String(name)
 		self.sounds = Array(Sound())
 		self.last = Int(-1)
-		for name in files:
-			self.sounds.Add(Sound(name))
+		for filename in files:
+			self.sounds.Add(Sound(filename))
 
 class Image(Struct):
 	def __init__(self, name="", filename="", linear_mapping=0):
@@ -24,7 +24,7 @@ class Image(Struct):
 		self.id = TextureHandle()
 
 class SpriteSet(Struct):
-	def __init__(self, name="", image=None, gridx=0, gridy=0):
+	def __init__(self, _name="", image=None, gridx=0, gridy=0):
 		Struct.__init__(self, "CDataSpriteset")
 		self.image = Pointer(Image, image) # TODO
 		self.gridx = Int(gridx)
@@ -70,7 +70,7 @@ class Animation(Struct):
 		self.attach = AnimSequence()
 
 class WeaponSpec(Struct):
-	def __init__(self, container=None, name=""):
+	def __init__(self, cont=None, name=""):
 		Struct.__init__(self, "CDataWeaponspec")
 		self.name = String(name)
 		self.sprite_body = Pointer(Sprite, Sprite())
@@ -91,11 +91,14 @@ class WeaponSpec(Struct):
 		self.muzzleduration = Float(5)
 
 		# dig out sprites if we have a container
-		if container:
-			for sprite in container.sprites.items:
-				if sprite.name.value == "weapon_"+name+"_body": self.sprite_body.Set(sprite)
-				elif sprite.name.value == "weapon_"+name+"_cursor": self.sprite_cursor.Set(sprite)
-				elif sprite.name.value == "weapon_"+name+"_proj": self.sprite_proj.Set(sprite)
+		if cont:
+			for sprite in cont.sprites.items:
+				if sprite.name.value == "weapon_"+name+"_body":
+					self.sprite_body.Set(sprite)
+				elif sprite.name.value == "weapon_"+name+"_cursor":
+					self.sprite_cursor.Set(sprite)
+				elif sprite.name.value == "weapon_"+name+"_proj":
+					self.sprite_proj.Set(sprite)
 				elif "weapon_"+name+"_muzzle" in sprite.name.value:
 					self.sprite_muzzles.Add(Pointer(Sprite, sprite))
 
@@ -173,10 +176,10 @@ class DataContainer(Struct):
 		self.sprites = Array(Sprite())
 		self.animations = Array(Animation())
 		self.weapons = Weapons()
-		self.explosion = Explosion()
+		#self.explosion = Explosion()
 
-def FileList(format, num):
-	return [format%(x+1) for x in range(0,num)]
+def FileList(fmt, num):
+	return [fmt%(x+1) for x in range(0,num)]
 
 container = DataContainer()
 container.sounds.Add(SoundSet("gun_fire", FileList("audio/wp_gun_fire-%02d.wv", 3)))
@@ -373,16 +376,16 @@ container.sprites.Add(Sprite("part9", set_game, 13,0,2,2))
 container.sprites.Add(Sprite("weapon_gun_body", set_game, 2,4,4,2))
 container.sprites.Add(Sprite("weapon_gun_cursor", set_game, 0,4,2,2))
 container.sprites.Add(Sprite("weapon_gun_proj", set_game, 6,4,2,2))
-container.sprites.Add(Sprite("weapon_gun_muzzle1", set_game, 8,4,3,2))
-container.sprites.Add(Sprite("weapon_gun_muzzle2", set_game, 12,4,3,2))
-container.sprites.Add(Sprite("weapon_gun_muzzle3", set_game, 16,4,3,2))
+container.sprites.Add(Sprite("weapon_gun_muzzle1", set_game, 8,4,4,2))
+container.sprites.Add(Sprite("weapon_gun_muzzle2", set_game, 12,4,4,2))
+container.sprites.Add(Sprite("weapon_gun_muzzle3", set_game, 16,4,4,2))
 
 container.sprites.Add(Sprite("weapon_shotgun_body", set_game, 2,6,8,2))
 container.sprites.Add(Sprite("weapon_shotgun_cursor", set_game, 0,6,2,2))
 container.sprites.Add(Sprite("weapon_shotgun_proj", set_game, 10,6,2,2))
-container.sprites.Add(Sprite("weapon_shotgun_muzzle1", set_game, 12,6,3,2))
-container.sprites.Add(Sprite("weapon_shotgun_muzzle2", set_game, 16,6,3,2))
-container.sprites.Add(Sprite("weapon_shotgun_muzzle3", set_game, 20,6,3,2))
+container.sprites.Add(Sprite("weapon_shotgun_muzzle1", set_game, 12,6,4,2))
+container.sprites.Add(Sprite("weapon_shotgun_muzzle2", set_game, 16,6,4,2))
+container.sprites.Add(Sprite("weapon_shotgun_muzzle3", set_game, 20,6,4,2))
 
 container.sprites.Add(Sprite("weapon_grenade_body", set_game, 2,8,7,2))
 container.sprites.Add(Sprite("weapon_grenade_cursor", set_game, 0,8,2,2))
@@ -633,7 +636,7 @@ weapon.ammoregentime.Set(500)
 weapon.visual_size.Set(64)
 weapon.offsetx.Set(32)
 weapon.offsety.Set(-4)
-weapon.muzzleoffsetx.Set(50)
+weapon.muzzleoffsetx.Set(50 + 8.8752) # see gun in 0.6 content.py for the number after the plus sign (TODO: also copy the comment from 0.6 content.py, if it gets removed)
 weapon.muzzleoffsety.Set(6)
 container.weapons.gun.base.Set(weapon)
 container.weapons.id.Add(weapon)
@@ -644,7 +647,7 @@ weapon.damage.Set(1)
 weapon.visual_size.Set(96)
 weapon.offsetx.Set(24)
 weapon.offsety.Set(-2)
-weapon.muzzleoffsetx.Set(70)
+weapon.muzzleoffsetx.Set(70 + 13.3128) # see gun in 0.6 content.py for the number after the plus sign (TODO: also copy the comment from 0.6 content.py, if it gets removed)
 weapon.muzzleoffsety.Set(6)
 container.weapons.shotgun.base.Set(weapon)
 container.weapons.id.Add(weapon)
