@@ -5,10 +5,6 @@ try:
 except ImportError:
 	import os
 	DEVNULL = open(os.devnull, 'wb')
-try:
-	FileNotFoundError
-except NameError:
-	FileNotFoundError = OSError
 try:
 	git_hash = subprocess.check_output(["git", "rev-parse", "--short=16", "HEAD"], stderr=DEVNULL).decode().strip()
 	definition = '"{}"'.format(git_hash)
@@ -17,5 +13,5 @@ except FileNotFoundError as e:
 		raise
 	definition = "0"
 except subprocess.CalledProcessError:
-	definition = "0";
+	definition = "0"
 print("const char *GIT_SHORTREV_HASH = {};".format(definition))
