@@ -2,8 +2,7 @@
 #define ANTIBOT_ANTIBOT_INTERFACE_H
 
 #include "antibot_data.h"
-extern "C"
-{
+extern "C" {
 
 int AntibotAbiVersion();
 void AntibotInit(CAntibotData *pCallbackData);
@@ -22,10 +21,9 @@ void AntibotOnDirectInput(int ClientID);
 void AntibotOnCharacterTick(int ClientID);
 void AntibotOnHookAttach(int ClientID, bool Player);
 void AntibotOnEngineTick(void);
-void AntibotOnEngineClientJoin(int ClientID);
+void AntibotOnEngineClientJoin(int ClientID, bool Sixup);
 void AntibotOnEngineClientDrop(int ClientID, const char *pReason);
 void AntibotOnEngineClientMessage(int ClientID, const void *pData, int Size, int Flags);
-
 }
 
 #endif // ANTIBOT_ANTIBOT_INTERFACE_H
