@@ -24,7 +24,7 @@ public:
 	// Commands
 	virtual void Dump() = 0;
 
-	virtual ~IAntibot() { };
+	virtual ~IAntibot(){};
 };
 
 class IEngineAntibot : public IAntibot
@@ -35,11 +35,11 @@ public:
 
 	// Hooks
 	virtual void OnEngineTick() = 0;
-	virtual void OnEngineClientJoin(int ClientID) = 0;
+	virtual void OnEngineClientJoin(int ClientID, bool Sixup) = 0;
 	virtual void OnEngineClientDrop(int ClientID, const char *pReason) = 0;
 	virtual void OnEngineClientMessage(int ClientID, const void *pData, int Size, int Flags) = 0;
 
-	virtual ~IEngineAntibot() { };
+	virtual ~IEngineAntibot(){};
 };
 
 #endif //ENGINE_ANTIBOT_H
