@@ -6,6 +6,7 @@
 
 #include "graphics.h"
 #include "message.h"
+#include <base/hash.h>
 #include <engine/friends.h>
 
 struct SWarning;
@@ -17,6 +18,8 @@ enum
 	RECORDER_RACE = 2,
 	RECORDER_REPLAYS = 3,
 	RECORDER_MAX = 4,
+
+	NUM_DUMMIES = 2,
 };
 
 typedef bool (*CLIENTFUNC_FILTER)(const void *pData, int DataSize, void *pUser);
@@ -29,13 +32,13 @@ protected:
 	int m_State;
 
 	// quick access to time variables
-	int m_PrevGameTick[2];
-	int m_CurGameTick[2];
-	float m_GameIntraTick[2];
-	float m_GameTickTime[2];
+	int m_PrevGameTick[NUM_DUMMIES];
+	int m_CurGameTick[NUM_DUMMIES];
+	float m_GameIntraTick[NUM_DUMMIES];
+	float m_GameTickTime[NUM_DUMMIES];
 
-	int m_PredTick[2];
-	float m_PredIntraTick[2];
+	int m_PredTick[NUM_DUMMIES];
+	float m_PredIntraTick[NUM_DUMMIES];
 
 	float m_LocalTime;
 	float m_RenderFrameTime;
@@ -46,6 +49,8 @@ protected:
 
 public:
 	char m_aNews[3000];
+	char m_aMapDownloadUrl[256];
+	int m_Points;
 	int64 m_ReconnectTime;
 
 	class CSnapItem
@@ -193,7 +198,8 @@ public:
 
 	virtual const char *GetCurrentMap() = 0;
 	virtual const char *GetCurrentMapPath() = 0;
-	virtual unsigned GetMapCrc() = 0;
+	virtual SHA256_DIGEST GetCurrentMapSha256() = 0;
+	virtual unsigned GetCurrentMapCrc() = 0;
 
 	virtual int GetCurrentRaceTime() = 0;
 
