@@ -4,6 +4,7 @@
 #include "SDL.h"
 #include "SDL_opengl.h"
 
+#include "blocklist_driver.h"
 #include "graphics_threaded.h"
 
 #include <base/tl/threading.h>
@@ -61,8 +62,8 @@ private:
 	ICommandProcessor *m_pProcessor;
 	CCommandBuffer *volatile m_pBuffer;
 	volatile bool m_Shutdown;
-	semaphore m_Activity;
-	semaphore m_BufferDone;
+	CSemaphore m_Activity;
+	CSemaphore m_BufferDone;
 	void *m_pThread;
 
 	static void ThreadFunc(void *pUser);
@@ -103,7 +104,7 @@ class CGLSLPrimitiveProgram;
 class CGLSLQuadProgram;
 class CGLSLTileProgram;
 class CGLSLTextProgram;
-class CGLSLSpriteProgram;
+class CGLSLPrimitiveExProgram;
 class CGLSLSpriteMultipleProgram;
 
 // takes care of opengl related rendering
@@ -113,7 +114,10 @@ protected:
 	struct CTexture
 	{
 		CTexture() :
-			m_Tex(0), m_Tex2DArray(0), m_Sampler(0), m_Sampler2DArray(0) {}
+			m_Tex(0), m_Tex2DArray(0), m_Sampler(0), m_Sampler2DArray(0), m_LastWrapMode(CCommandBuffer::WRAP_REPEAT), m_MemSize(0), m_Width(0), m_Height(0), m_RescaleCount(0), m_ResizeWidth(0), m_ResizeHeight(0)
+		{
+		}
+
 		GLuint m_Tex;
 		GLuint m_Tex2DArray; //or 3D texture as fallback
 		GLuint m_Sampler;
@@ -128,7 +132,7 @@ protected:
 		float m_ResizeWidth;
 		float m_ResizeHeight;
 	};
-	CTexture m_aTextures[CCommandBuffer::MAX_TEXTURES];
+	std::vector<CTexture> m_Textures;
 	std::atomic<int> *m_pTextureMemoryUsage;
 
 	GLint m_MaxTexSize;
@@ -170,6 +174,7 @@ public:
 	};
 
 protected:
+	bool IsTexturedState(const CCommandBuffer::SState &State);
 	void SetState(const CCommandBuffer::SState &State, bool Use2DArrayTexture = false);
 	virtual bool IsNewApi() { return false; }
 	void DestroyTexture(int Slot);
@@ -206,7 +211,7 @@ protected:
 	virtual void Cmd_RenderText(const CCommandBuffer::SCommand_RenderText *pCommand) {}
 	virtual void Cmd_RenderTextStream(const CCommandBuffer::SCommand_RenderTextStream *pCommand) {}
 	virtual void Cmd_RenderQuadContainer(const CCommandBuffer::SCommand_RenderQuadContainer *pCommand) {}
-	virtual void Cmd_RenderQuadContainerAsSprite(const CCommandBuffer::SCommand_RenderQuadContainerAsSprite *pCommand) {}
+	virtual void Cmd_RenderQuadContainerEx(const CCommandBuffer::SCommand_RenderQuadContainerEx *pCommand) {}
 	virtual void Cmd_RenderQuadContainerAsSpriteMultiple(const CCommandBuffer::SCommand_RenderQuadContainerAsSpriteMultiple *pCommand) {}
 
 public:
@@ -310,6 +315,7 @@ class CCommandProcessorFragment_OpenGL3_3 : public CCommandProcessorFragment_Ope
 	static const int m_MaxQuadsPossible = 256;
 
 	CGLSLPrimitiveProgram *m_pPrimitiveProgram;
+	CGLSLPrimitiveProgram *m_pPrimitiveProgramTextured;
 	CGLSLTileProgram *m_pBorderTileProgram;
 	CGLSLTileProgram *m_pBorderTileProgramTextured;
 	CGLSLTileProgram *m_pBorderTileLineProgram;
@@ -317,7 +323,10 @@ class CCommandProcessorFragment_OpenGL3_3 : public CCommandProcessorFragment_Ope
 	CGLSLQuadProgram *m_pQuadProgram;
 	CGLSLQuadProgram *m_pQuadProgramTextured;
 	CGLSLTextProgram *m_pTextProgram;
-	CGLSLSpriteProgram *m_pSpriteProgram;
+	CGLSLPrimitiveExProgram *m_pPrimitiveExProgram;
+	CGLSLPrimitiveExProgram *m_pPrimitiveExProgramTextured;
+	CGLSLPrimitiveExProgram *m_pPrimitiveExProgramRotationless;
+	CGLSLPrimitiveExProgram *m_pPrimitiveExProgramTexturedRotationless;
 	CGLSLSpriteMultipleProgram *m_pSpriteProgramMultiple;
 
 	GLuint m_LastProgramID;
@@ -352,6 +361,8 @@ class CCommandProcessorFragment_OpenGL3_3 : public CCommandProcessorFragment_Ope
 
 	CCommandBuffer::SColorf m_ClearColor;
 
+	void InitPrimExProgram(CGLSLPrimitiveExProgram *pProgram, class CGLSLCompiler *pCompiler, class IStorage *pStorage, bool Textured, bool Rotationless);
+
 protected:
 	static int TexFormatToNewOpenGLFormat(int TexFormat);
 	bool IsNewApi() override { return true; }
@@ -388,7 +399,7 @@ protected:
 	void Cmd_RenderText(const CCommandBuffer::SCommand_RenderText *pCommand) override;
 	void Cmd_RenderTextStream(const CCommandBuffer::SCommand_RenderTextStream *pCommand) override;
 	void Cmd_RenderQuadContainer(const CCommandBuffer::SCommand_RenderQuadContainer *pCommand) override;
-	void Cmd_RenderQuadContainerAsSprite(const CCommandBuffer::SCommand_RenderQuadContainerAsSprite *pCommand) override;
+	void Cmd_RenderQuadContainerEx(const CCommandBuffer::SCommand_RenderQuadContainerEx *pCommand) override;
 	void Cmd_RenderQuadContainerAsSpriteMultiple(const CCommandBuffer::SCommand_RenderQuadContainerAsSpriteMultiple *pCommand) override;
 
 public:
@@ -418,6 +429,8 @@ public:
 		SDL_GLContext m_GLContext;
 		SBackendCapabilites *m_pCapabilities;
 
+		const char **m_pErrStringPtr;
+
 		int *m_pInitError;
 
 		int m_RequestedMajor;
@@ -486,6 +499,8 @@ class CGraphicsBackend_SDL_OpenGL : public CGraphicsBackend_Threaded
 
 	bool m_UseNewOpenGL;
 
+	char m_aErrorString[256];
+
 public:
 	virtual int Init(const char *pName, int *Screen, int *pWidth, int *pHeight, int FsaaSamples, int Flags, int *pDesktopWidth, int *pDesktopHeight, int *pCurrentWidth, int *pCurrentHeight, class IStorage *pStorage);
 	virtual int Shutdown();
@@ -503,6 +518,8 @@ public:
 	virtual int WindowActive();
 	virtual int WindowOpen();
 	virtual void SetWindowGrab(bool Grab);
+	virtual void ResizeWindow(int w, int h);
+	virtual void GetViewportSize(int &w, int &h);
 	virtual void NotifyWindow();
 
 	virtual bool IsNewOpenGL() { return m_UseNewOpenGL; }
@@ -511,6 +528,14 @@ public:
 	virtual bool HasTextBuffering() { return m_Capabilites.m_TextBuffering; }
 	virtual bool HasQuadContainerBuffering() { return m_Capabilites.m_QuadContainerBuffering; }
 	virtual bool Has2DTextureArrays() { return m_Capabilites.m_2DArrayTextures; }
+
+	virtual const char *GetErrorString()
+	{
+		if(m_aErrorString[0] != '\0')
+			return m_aErrorString;
+
+		return NULL;
+	}
 };
 
 #endif // ENGINE_CLIENT_BACKEND_SDL_H
