@@ -10,9 +10,11 @@
 #include <engine/client.h>
 #include <engine/client/demoedit.h>
 #include <engine/client/friends.h>
+#include <engine/client/ghost.h>
 #include <engine/client/http.h>
 #include <engine/client/serverbrowser.h>
 #include <engine/client/updater.h>
+#include <engine/discord.h>
 #include <engine/editor.h>
 #include <engine/engine.h>
 #include <engine/graphics.h>
@@ -22,7 +24,6 @@
 #include <engine/shared/config.h>
 #include <engine/shared/demo.h>
 #include <engine/shared/fifo.h>
-#include <engine/shared/ghost.h>
 #include <engine/shared/network.h>
 #include <engine/sound.h>
 #include <engine/steam.h>
@@ -90,9 +91,12 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	IEngineSound *m_pSound;
 	IGameClient *m_pGameClient;
 	IEngineMap *m_pMap;
+	IConfigManager *m_pConfigManager;
+	CConfig *m_pConfig;
 	IConsole *m_pConsole;
 	IStorage *m_pStorage;
 	IUpdater *m_pUpdater;
+	IDiscord *m_pDiscord;
 	ISteam *m_pSteam;
 	IEngineMasterServer *m_pMasterServer;
 
@@ -104,7 +108,7 @@ class CClient : public IClient, public CDemoPlayer::IListener
 
 	enum
 	{
-		CLIENT_MAIN,
+		CLIENT_MAIN = 0,
 		CLIENT_DUMMY,
 		CLIENT_CONTACT,
 		NUM_CLIENTS,
@@ -125,7 +129,7 @@ class CClient : public IClient, public CDemoPlayer::IListener
 
 	CUuid m_ConnectionID;
 
-	unsigned m_SnapshotParts[2];
+	unsigned m_SnapshotParts[NUM_DUMMIES];
 	int64 m_LocalStartTime;
 
 	IGraphics::CTextureHandle m_DebugFont;
@@ -145,9 +149,9 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	bool m_SoundInitFailed;
 	bool m_ResortServerBrowser;
 
-	int m_AckGameTick[2];
-	int m_CurrentRecvTick[2];
-	int m_RconAuthed[2];
+	int m_AckGameTick[NUM_DUMMIES];
+	int m_CurrentRecvTick[NUM_DUMMIES];
+	int m_RconAuthed[NUM_DUMMIES];
 	char m_RconPassword[32];
 	int m_UseTempRconCommands;
 	char m_Password[32];
@@ -163,8 +167,8 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	char m_aCurrentMap[MAX_PATH_LENGTH];
 	char m_aCurrentMapPath[MAX_PATH_LENGTH];
 
-	char m_aTimeoutCodes[2][32];
-	bool m_aTimeoutCodeSent[2];
+	char m_aTimeoutCodes[NUM_DUMMIES][32];
+	bool m_aTimeoutCodeSent[NUM_DUMMIES];
 	bool m_GenerateTimeoutSeed;
 
 	//
@@ -195,7 +199,7 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	char m_aDummyNameBuf[16];
 
 	// time
-	CSmoothTime m_GameTime[2];
+	CSmoothTime m_GameTime[NUM_DUMMIES];
 	CSmoothTime m_PredictedTime;
 
 	// input
@@ -205,9 +209,9 @@ class CClient : public IClient, public CDemoPlayer::IListener
 		int m_Tick; // the tick that the input is for
 		int64 m_PredictedTime; // prediction latency when we sent this input
 		int64 m_Time;
-	} m_aInputs[2][200];
+	} m_aInputs[NUM_DUMMIES][200];
 
-	int m_CurrentInput[2];
+	int m_CurrentInput[NUM_DUMMIES];
 	bool m_LastDummy;
 	bool m_DummySendConnInfo;
 
@@ -217,10 +221,10 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	CGraph m_FpsGraph;
 
 	// the game snapshots are modifiable by the game
-	class CSnapshotStorage m_SnapshotStorage[2];
-	CSnapshotStorage::CHolder *m_aSnapshots[2][NUM_SNAPSHOT_TYPES];
+	class CSnapshotStorage m_SnapshotStorage[NUM_DUMMIES];
+	CSnapshotStorage::CHolder *m_aSnapshots[NUM_DUMMIES][NUM_SNAPSHOT_TYPES];
 
-	int m_ReceivedSnapshots[2];
+	int m_ReceivedSnapshots[NUM_DUMMIES];
 	char m_aSnapshotIncomingData[CSnapshot::MAX_SIZE];
 
 	class CSnapshotStorage::CHolder m_aDemorecSnapshotHolders[NUM_SNAPSHOT_TYPES];
@@ -264,6 +268,9 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	CFifo m_Fifo;
 #endif
 
+	IOHANDLE m_BenchmarkFile;
+	int64 m_BenchmarkStopTime;
+
 public:
 	IEngine *Engine() { return m_pEngine; }
 	IEngineGraphics *Graphics() { return m_pGraphics; }
@@ -271,8 +278,11 @@ public:
 	IEngineSound *Sound() { return m_pSound; }
 	IGameClient *GameClient() { return m_pGameClient; }
 	IEngineMasterServer *MasterServer() { return m_pMasterServer; }
+	IConfigManager *ConfigManager() { return m_pConfigManager; }
+	CConfig *Config() { return m_pConfig; }
 	IStorage *Storage() { return m_pStorage; }
 	IUpdater *Updater() { return m_pUpdater; }
+	IDiscord *Discord() { return m_pDiscord; }
 	ISteam *Steam() { return m_pSteam; }
 
 	CClient();
@@ -352,7 +362,7 @@ public:
 	const char *LoadMap(const char *pName, const char *pFilename, SHA256_DIGEST *pWantedSha256, unsigned WantedCrc);
 	const char *LoadMapSearch(const char *pMapName, SHA256_DIGEST *pWantedSha256, int WantedCrc);
 
-	static int PlayerScoreNameComp(const void *a, const void *b);
+	static bool PlayerScoreNameLess(const CServerInfo::CClient &p0, const CServerInfo::CClient &p1);
 
 	void ProcessConnlessPacket(CNetChunk *pPacket);
 	void ProcessServerInfo(int Type, NETADDR *pFrom, const void *pData, int DataSize);
@@ -415,6 +425,7 @@ public:
 	static void Con_Record(IConsole::IResult *pResult, void *pUserData);
 	static void Con_StopRecord(IConsole::IResult *pResult, void *pUserData);
 	static void Con_AddDemoMarker(IConsole::IResult *pResult, void *pUserData);
+	static void Con_BenchmarkQuit(IConsole::IResult *pResult, void *pUserData);
 	static void ConchainServerBrowserUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainFullscreen(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainWindowBordered(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
@@ -461,6 +472,7 @@ public:
 	void ToggleWindowVSync();
 	void LoadFont();
 	void Notify(const char *pTitle, const char *pMessage);
+	void BenchmarkQuit(int Seconds, const char *pFilename);
 
 	// DDRace
 
@@ -471,7 +483,8 @@ public:
 
 	virtual const char *GetCurrentMap();
 	virtual const char *GetCurrentMapPath();
-	virtual unsigned GetMapCrc();
+	virtual SHA256_DIGEST GetCurrentMapSha256();
+	virtual unsigned GetCurrentMapCrc();
 
 	virtual void RaceRecord_Start(const char *pFilename);
 	virtual void RaceRecord_Stop();
