@@ -7,6 +7,10 @@
 #include <engine/storage.h>
 #include <game/version.h>
 
+#if !defined(CONF_FAMILY_WINDOWS)
+#include <signal.h>
+#endif
+
 #define WIN32_LEAN_AND_MEAN
 #include "curl/curl.h"
 #include "curl/easy.h"
@@ -24,7 +28,7 @@ static int GetLockIndex(int Data)
 	return Data;
 }
 
-static void CurlLock(CURL *pHandle, curl_lock_data Data, curl_lock_access Access, void *pUser)
+static void CurlLock(CURL *pHandle, curl_lock_data Data, curl_lock_access Access, void *pUser) ACQUIRE(gs_aLocks[GetLockIndex(Data)])
 {
 	(void)pHandle;
 	(void)Access;
@@ -32,7 +36,7 @@ static void CurlLock(CURL *pHandle, curl_lock_data Data, curl_lock_access Access
 	lock_wait(gs_aLocks[GetLockIndex(Data)]);
 }
 
-static void CurlUnlock(CURL *pHandle, curl_lock_data Data, void *pUser)
+static void CurlUnlock(CURL *pHandle, curl_lock_data Data, void *pUser) RELEASE(gs_aLocks[GetLockIndex(Data)])
 {
 	(void)pHandle;
 	(void)pUser;
@@ -56,15 +60,22 @@ bool HttpInit(IStorage *pStorage)
 		dbg_msg("http", "libcurl version %s (compiled = " LIBCURL_VERSION ")", pVersion->version);
 	}
 
-	for(unsigned int i = 0; i < sizeof(gs_aLocks) / sizeof(gs_aLocks[0]); i++)
+	for(auto &Lock : gs_aLocks)
 	{
-		gs_aLocks[i] = lock_create();
+		Lock = lock_create();
 	}
 	curl_share_setopt(gs_Share, CURLSHOPT_SHARE, CURL_LOCK_DATA_DNS);
 	curl_share_setopt(gs_Share, CURLSHOPT_SHARE, CURL_LOCK_DATA_SSL_SESSION);
 	curl_share_setopt(gs_Share, CURLSHOPT_SHARE, CURL_LOCK_DATA_CONNECT);
 	curl_share_setopt(gs_Share, CURLSHOPT_LOCKFUNC, CurlLock);
 	curl_share_setopt(gs_Share, CURLSHOPT_UNLOCKFUNC, CurlUnlock);
+
+#if !defined(CONF_FAMILY_WINDOWS)
+	// As a multithreaded application we have to tell curl to not install signal
+	// handlers and instead ignore SIGPIPE from OpenSSL ourselves.
+	signal(SIGPIPE, SIG_IGN);
+#endif
+
 	return false;
 }
 
@@ -250,8 +261,8 @@ size_t CGet::OnData(char *pData, size_t DataSize)
 CGetFile::CGetFile(IStorage *pStorage, const char *pUrl, const char *pDest, int StorageType, CTimeout Timeout, bool LogProgress) :
 	CRequest(pUrl, Timeout, LogProgress),
 	m_pStorage(pStorage),
-	m_StorageType(StorageType),
-	m_File(0)
+	m_File(0),
+	m_StorageType(StorageType)
 {
 	str_copy(m_aDest, pDest, sizeof(m_aDest));
 
@@ -287,6 +298,7 @@ int CGetFile::OnCompletion(int State)
 {
 	if(m_File && io_close(m_File) != 0)
 	{
+		dbg_msg("http", "i/o error, cannot close file: %s", m_aDest);
 		State = HTTP_ERROR;
 	}
 
