@@ -45,13 +45,12 @@ class CGLSLTWProgram : public CGLSLProgram
 {
 public:
 	CGLSLTWProgram() :
-		m_LocPos(-1), m_LocIsTextured(-1), m_LocTextureSampler(-1), m_LastTextureSampler(-1), m_LastIsTextured(-1)
+		m_LocPos(-1), m_LocTextureSampler(-1), m_LastTextureSampler(-1), m_LastIsTextured(-1)
 	{
 		m_LastScreen[0] = m_LastScreen[1] = m_LastScreen[2] = m_LastScreen[3] = -1.f;
 	}
 
 	int m_LocPos;
-	int m_LocIsTextured;
 	int m_LocTextureSampler;
 
 	int m_LastTextureSampler;
@@ -89,10 +88,10 @@ class CGLSLPrimitiveProgram : public CGLSLTWProgram
 public:
 };
 
-class CGLSLSpriteProgram : public CGLSLTWProgram
+class CGLSLPrimitiveExProgram : public CGLSLTWProgram
 {
 public:
-	CGLSLSpriteProgram() :
+	CGLSLPrimitiveExProgram() :
 		CGLSLTWProgram()
 	{
 		m_LastRotation = 0.f;
@@ -133,6 +132,7 @@ public:
 	int m_LocColors;
 	int m_LocOffsets;
 	int m_LocRotations;
+	int m_LocQuadOffset;
 };
 
 class CGLSLTileProgram : public CGLSLTWProgram
