@@ -66,10 +66,18 @@ CServerBrowser::CServerBrowser()
 	m_RequestNumber = 0;
 
 	m_pDDNetInfo = 0;
+
+	m_SortOnNextUpdate = false;
 }
 
 CServerBrowser::~CServerBrowser()
 {
+	if(m_ppServerlist)
+		free(m_ppServerlist);
+
+	if(m_pSortedServerlist)
+		free(m_pSortedServerlist);
+
 	if(m_pDDNetInfo)
 		json_value_free(m_pDDNetInfo);
 }
@@ -81,9 +89,9 @@ void CServerBrowser::SetBaseInfo(class CNetClient *pClient, const char *pNetVers
 	m_pMasterServer = Kernel()->RequestInterface<IMasterServer>();
 	m_pConsole = Kernel()->RequestInterface<IConsole>();
 	m_pFriends = Kernel()->RequestInterface<IFriends>();
-	IConfig *pConfig = Kernel()->RequestInterface<IConfig>();
-	if(pConfig)
-		pConfig->RegisterCallback(ConfigSaveCallback, this);
+	IConfigManager *pConfigManager = Kernel()->RequestInterface<IConfigManager>();
+	if(pConfigManager)
+		pConfigManager->RegisterCallback(ConfigSaveCallback, this);
 }
 
 const CServerInfo *CServerBrowser::SortedGet(int Index) const
@@ -164,10 +172,10 @@ bool CServerBrowser::SortCompareNumPlayersAndPing(int Index1, int Index2) const
 
 	if(a->m_Info.m_NumFilteredPlayers == b->m_Info.m_NumFilteredPlayers)
 		return a->m_Info.m_Latency > b->m_Info.m_Latency;
-	if(a->m_Info.m_NumFilteredPlayers == 0 || b->m_Info.m_NumFilteredPlayers == 0)
-		return a->m_Info.m_NumFilteredPlayers == 0;
+	else if(a->m_Info.m_NumFilteredPlayers == 0 || b->m_Info.m_NumFilteredPlayers == 0 || a->m_Info.m_Latency / 100 == b->m_Info.m_Latency / 100)
+		return a->m_Info.m_NumFilteredPlayers < b->m_Info.m_NumFilteredPlayers;
 	else
-		return a->m_Info.m_NumFilteredPlayers - (a->m_Info.m_Latency / 100) * MAX_CLIENTS < b->m_Info.m_NumFilteredPlayers - (b->m_Info.m_Latency / 100) * MAX_CLIENTS;
+		return a->m_Info.m_Latency > b->m_Info.m_Latency;
 }
 
 void CServerBrowser::Filter()
@@ -213,7 +221,7 @@ void CServerBrowser::Filter()
 			{
 				Filtered = 1;
 				// match against player country
-				for(p = 0; p < m_ppServerlist[i]->m_Info.m_NumClients; p++)
+				for(p = 0; p < minimum(m_ppServerlist[i]->m_Info.m_NumClients, (int)MAX_CLIENTS); p++)
 				{
 					if(m_ppServerlist[i]->m_Info.m_aClients[p].m_Country == g_Config.m_BrFilterCountryIndex)
 					{
@@ -237,7 +245,7 @@ void CServerBrowser::Filter()
 				}
 
 				// match against players
-				for(p = 0; p < m_ppServerlist[i]->m_Info.m_NumClients; p++)
+				for(p = 0; p < minimum(m_ppServerlist[i]->m_Info.m_NumClients, (int)MAX_CLIENTS); p++)
 				{
 					if(str_find_nocase(m_ppServerlist[i]->m_Info.m_aClients[p].m_aName, g_Config.m_BrFilterString) ||
 						str_find_nocase(m_ppServerlist[i]->m_Info.m_aClients[p].m_aClan, g_Config.m_BrFilterString))
@@ -290,7 +298,7 @@ void CServerBrowser::Filter()
 		{
 			// check for friend
 			m_ppServerlist[i]->m_Info.m_FriendState = IFriends::FRIEND_NO;
-			for(p = 0; p < m_ppServerlist[i]->m_Info.m_NumClients; p++)
+			for(p = 0; p < minimum(m_ppServerlist[i]->m_Info.m_NumClients, (int)MAX_CLIENTS); p++)
 			{
 				m_ppServerlist[i]->m_Info.m_aClients[p].m_FriendState = m_pFriends->GetFriendState(m_ppServerlist[i]->m_Info.m_aClients[p].m_aName,
 					m_ppServerlist[i]->m_Info.m_aClients[p].m_aClan);
@@ -328,10 +336,8 @@ void SetFilteredPlayers(const CServerInfo &Item)
 		Item.m_NumFilteredPlayers = Item.m_NumClients;
 	if(g_Config.m_BrFilterConnectingPlayers)
 	{
-		for(int i = 0; i < MAX_CLIENTS; i++)
+		for(const auto &Client : Item.m_aClients)
 		{
-			const CServerInfo::CClient &Client = Item.m_aClients[i];
-
 			if(str_comp(Client.m_aName, "(connecting)") == 0 && Client.m_aClan[0] == '\0' && Client.m_Country == -1 && Client.m_Score == 0)
 				Item.m_NumFilteredPlayers--;
 		}
@@ -365,10 +371,6 @@ void CServerBrowser::Sort()
 	else if(g_Config.m_BrSort == IServerBrowser::SORT_GAMETYPE)
 		std::stable_sort(m_pSortedServerlist, m_pSortedServerlist + m_NumSortedServers, SortWrap(this, &CServerBrowser::SortCompareGametype));
 
-	// set indexes
-	for(i = 0; i < m_NumSortedServers; i++)
-		m_ppServerlist[m_pSortedServerlist[i]]->m_Info.m_SortedIndex = i;
-
 	str_copy(m_aFilterGametypeString, g_Config.m_BrFilterGametype, sizeof(m_aFilterGametypeString));
 	str_copy(m_aFilterString, g_Config.m_BrFilterString, sizeof(m_aFilterString));
 	m_Sorthash = SortHash();
@@ -475,11 +477,11 @@ CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR &Addr)
 	}
 
 	// check if it's an official server
-	for(int Network = 0; Network < NUM_NETWORKS; Network++)
+	for(auto &Network : m_aNetworks)
 	{
-		for(int i = 0; i < m_aNetworks[Network].m_NumCountries; i++)
+		for(int i = 0; i < Network.m_NumCountries; i++)
 		{
-			CNetworkCountry *pCntr = &m_aNetworks[Network].m_aCountries[i];
+			CNetworkCountry *pCntr = &Network.m_aCountries[i];
 			for(int j = 0; j < pCntr->m_NumServers; j++)
 			{
 				if(net_addr_comp(&Addr, &pCntr->m_aServers[j]) == 0)
@@ -499,8 +501,9 @@ CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR &Addr)
 	{
 		CServerEntry **ppNewlist;
 		m_NumServerCapacity += 100;
-		ppNewlist = (CServerEntry **)calloc(m_NumServerCapacity, sizeof(CServerEntry *));
-		mem_copy(ppNewlist, m_ppServerlist, m_NumServers * sizeof(CServerEntry *));
+		ppNewlist = (CServerEntry **)calloc(m_NumServerCapacity, sizeof(CServerEntry *)); // NOLINT(bugprone-sizeof-expression)
+		if(m_NumServers > 0)
+			mem_copy(ppNewlist, m_ppServerlist, m_NumServers * sizeof(CServerEntry *)); // NOLINT(bugprone-sizeof-expression)
 		free(m_ppServerlist);
 		m_ppServerlist = ppNewlist;
 	}
@@ -616,7 +619,7 @@ void CServerBrowser::Set(const NETADDR &Addr, int Type, int Token, const CServer
 		RemoveRequest(pEntry);
 	}
 
-	Sort();
+	m_SortOnNextUpdate = true;
 }
 
 void CServerBrowser::Refresh(int Type)
@@ -976,8 +979,11 @@ void CServerBrowser::Update(bool ForceResort)
 	}
 
 	// check if we need to resort
-	if(m_Sorthash != SortHash() || ForceResort)
+	if(m_Sorthash != SortHash() || ForceResort || m_SortOnNextUpdate)
+	{
 		Sort();
+		m_SortOnNextUpdate = false;
+	}
 }
 
 bool CServerBrowser::IsFavorite(const NETADDR &Addr) const
@@ -1134,11 +1140,11 @@ void CServerBrowser::LoadDDNetServers()
 
 void CServerBrowser::RecheckOfficial()
 {
-	for(int Network = 0; Network < NUM_NETWORKS; Network++)
+	for(auto &Network : m_aNetworks)
 	{
-		for(int i = 0; i < m_aNetworks[Network].m_NumCountries; i++)
+		for(int i = 0; i < Network.m_NumCountries; i++)
 		{
-			CNetworkCountry *pCntr = &m_aNetworks[Network].m_aCountries[i];
+			CNetworkCountry *pCntr = &Network.m_aCountries[i];
 			for(int j = 0; j < pCntr->m_NumServers; j++)
 			{
 				CServerEntry *pEntry = Find(pCntr->m_aServers[j]);
@@ -1258,7 +1264,7 @@ int CServerBrowser::LoadingProgression() const
 	return 100.0f * Loaded / Servers;
 }
 
-void CServerBrowser::ConfigSaveCallback(IConfig *pConfig, void *pUserData)
+void CServerBrowser::ConfigSaveCallback(IConfigManager *pConfigManager, void *pUserData)
 {
 	CServerBrowser *pSelf = (CServerBrowser *)pUserData;
 
@@ -1268,7 +1274,7 @@ void CServerBrowser::ConfigSaveCallback(IConfig *pConfig, void *pUserData)
 	{
 		net_addr_str(&pSelf->m_aFavoriteServers[i], aAddrStr, sizeof(aAddrStr), true);
 		str_format(aBuffer, sizeof(aBuffer), "add_favorite %s", aAddrStr);
-		pConfig->WriteLine(aBuffer);
+		pConfigManager->WriteLine(aBuffer);
 	}
 }
 
@@ -1316,11 +1322,11 @@ void CServerBrowser::CountryFilterClean(int Network)
 	char aNewList[128];
 	aNewList[0] = '\0';
 
-	for(int Network = 0; Network < NUM_NETWORKS; Network++)
+	for(auto &Network : m_aNetworks)
 	{
-		for(int i = 0; i < m_aNetworks[Network].m_NumCountries; i++)
+		for(int i = 0; i < Network.m_NumCountries; i++)
 		{
-			const char *pName = m_aNetworks[Network].m_aCountries[i].m_aName;
+			const char *pName = Network.m_aCountries[i].m_aName;
 			if(DDNetFiltered(pExcludeCountries, pName))
 			{
 				char aBuf[128];
