@@ -45,8 +45,8 @@ class CUpdater : public IUpdater
 	LOCK m_Lock;
 
 	int m_State;
-	char m_aStatus[256];
-	int m_Percent;
+	char m_aStatus[256] GUARDED_BY(m_Lock);
+	int m_Percent GUARDED_BY(m_Lock);
 	char m_aLastFile[256];
 	char m_aClientExecTmp[64];
 	char m_aServerExecTmp[64];
