@@ -11,7 +11,7 @@
 #define STREAM_PIX_FMT AV_PIX_FMT_YUV420P /* default pix_fmt */
 
 const size_t FORMAT_NCHANNELS = 3;
-static LOCK g_WriteLock = 0;
+LOCK g_WriteLock = 0;
 
 CVideo::CVideo(CGraphics_Threaded *pGraphics, IStorage *pStorage, IConsole *pConsole, int Width, int Height, const char *pName) :
 	m_pGraphics(pGraphics),
@@ -278,7 +278,7 @@ void CVideo::NextAudioFrame(void (*Mix)(short *pFinalOut, unsigned Frames))
 	if(m_NextAudioFrame && m_Recording && m_HasAudio)
 	{
 		m_ProcessingAudioFrame = true;
-		//dbg_msg("video recorder", "video_frame: %lf", (double)(m_Vframe/m_FPS));
+		//dbg_msg("video_recorder", "video_frame: %lf", (double)(m_Vframe/m_FPS));
 		//if((double)(m_Vframe/m_FPS) < m_AudioStream.pEnc->frame_number*m_AudioStream.pEnc->frame_size/m_AudioStream.pEnc->sample_rate)
 		//return;
 		Mix(m_aBuffer, ALEN);
@@ -545,7 +545,7 @@ bool CVideo::OpenAudio()
 	av_opt_set_sample_fmt(m_AudioStream.pSwrCtx, "out_sample_fmt", c->sample_fmt, 0);
 
 	/* initialize the resampling context */
-	if((Ret = swr_init(m_AudioStream.pSwrCtx)) < 0)
+	if(swr_init(m_AudioStream.pSwrCtx) < 0)
 	{
 		dbg_msg("video_recorder", "Failed to initialize the resampling context");
 		return false;
@@ -667,7 +667,6 @@ bool CVideo::AddStream(OutputStream *pStream, AVFormatContext *pOC, AVCodec **pp
 
 void CVideo::WriteFrame(OutputStream *pStream)
 {
-	//lock_wait(g_WriteLock);
 	int RetRecv = 0;
 
 	AVPacket Packet = {0};
@@ -701,7 +700,6 @@ void CVideo::WriteFrame(OutputStream *pStream)
 	{
 		dbg_msg("video_recorder", "Error encoding frame, error: %d", RetRecv);
 	}
-	//lock_unlock(g_WriteLock);
 }
 
 void CVideo::FinishFrames(OutputStream *pStream)
