@@ -12,7 +12,8 @@ public:
 	int m_MsgID;
 	bool m_System;
 	bool m_NoTranslate;
-	CMsgPacker(int Type, bool System = false, bool NoTranslate = false) : m_MsgID(Type), m_System(System), m_NoTranslate(NoTranslate)
+	CMsgPacker(int Type, bool System = false, bool NoTranslate = false) :
+		m_MsgID(Type), m_System(System), m_NoTranslate(NoTranslate)
 	{
 		Reset();
 	}
