@@ -21,7 +21,7 @@ public:
 	virtual IDbConnection *Copy() = 0;
 
 	// returns the database prefix
-	const char *GetPrefix() { return m_aPrefix; }
+	const char *GetPrefix() const { return m_aPrefix; }
 	virtual const char *BinaryCollate() const = 0;
 	// can be inserted into queries to convert a timestamp variable to the unix timestamp
 	virtual void ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsigned int BufferSize) = 0;
@@ -33,6 +33,10 @@ public:
 	virtual const char *CollateNocase() const = 0;
 	// syntax to insert a row into table or ignore if it already exists
 	virtual const char *InsertIgnore() const = 0;
+	// ORDER BY RANDOM()/RAND()
+	virtual const char *Random() const = 0;
+	// Get Median Map Time from l.Map
+	virtual const char *MedianMapTime(char *pBuffer, int BufferSize) const = 0;
 
 	enum Status
 	{
@@ -45,10 +49,6 @@ public:
 	// has to be called to return the connection back to the pool
 	virtual void Disconnect() = 0;
 
-	// get exclusive read/write access to the database
-	virtual void Lock(const char *pTable) = 0;
-	virtual void Unlock() = 0;
-
 	// ? for Placeholders, connection has to be established, can overwrite previous prepared statements
 	virtual void PrepareStatement(const char *pStmt) = 0;
 
@@ -63,6 +63,9 @@ public:
 	// executes the query and returns if a result row exists and selects it
 	// when called multiple times the next row is selected
 	virtual bool Step() = 0;
+	// executes the query and returns the number of rows affected by the update/insert/delete
+	// FIXME(2020-01-20): change function to AffectedRows() when moved to c-api of MySQL
+	virtual int ExecuteUpdate() = 0;
 
 	virtual bool IsNull(int Col) const = 0;
 	virtual float GetFloat(int Col) const = 0;
