@@ -29,6 +29,7 @@ class CRegister
 
 	class CNetServer *m_pNetServer;
 	class IEngineMasterServer *m_pMasterServer;
+	class CConfig *m_pConfig;
 	class IConsole *m_pConsole;
 
 	bool m_Sixup;
@@ -51,7 +52,7 @@ class CRegister
 
 public:
 	CRegister(bool Sixup);
-	void Init(class CNetServer *pNetServer, class IEngineMasterServer *pMasterServer, class IConsole *pConsole);
+	void Init(class CNetServer *pNetServer, class IEngineMasterServer *pMasterServer, class CConfig *pConfig, class IConsole *pConsole);
 	void RegisterUpdate(int Nettype);
 	int RegisterProcessPacket(struct CNetChunk *pPacket, SECURITY_TOKEN ResponseToken = 0);
 	void FeedToken(NETADDR Addr, SECURITY_TOKEN ResponseToken);
