@@ -4,24 +4,24 @@
 #include <cmath>
 #include <cstring>
 
-void sqlstr::FuzzyString(char *pString, int size)
+void sqlstr::FuzzyString(char *pString, int Size)
 {
-	char *newString = new char[size * 4 - 1];
-	int pos = 0;
+	char *pNewString = new char[Size * 4 - 1];
+	int OutPos = 0;
 
-	for(int i = 0; i < size; i++)
+	for(int i = 0; i < Size; i++)
 	{
 		if(!pString[i])
 			break;
 
-		newString[pos++] = pString[i];
+		pNewString[OutPos++] = pString[i];
 		if(pString[i] != '\\' && str_utf8_isstart(pString[i + 1]))
-			newString[pos++] = '%';
+			pNewString[OutPos++] = '%';
 	}
 
-	newString[pos] = '\0';
-	str_copy(pString, newString, size);
-	delete[] newString;
+	pNewString[OutPos] = '\0';
+	str_copy(pString, pNewString, Size);
+	delete[] pNewString;
 }
 
 int sqlstr::EscapeLike(char *pDst, const char *pSrc, int DstSize)
@@ -40,7 +40,7 @@ int sqlstr::EscapeLike(char *pDst, const char *pSrc, int DstSize)
 	return DstPos;
 }
 
-void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString)
+void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString, int Size)
 {
 	char aBuf[20];
 	int aTimes[7] =
@@ -71,9 +71,9 @@ void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString)
 	for(i = 0; i < 7; i++)
 	{
 		Seconds = aTimes[i];
-		strcpy(aName, aaNames[i]);
+		str_copy(aName, aaNames[i], sizeof(aName));
 
-		Count = floor((float)AgoTime / (float)Seconds);
+		Count = std::floor((float)AgoTime / (float)Seconds);
 		if(Count != 0)
 		{
 			break;
@@ -88,17 +88,17 @@ void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString)
 	{
 		str_format(aBuf, sizeof(aBuf), "%d %ss", Count, aName);
 	}
-	strcat(pAgoString, aBuf);
+	str_append(pAgoString, aBuf, Size);
 
 	if(i + 1 < 7)
 	{
 		// getting second piece now
 		int Seconds2 = aTimes[i + 1];
 		char aName2[6];
-		strcpy(aName2, aaNames[i + 1]);
+		str_copy(aName2, aaNames[i + 1], sizeof(aName2));
 
 		// add second piece if it's greater than 0
-		int Count2 = floor((float)(AgoTime - (Seconds * Count)) / (float)Seconds2);
+		int Count2 = std::floor((float)(AgoTime - (Seconds * Count)) / (float)Seconds2);
 
 		if(Count2 != 0)
 		{
@@ -110,7 +110,7 @@ void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString)
 			{
 				str_format(aBuf, sizeof(aBuf), " and %d %ss", Count2, aName2);
 			}
-			strcat(pAgoString, aBuf);
+			str_append(pAgoString, aBuf, Size);
 		}
 	}
 }
