@@ -5,6 +5,7 @@
 
 #include <engine/map.h>
 #include <engine/shared/protocol.h>
+#include <game/client/ui.h>
 
 #include "kernel.h"
 
@@ -31,7 +32,6 @@ public:
 		int m_FriendState;
 	};
 
-	int m_SortedIndex;
 	int m_ServerIndex;
 
 	int m_Type;
@@ -61,6 +61,8 @@ public:
 	char m_aAddress[NETADDR_MAXSTRSIZE];
 	CClient m_aClients[MAX_CLIENTS];
 	mutable int m_NumFilteredPlayers;
+
+	mutable CUIElement *m_pUIElement;
 };
 
 bool IsVanilla(const CServerInfo *pInfo);
