@@ -7,10 +7,10 @@
 // Format: ESDDDDDD EDDDDDDD EDD... Extended, Data, Sign
 unsigned char *CVariableInt::Pack(unsigned char *pDst, int i)
 {
-	*pDst = (i>>25)&0x40; // set sign bit if i<0
-	i = i^(i>>31); // if(i<0) i = ~i
+	*pDst = (i >> 25) & 0x40; // set sign bit if i<0
+	i = i ^ (i >> 31); // if(i<0) i = ~i
 
-	*pDst |= i&0x3F; // pack 6bit into dst
+	*pDst |= i & 0x3F; // pack 6bit into dst
 	i >>= 6; // discard 6 bits
 	if(i)
 	{
@@ -18,9 +18,9 @@ unsigned char *CVariableInt::Pack(unsigned char *pDst, int i)
 		while(1)
 		{
 			pDst++;
-			*pDst = i&(0x7F); // pack 7bit
+			*pDst = i & (0x7F); // pack 7bit
 			i >>= 7; // discard 7 bits
-			*pDst |= (i!=0)<<7; // set extend bit (may branch)
+			*pDst |= (i != 0) << 7; // set extend bit (may branch)
 			if(!i)
 				break;
 		}
@@ -32,26 +32,30 @@ unsigned char *CVariableInt::Pack(unsigned char *pDst, int i)
 
 const unsigned char *CVariableInt::Unpack(const unsigned char *pSrc, int *pInOut)
 {
-	int Sign = (*pSrc>>6)&1;
-	*pInOut = *pSrc&0x3F;
+	int Sign = (*pSrc >> 6) & 1;
+	*pInOut = *pSrc & 0x3F;
 
 	do
 	{
-		if(!(*pSrc&0x80)) break;
+		if(!(*pSrc & 0x80))
+			break;
 		pSrc++;
-		*pInOut |= (*pSrc&(0x7F))<<(6);
+		*pInOut |= (*pSrc & (0x7F)) << (6);
 
-		if(!(*pSrc&0x80)) break;
+		if(!(*pSrc & 0x80))
+			break;
 		pSrc++;
-		*pInOut |= (*pSrc&(0x7F))<<(6+7);
+		*pInOut |= (*pSrc & (0x7F)) << (6 + 7);
 
-		if(!(*pSrc&0x80)) break;
+		if(!(*pSrc & 0x80))
+			break;
 		pSrc++;
-		*pInOut |= (*pSrc&(0x7F))<<(6+7+7);
+		*pInOut |= (*pSrc & (0x7F)) << (6 + 7 + 7);
 
-		if(!(*pSrc&0x80)) break;
+		if(!(*pSrc & 0x80))
+			break;
 		pSrc++;
-		*pInOut |= (*pSrc&(0x7F))<<(6+7+7+7);
+		*pInOut |= (*pSrc & (0x7F)) << (6 + 7 + 7 + 7);
 	} while(0);
 
 	pSrc++;
@@ -59,7 +63,6 @@ const unsigned char *CVariableInt::Unpack(const unsigned char *pSrc, int *pInOut
 	return pSrc;
 }
 
-
 long CVariableInt::Decompress(const void *pSrc_, int Size, void *pDst_, int DstSize)
 {
 	const unsigned char *pSrc = (unsigned char *)pSrc_;
@@ -73,7 +76,7 @@ long CVariableInt::Decompress(const void *pSrc_, int Size, void *pDst_, int DstS
 		pSrc = CVariableInt::Unpack(pSrc, pDst);
 		pDst++;
 	}
-	return (unsigned char *)pDst-(unsigned char *)pDst_;
+	return (unsigned char *)pDst - (unsigned char *)pDst_;
 }
 
 long CVariableInt::Compress(const void *pSrc_, int Size, void *pDst_, int DstSize)
@@ -90,5 +93,5 @@ long CVariableInt::Compress(const void *pSrc_, int Size, void *pDst_, int DstSiz
 		Size--;
 		pSrc++;
 	}
-	return pDst-(unsigned char *)pDst_;
+	return pDst - (unsigned char *)pDst_;
 }
