@@ -3,13 +3,17 @@
 #ifndef ENGINE_SHARED_CONFIG_H
 #define ENGINE_SHARED_CONFIG_H
 
+#include <base/detect.h>
+#include <engine/config.h>
+
 #define CONFIG_FILE "settings_ddnet.cfg"
 #define AUTOEXEC_FILE "autoexec.cfg"
 #define AUTOEXEC_CLIENT_FILE "autoexec_client.cfg"
 #define AUTOEXEC_SERVER_FILE "autoexec_server.cfg"
 
-struct CConfiguration
+class CConfig
 {
+public:
 #define MACRO_CONFIG_INT(Name, ScriptName, Def, Min, Max, Save, Desc) int m_##Name;
 #define MACRO_CONFIG_COL(Name, ScriptName, Def, Save, Desc) unsigned m_##Name;
 #define MACRO_CONFIG_STR(Name, ScriptName, Len, Def, Save, Desc) char m_##Name[Len]; // Flawfinder: ignore
@@ -19,7 +23,7 @@ struct CConfiguration
 #undef MACRO_CONFIG_STR
 };
 
-extern CConfiguration g_Config;
+extern CConfig g_Config;
 
 enum
 {
@@ -39,4 +43,36 @@ enum
 	CFGFLAG_COLALPHA = 1 << 11,
 };
 
+class CConfigManager : public IConfigManager
+{
+	enum
+	{
+		MAX_CALLBACKS = 16
+	};
+
+	struct CCallback
+	{
+		SAVECALLBACKFUNC m_pfnFunc;
+		void *m_pUserData;
+	};
+
+	class IStorage *m_pStorage;
+	IOHANDLE m_ConfigFile;
+	bool m_Failed;
+	CCallback m_aCallbacks[MAX_CALLBACKS];
+	int m_NumCallbacks;
+
+public:
+	CConfigManager();
+
+	virtual void Init();
+	virtual void Reset();
+	virtual bool Save();
+	virtual CConfig *Values() { return &g_Config; }
+
+	virtual void RegisterCallback(SAVECALLBACKFUNC pfnFunc, void *pUserData);
+
+	virtual void WriteLine(const char *pLine);
+};
+
 #endif
