@@ -39,62 +39,62 @@ float CConsole::CResult::GetFloat(unsigned Index)
 
 ColorHSLA CConsole::CResult::GetColor(unsigned Index, bool Light)
 {
-	ColorHSLA hsl = ColorHSLA(0, 0, 0);
+	ColorHSLA Hsla = ColorHSLA(0, 0, 0);
 	if(Index >= m_NumArgs)
-		return hsl;
+		return Hsla;
 
 	const char *pStr = m_apArgs[Index];
 	if(str_isallnum(pStr) || ((pStr[0] == '-' || pStr[0] == '+') && str_isallnum(pStr + 1))) // Teeworlds Color (Packed HSL)
 	{
-		hsl = ColorHSLA(str_toulong_base(pStr, 10), true);
+		Hsla = ColorHSLA(str_toulong_base(pStr, 10), true);
 		if(Light)
-			hsl = hsl.UnclampLighting();
+			Hsla = Hsla.UnclampLighting();
 	}
 	else if(*pStr == '$') // Hex RGB
 	{
-		ColorRGBA rgb = ColorRGBA(0, 0, 0, 1);
+		ColorRGBA Rgba = ColorRGBA(0, 0, 0, 1);
 		int Len = str_length(pStr);
 		if(Len == 4)
 		{
 			unsigned Num = str_toulong_base(pStr + 1, 16);
-			rgb.r = (((Num >> 8) & 0x0F) + ((Num >> 4) & 0xF0)) / 255.0f;
-			rgb.g = (((Num >> 4) & 0x0F) + ((Num >> 0) & 0xF0)) / 255.0f;
-			rgb.b = (((Num >> 0) & 0x0F) + ((Num << 4) & 0xF0)) / 255.0f;
+			Rgba.r = (((Num >> 8) & 0x0F) + ((Num >> 4) & 0xF0)) / 255.0f;
+			Rgba.g = (((Num >> 4) & 0x0F) + ((Num >> 0) & 0xF0)) / 255.0f;
+			Rgba.b = (((Num >> 0) & 0x0F) + ((Num << 4) & 0xF0)) / 255.0f;
 		}
 		else if(Len == 7)
 		{
 			unsigned Num = str_toulong_base(pStr + 1, 16);
-			rgb.r = ((Num >> 16) & 0xFF) / 255.0f;
-			rgb.g = ((Num >> 8) & 0xFF) / 255.0f;
-			rgb.b = ((Num >> 0) & 0xFF) / 255.0f;
+			Rgba.r = ((Num >> 16) & 0xFF) / 255.0f;
+			Rgba.g = ((Num >> 8) & 0xFF) / 255.0f;
+			Rgba.b = ((Num >> 0) & 0xFF) / 255.0f;
 		}
 		else
 		{
-			return hsl;
+			return Hsla;
 		}
 
-		hsl = color_cast<ColorHSLA>(rgb);
+		Hsla = color_cast<ColorHSLA>(Rgba);
 	}
 	else if(!str_comp_nocase(pStr, "red"))
-		hsl = ColorHSLA(0.0f / 6.0f, 1, .5f);
+		Hsla = ColorHSLA(0.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "yellow"))
-		hsl = ColorHSLA(1.0f / 6.0f, 1, .5f);
+		Hsla = ColorHSLA(1.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "green"))
-		hsl = ColorHSLA(2.0f / 6.0f, 1, .5f);
+		Hsla = ColorHSLA(2.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "cyan"))
-		hsl = ColorHSLA(3.0f / 6.0f, 1, .5f);
+		Hsla = ColorHSLA(3.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "blue"))
-		hsl = ColorHSLA(4.0f / 6.0f, 1, .5f);
+		Hsla = ColorHSLA(4.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "magenta"))
-		hsl = ColorHSLA(5.0f / 6.0f, 1, .5f);
+		Hsla = ColorHSLA(5.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "white"))
-		hsl = ColorHSLA(0, 0, 1);
+		Hsla = ColorHSLA(0, 0, 1);
 	else if(!str_comp_nocase(pStr, "gray"))
-		hsl = ColorHSLA(0, 0, .5f);
+		Hsla = ColorHSLA(0, 0, .5f);
 	else if(!str_comp_nocase(pStr, "black"))
-		hsl = ColorHSLA(0, 0, 0);
+		Hsla = ColorHSLA(0, 0, 0);
 
-	return hsl;
+	return Hsla;
 }
 
 const IConsole::CCommandInfo *CConsole::CCommand::NextCommandInfo(int AccessLevel, int FlagMask) const
@@ -456,7 +456,7 @@ void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientID, bo
 				if(Result.m_pCommand[0] == '+')
 				{
 					// insert the stroke direction token
-					Result.AddArgument(m_paStrokeStr[Stroke]);
+					Result.AddArgument(m_apStrokeStr[Stroke]);
 					IsStrokeCommand = 1;
 				}
 
@@ -571,8 +571,6 @@ void CConsole::ExecuteFile(const char *pFilename, int ClientID, bool LogFailure,
 		if(str_comp(pFilename, pCur->m_pFilename) == 0)
 			return;
 
-	if(!m_pStorage)
-		m_pStorage = Kernel()->RequestInterface<IStorage>();
 	if(!m_pStorage)
 		return;
 
@@ -590,13 +588,13 @@ void CConsole::ExecuteFile(const char *pFilename, int ClientID, bool LogFailure,
 	if(File)
 	{
 		char *pLine;
-		CLineReader lr;
+		CLineReader Reader;
 
 		str_format(aBuf, sizeof(aBuf), "executing '%s'", pFilename);
 		Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-		lr.Init(File);
+		Reader.Init(File);
 
-		while((pLine = lr.Get()))
+		while((pLine = Reader.Get()))
 			ExecuteLine(pLine, ClientID);
 
 		io_close(File);
@@ -772,19 +770,19 @@ static void ColVariableCommand(IConsole::IResult *pResult, void *pUserData)
 		str_format(aBuf, sizeof(aBuf), "Value: %u", *(pData->m_pVariable));
 		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
 
-		ColorHSLA hsl(*(pData->m_pVariable), true);
+		ColorHSLA Hsla(*(pData->m_pVariable), true);
 		if(pData->m_Light)
-			hsl = hsl.UnclampLighting();
-		str_format(aBuf, sizeof(aBuf), "H: %d°, S: %d%%, L: %d%%", round_truncate(hsl.h * 360), round_truncate(hsl.s * 100), round_truncate(hsl.l * 100));
+			Hsla = Hsla.UnclampLighting();
+		str_format(aBuf, sizeof(aBuf), "H: %d°, S: %d%%, L: %d%%", round_truncate(Hsla.h * 360), round_truncate(Hsla.s * 100), round_truncate(Hsla.l * 100));
 		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
 
-		ColorRGBA rgb = color_cast<ColorRGBA>(hsl);
-		str_format(aBuf, sizeof(aBuf), "R: %d, G: %d, B: %d, #%06X", round_truncate(rgb.r * 255), round_truncate(rgb.g * 255), round_truncate(rgb.b * 255), rgb.Pack(false));
+		ColorRGBA Rgba = color_cast<ColorRGBA>(Hsla);
+		str_format(aBuf, sizeof(aBuf), "R: %d, G: %d, B: %d, #%06X", round_truncate(Rgba.r * 255), round_truncate(Rgba.g * 255), round_truncate(Rgba.b * 255), Rgba.Pack(false));
 		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
 
 		if(pData->m_Alpha)
 		{
-			str_format(aBuf, sizeof(aBuf), "A: %d%%", round_truncate(hsl.a * 100));
+			str_format(aBuf, sizeof(aBuf), "A: %d%%", round_truncate(Hsla.a * 100));
 			pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
 		}
 	}
@@ -928,8 +926,8 @@ CConsole::CConsole(int FlagMask)
 	m_pRecycleList = 0;
 	m_TempCommands.Reset();
 	m_StoreCommands = true;
-	m_paStrokeStr[0] = "0";
-	m_paStrokeStr[1] = "1";
+	m_apStrokeStr[0] = "0";
+	m_apStrokeStr[1] = "1";
 	m_ExecutionQueue.Reset();
 	m_pFirstCommand = 0;
 	m_pFirstExec = 0;
@@ -951,6 +949,31 @@ CConsole::CConsole(int FlagMask)
 	Register("access_status", "i[accesslevel]", CFGFLAG_SERVER, ConCommandStatus, this, "List all commands which are accessible for admin = 0, moderator = 1, helper = 2, all = 3");
 	Register("cmdlist", "", CFGFLAG_SERVER | CFGFLAG_CHAT, ConUserCommandStatus, this, "List all commands which are accessible for users");
 
+	// DDRace
+
+	m_Cheated = false;
+}
+
+CConsole::~CConsole()
+{
+	CCommand *pCommand = m_pFirstCommand;
+	while(pCommand)
+	{
+		CCommand *pNext = pCommand->m_pNext;
+		if(pCommand->m_pfnCallback == Con_Chain)
+			delete static_cast<CChain *>(pCommand->m_pUserData);
+		// Temp commands are on m_TempCommands heap, so don't delete them
+		if(!pCommand->m_Temp)
+			delete pCommand;
+		pCommand = pNext;
+	}
+}
+
+void CConsole::Init()
+{
+	m_pConfig = Kernel()->RequestInterface<IConfigManager>()->Values();
+	m_pStorage = Kernel()->RequestInterface<IStorage>();
+
 // TODO: this should disappear
 #define MACRO_CONFIG_INT(Name, ScriptName, Def, Min, Max, Flags, Desc) \
 	{ \
@@ -977,25 +1000,6 @@ CConsole::CConsole(int FlagMask)
 #undef MACRO_CONFIG_INT
 #undef MACRO_CONFIG_COL
 #undef MACRO_CONFIG_STR
-
-	// DDRace
-
-	m_Cheated = false;
-}
-
-CConsole::~CConsole()
-{
-	CCommand *pCommand = m_pFirstCommand;
-	while(pCommand)
-	{
-		CCommand *pNext = pCommand->m_pNext;
-		if(pCommand->m_pfnCallback == Con_Chain)
-			delete static_cast<CChain *>(pCommand->m_pUserData);
-		// Temp commands are on m_TempCommands heap, so don't delete them
-		if(!pCommand->m_Temp)
-			delete pCommand;
-		pCommand = pNext;
-	}
 }
 
 void CConsole::ParseArguments(int NumArgs, const char **ppArguments)
