@@ -8,6 +8,8 @@
 #include <base/hash.h>
 #include <base/system.h>
 
+#include <zlib.h>
+
 // raw datafile access
 class CDataFileReader
 {
@@ -99,8 +101,8 @@ public:
 	~CDataFileWriter();
 	void Init();
 	bool OpenFile(class IStorage *pStorage, const char *pFilename, int StorageType = IStorage::TYPE_SAVE);
-	bool Open(class IStorage *pStorage, const char *Filename, int StorageType = IStorage::TYPE_SAVE);
-	int AddData(int Size, void *pData);
+	bool Open(class IStorage *pStorage, const char *pFilename, int StorageType = IStorage::TYPE_SAVE);
+	int AddData(int Size, void *pData, int CompressionLevel = Z_DEFAULT_COMPRESSION);
 	int AddDataSwapped(int Size, void *pData);
 	int AddItem(int Type, int ID, int Size, void *pData);
 	int Finish();
