@@ -3,12 +3,16 @@
 #ifndef ENGINE_SHARED_FILECOLLECTION_H
 #define ENGINE_SHARED_FILECOLLECTION_H
 
+#include <engine/storage.h>
+
+#include <stddef.h>
+
 class CFileCollection
 {
 	enum
 	{
-		MAX_ENTRIES=1001,
-		TIMESTAMP_LENGTH=20,	// _YYYY-MM-DD_HH-MM-SS
+		MAX_ENTRIES = 1001,
+		TIMESTAMP_LENGTH = 20, // _YYYY-MM-DD_HH-MM-SS
 	};
 
 	int64 m_aTimestamps[MAX_ENTRIES];
