@@ -31,13 +31,13 @@ public:
 	virtual IOHANDLE OpenFile(const char *pFilename, int Flags, int Type, char *pBuffer = 0, int BufferSize = 0) = 0;
 	virtual bool FindFile(const char *pFilename, const char *pPath, int Type, char *pBuffer, int BufferSize) = 0;
 	virtual bool RemoveFile(const char *pFilename, int Type) = 0;
-	virtual bool RenameFile(const char* pOldFilename, const char* pNewFilename, int Type) = 0;
+	virtual bool RenameFile(const char *pOldFilename, const char *pNewFilename, int Type) = 0;
 	virtual bool CreateFolder(const char *pFoldername, int Type) = 0;
 	virtual void GetCompletePath(int Type, const char *pDir, char *pBuffer, unsigned BufferSize) = 0;
 
 	virtual bool RemoveBinaryFile(const char *pFilename) = 0;
-	virtual bool RenameBinaryFile(const char* pOldFilename, const char* pNewFilename) = 0;
-	virtual const char* GetBinaryPath(const char *pDir, char *pBuffer, unsigned BufferSize) = 0;
+	virtual bool RenameBinaryFile(const char *pOldFilename, const char *pNewFilename) = 0;
+	virtual const char *GetBinaryPath(const char *pFilename, char *pBuffer, unsigned BufferSize) = 0;
 
 	static void StripPathAndExtension(const char *pFilename, char *pBuffer, int BufferSize);
 };
@@ -45,5 +45,4 @@ public:
 extern IStorage *CreateStorage(const char *pApplicationName, int StorageType, int NumArgs, const char **ppArguments);
 extern IStorage *CreateLocalStorage();
 
-
 #endif
