@@ -1,23 +1,22 @@
 #include <base/system.h>
 
-#include <engine/shared/config.h>
-#include <engine/map.h>
 #include <engine/graphics.h>
+#include <engine/map.h>
+#include <engine/shared/config.h>
 
 #include <game/client/components/camera.h>
-#include <game/client/components/maplayers.h>
 #include <game/client/components/mapimages.h>
+#include <game/client/components/maplayers.h>
 
 #include "background.h"
 
-CBackground::CBackground() : CMapLayers(CMapLayers::TYPE_BACKGROUND_FORCE)
+CBackground::CBackground(int MapType, bool OnlineOnly) :
+	CMapLayers(MapType, OnlineOnly)
 {
 	m_pLayers = new CLayers;
 	m_pBackgroundLayers = m_pLayers;
 	m_pImages = new CMapImages;
 	m_pBackgroundImages = m_pImages;
-	m_pMap = CreateEngineMap();
-	m_pBackgroundMap = m_pMap;
 	m_Loaded = false;
 	m_aMapName[0] = '\0';
 	m_LastLoad = 0;
@@ -29,17 +28,25 @@ CBackground::~CBackground()
 	delete m_pBackgroundImages;
 }
 
+CBackgroundEngineMap *CBackground::CreateBGMap()
+{
+	return new CBackgroundEngineMap;
+}
+
 void CBackground::OnInit()
 {
+	m_pBackgroundMap = CreateBGMap();
+	m_pMap = m_pBackgroundMap;
+
 	m_pImages->m_pClient = GameClient();
-	Kernel()->ReregisterInterface(m_pMap);
-	if(g_Config.m_ClBackgroundEntities[0] != '\0' && str_comp(g_Config.m_ClBackgroundEntities, CURRENT))
+	Kernel()->RegisterInterface(m_pBackgroundMap);
+	if(g_Config.m_ClBackgroundEntities[0] != '\0' && str_comp(g_Config.m_ClBackgroundEntities, CURRENT_MAP))
 		LoadBackground();
 }
 
 void CBackground::LoadBackground()
 {
-	if(time_get()-m_LastLoad < 10*time_freq())
+	if(time_get() - m_LastLoad < 10 * time_freq())
 		return;
 
 	if(m_Loaded && m_pMap == m_pBackgroundMap)
@@ -50,17 +57,12 @@ void CBackground::LoadBackground()
 	m_pLayers = m_pBackgroundLayers;
 	m_pImages = m_pBackgroundImages;
 
+	bool NeedImageLoading = false;
+
 	str_copy(m_aMapName, g_Config.m_ClBackgroundEntities, sizeof(m_aMapName));
 	char aBuf[128];
 	str_format(aBuf, sizeof(aBuf), "maps/%s", g_Config.m_ClBackgroundEntities);
-	if(m_pMap->Load(aBuf))
-	{
-		m_pLayers->InitBackground(m_pMap);
-		m_pImages->LoadBackground(m_pMap);
-		RenderTools()->RenderTilemapGenerateSkip(m_pLayers);
-		m_Loaded = true;
-	}
-	else if(str_comp(g_Config.m_ClBackgroundEntities, CURRENT) == 0)
+	if(str_comp(g_Config.m_ClBackgroundEntities, CURRENT_MAP) == 0)
 	{
 		m_pMap = Kernel()->RequestInterface<IEngineMap>();
 		if(m_pMap->IsLoaded())
@@ -70,16 +72,27 @@ void CBackground::LoadBackground()
 			m_Loaded = true;
 		}
 	}
-	
-	if(m_Loaded) 
+	else if(m_pMap->Load(aBuf))
+	{
+		m_pLayers->InitBackground(m_pMap);
+		RenderTools()->RenderTilemapGenerateSkip(m_pLayers);
+		NeedImageLoading = true;
+		m_Loaded = true;
+	}
+
+	if(m_Loaded)
+	{
 		CMapLayers::OnMapLoad();
+		if(NeedImageLoading)
+			m_pImages->LoadBackground(m_pLayers, m_pMap);
+	}
 
 	m_LastLoad = time_get();
 }
 
 void CBackground::OnMapLoad()
 {
-	if(str_comp(g_Config.m_ClBackgroundEntities, CURRENT) == 0 || str_comp(g_Config.m_ClBackgroundEntities, m_aMapName))
+	if(str_comp(g_Config.m_ClBackgroundEntities, CURRENT_MAP) == 0 || str_comp(g_Config.m_ClBackgroundEntities, m_aMapName))
 	{
 		m_LastLoad = 0;
 		LoadBackground();
@@ -100,6 +113,6 @@ void CBackground::OnRender()
 
 	if(g_Config.m_ClOverlayEntities != 100)
 		return;
-	
+
 	CMapLayers::OnRender();
 }
