@@ -1,13 +1,20 @@
 #ifndef GAME_CLIENT_COMPONENTS_BACKGROUND_H
 #define GAME_CLIENT_COMPONENTS_BACKGROUND_H
+#include <engine/shared/map.h>
 #include <game/client/component.h>
 #include <game/client/components/maplayers.h>
 
 // Special value to use background of current map
-#define CURRENT "%current%"
+#define CURRENT_MAP "%current%"
+
+class CBackgroundEngineMap : public CMap
+{
+	MACRO_INTERFACE("background_enginemap", 0)
+};
 
 class CBackground : public CMapLayers
 {
+protected:
 	IEngineMap *m_pMap;
 	bool m_Loaded;
 	char m_aMapName[MAX_MAP_LENGTH];
@@ -16,13 +23,15 @@ class CBackground : public CMapLayers
 	int64 m_LastLoad;
 
 	//to avoid memory leak when switching to %current%
-	IEngineMap *m_pBackgroundMap;
+	CBackgroundEngineMap *m_pBackgroundMap;
 	CLayers *m_pBackgroundLayers;
 	CMapImages *m_pBackgroundImages;
 
+	virtual CBackgroundEngineMap *CreateBGMap();
+
 public:
-	CBackground();
-	~CBackground();
+	CBackground(int MapType = CMapLayers::TYPE_BACKGROUND_FORCE, bool OnlineOnly = true);
+	virtual ~CBackground();
 
 	virtual void OnInit();
 	virtual void OnMapLoad();
