@@ -14,6 +14,7 @@
 #include "controls.h"
 
 #include <engine/serverbrowser.h>
+#include <limits>
 
 const float ZoomStep = 0.866025f;
 
@@ -23,6 +24,7 @@ CCamera::CCamera()
 	m_ZoomSet = false;
 	m_Zoom = 1.0f;
 	m_Zooming = false;
+	m_ForceFreeviewPos = vec2(-1, -1);
 }
 
 float CCamera::ZoomProgress(float CurrentTime) const
@@ -38,7 +40,7 @@ void CCamera::ScaleZoom(float Factor)
 
 float CCamera::MaxZoomLevel()
 {
-	return (Graphics()->IsTileBufferingEnabled() ? 60 : 30);
+	return (g_Config.m_ClLimitMaxZoomLevel) ? ((Graphics()->IsTileBufferingEnabled() ? 60 : 30)) : std::numeric_limits<float>::max();
 }
 
 float CCamera::MinZoomLevel()
@@ -124,24 +126,56 @@ void CCamera::OnRender()
 			m_CamType = CAMTYPE_PLAYER;
 		}
 
-		vec2 CameraOffset(0, 0);
+		float DeltaTime = Client()->RenderFrameTime();
+		static vec2 s_LastMousePos(0, 0);
+		static vec2 s_CurrentCameraOffset[2] = {vec2(0, 0), vec2(0, 0)};
+		static float s_SpeedBias = 0.5f;
 
-		float l = length(m_pClient->m_pControls->m_MousePos[g_Config.m_ClDummy]);
+		if(g_Config.m_ClDyncamSmoothness > 0)
+		{
+			float CameraSpeed = (1.0f - (g_Config.m_ClDyncamSmoothness / 100.0f)) * 9.5f + 0.5f;
+			float CameraStabilizingFactor = 1 + g_Config.m_ClDyncamStabilizing / 100.0f;
+
+			s_SpeedBias += CameraSpeed * DeltaTime;
+			if(g_Config.m_ClDyncam)
+			{
+				s_SpeedBias -= length(m_pClient->m_pControls->m_MousePos[g_Config.m_ClDummy] - s_LastMousePos) * log10f(CameraStabilizingFactor) * 0.02f;
+				s_SpeedBias = clamp(s_SpeedBias, 0.5f, CameraSpeed);
+			}
+			else
+			{
+				s_SpeedBias = maximum(5.0f, CameraSpeed); // make sure toggle back is fast
+			}
+		}
+
+		vec2 TargetCameraOffset(0, 0);
+		s_LastMousePos = m_pClient->m_pControls->m_MousePos[g_Config.m_ClDummy];
+		float l = length(s_LastMousePos);
 		if(l > 0.0001f) // make sure that this isn't 0
 		{
 			float DeadZone = g_Config.m_ClDyncam ? g_Config.m_ClDyncamDeadzone : g_Config.m_ClMouseDeadzone;
 			float FollowFactor = (g_Config.m_ClDyncam ? g_Config.m_ClDyncamFollowFactor : g_Config.m_ClMouseFollowfactor) / 100.0f;
 			float OffsetAmount = maximum(l - DeadZone, 0.0f) * FollowFactor;
 
-			CameraOffset = normalize(m_pClient->m_pControls->m_MousePos[g_Config.m_ClDummy]) * OffsetAmount;
+			TargetCameraOffset = normalize(m_pClient->m_pControls->m_MousePos[g_Config.m_ClDummy]) * OffsetAmount;
 		}
 
-		if(m_pClient->m_Snap.m_SpecInfo.m_Active)
-			m_Center = m_pClient->m_Snap.m_SpecInfo.m_Position + CameraOffset;
+		if(g_Config.m_ClDyncamSmoothness > 0)
+			s_CurrentCameraOffset[g_Config.m_ClDummy] += (TargetCameraOffset - s_CurrentCameraOffset[g_Config.m_ClDummy]) * minimum(DeltaTime * s_SpeedBias, 1.0f);
 		else
-			m_Center = m_pClient->m_LocalCharacterPos + CameraOffset;
+			s_CurrentCameraOffset[g_Config.m_ClDummy] = TargetCameraOffset;
+
+		if(m_pClient->m_Snap.m_SpecInfo.m_Active)
+			m_Center = m_pClient->m_Snap.m_SpecInfo.m_Position + s_CurrentCameraOffset[g_Config.m_ClDummy];
+		else
+			m_Center = m_pClient->m_LocalCharacterPos + s_CurrentCameraOffset[g_Config.m_ClDummy];
 	}
 
+	if(m_ForceFreeviewPos != vec2(-1, -1) && m_CamType == CAMTYPE_SPEC)
+	{
+		m_Center = m_pClient->m_pControls->m_MousePos[g_Config.m_ClDummy] = m_ForceFreeviewPos;
+		m_ForceFreeviewPos = vec2(-1, -1);
+	}
 	m_PrevCenter = m_Center;
 }
 
@@ -150,6 +184,7 @@ void CCamera::OnConsoleInit()
 	Console()->Register("zoom+", "", CFGFLAG_CLIENT, ConZoomPlus, this, "Zoom increase");
 	Console()->Register("zoom-", "", CFGFLAG_CLIENT, ConZoomMinus, this, "Zoom decrease");
 	Console()->Register("zoom", "", CFGFLAG_CLIENT, ConZoomReset, this, "Zoom reset");
+	Console()->Register("set_view", "i[x]i[y]", CFGFLAG_CLIENT, ConSetView, this, "Set camera position to x and y in the map");
 }
 
 void CCamera::OnReset()
@@ -178,3 +213,11 @@ void CCamera::ConZoomReset(IConsole::IResult *pResult, void *pUserData)
 {
 	((CCamera *)pUserData)->ChangeZoom(pow(ZoomStep, g_Config.m_ClDefaultZoom - 10));
 }
+void CCamera::ConSetView(IConsole::IResult *pResult, void *pUserData)
+{
+	CCamera *pSelf = (CCamera *)pUserData;
+	// wait until free view camera type to update the position
+	pSelf->m_ForceFreeviewPos = vec2(
+		clamp(pResult->GetInteger(0) * 32.0f, 200.0f, pSelf->Collision()->GetWidth() * 32 - 200.0f),
+		clamp(pResult->GetInteger(1) * 32.0f, 200.0f, pSelf->Collision()->GetWidth() * 32 - 200.0f));
+}
