@@ -18,7 +18,7 @@ class CCamera : public CComponent
 	};
 
 	int m_CamType;
-	vec2 m_LastPos[2];
+	vec2 m_LastPos[NUM_DUMMIES];
 	vec2 m_PrevCenter;
 
 	bool m_Zooming;
@@ -51,6 +51,9 @@ private:
 	static void ConZoomPlus(IConsole::IResult *pResult, void *pUserData);
 	static void ConZoomMinus(IConsole::IResult *pResult, void *pUserData);
 	static void ConZoomReset(IConsole::IResult *pResult, void *pUserData);
+	static void ConSetView(IConsole::IResult *pResult, void *pUserData);
+
+	vec2 m_ForceFreeviewPos;
 };
 
 #endif
