@@ -14,10 +14,12 @@
 #include <game/generated/client_data.h>
 #include <game/generated/protocol.h>
 
+#include <game/client/animstate.h>
 #include <game/client/gameclient.h>
 
 #include <game/client/components/console.h>
 #include <game/client/components/scoreboard.h>
+#include <game/client/components/skins.h>
 #include <game/client/components/sounds.h>
 #include <game/localization.h>
 
@@ -25,14 +27,15 @@
 
 CChat::CChat()
 {
-	for(int i = 0; i < MAX_LINES; i++)
+	for(auto &Line : m_aLines)
 	{
 		// reset the container indices, so the text containers can be deleted on reset
-		m_aLines[i].m_TextContainerIndex = -1;
+		Line.m_TextContainerIndex = -1;
+		Line.m_QuadContainerIndex = -1;
 	}
 
 #define CHAT_COMMAND(name, params, flags, callback, userdata, help) RegisterCommand(name, params, flags, help);
-#include <game/server/ddracechat.h>
+#include <game/ddracechat.h>
 	m_Commands.sort_range();
 
 	m_Mode = MODE_NONE;
@@ -44,31 +47,43 @@ void CChat::RegisterCommand(const char *pName, const char *pParams, int flags, c
 	m_Commands.add_unsorted(CCommand{pName, pParams});
 }
 
+void CChat::RebuildChat()
+{
+	for(auto &Line : m_aLines)
+	{
+		if(Line.m_TextContainerIndex != -1)
+			TextRender()->DeleteTextContainer(Line.m_TextContainerIndex);
+		Line.m_TextContainerIndex = -1;
+		if(Line.m_QuadContainerIndex != -1)
+			Graphics()->DeleteQuadContainer(Line.m_QuadContainerIndex);
+		Line.m_QuadContainerIndex = -1;
+		// recalculate sizes
+		Line.m_YOffset[0] = -1.f;
+		Line.m_YOffset[1] = -1.f;
+	}
+}
+
 void CChat::OnWindowResize()
 {
-	for(int i = 0; i < MAX_LINES; i++)
-	{
-		if(m_aLines[i].m_TextContainerIndex != -1)
-			TextRender()->DeleteTextContainer(m_aLines[i].m_TextContainerIndex);
-		m_aLines[i].m_TextContainerIndex = -1;
-		// recalculate sizes
-		m_aLines[i].m_YOffset[0] = -1.f;
-		m_aLines[i].m_YOffset[1] = -1.f;
-	}
+	RebuildChat();
 }
 
 void CChat::Reset()
 {
-	for(int i = 0; i < MAX_LINES; i++)
+	for(auto &Line : m_aLines)
 	{
-		if(m_aLines[i].m_TextContainerIndex != -1)
-			TextRender()->DeleteTextContainer(m_aLines[i].m_TextContainerIndex);
-		m_aLines[i].m_Time = 0;
-		m_aLines[i].m_aText[0] = 0;
-		m_aLines[i].m_aName[0] = 0;
-		m_aLines[i].m_Friend = false;
-		m_aLines[i].m_TextContainerIndex = -1;
-		m_aLines[i].m_TimesRepeated = 0;
+		if(Line.m_TextContainerIndex != -1)
+			TextRender()->DeleteTextContainer(Line.m_TextContainerIndex);
+		if(Line.m_QuadContainerIndex != -1)
+			Graphics()->DeleteQuadContainer(Line.m_QuadContainerIndex);
+		Line.m_Time = 0;
+		Line.m_aText[0] = 0;
+		Line.m_aName[0] = 0;
+		Line.m_Friend = false;
+		Line.m_TextContainerIndex = -1;
+		Line.m_QuadContainerIndex = -1;
+		Line.m_TimesRepeated = 0;
+		Line.m_HasRenderTee = false;
 	}
 	m_PrevScoreBoardShowed = false;
 	m_PrevShowChat = false;
@@ -87,8 +102,8 @@ void CChat::Reset()
 	m_CurrentLine = 0;
 	DisableMode();
 
-	for(int i = 0; i < CHAT_NUM; ++i)
-		m_aLastSoundPlayed[i] = 0;
+	for(long long &LastSoundPlayed : m_aLastSoundPlayed)
+		LastSoundPlayed = 0;
 }
 
 void CChat::OnRelease()
@@ -136,6 +151,12 @@ void CChat::ConEcho(IConsole::IResult *pResult, void *pUserData)
 	((CChat *)pUserData)->Echo(pResult->GetString(0));
 }
 
+void CChat::ConchainChatOld(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData)
+{
+	pfnCallback(pResult, pCallbackUserData);
+	((CChat *)pUserData)->RebuildChat();
+}
+
 void CChat::Echo(const char *pString)
 {
 	AddLine(-2, 0, pString);
@@ -148,6 +169,7 @@ void CChat::OnConsoleInit()
 	Console()->Register("chat", "s['team'|'all'] ?r[message]", CFGFLAG_CLIENT, ConChat, this, "Enable chat with all/team mode");
 	Console()->Register("+show_chat", "", CFGFLAG_CLIENT, ConShowChat, this, "Show chat");
 	Console()->Register("echo", "r[message]", CFGFLAG_CLIENT, ConEcho, this, "Echo the text in chat window");
+	Console()->Chain("cl_chat_old", ConchainChatOld, this);
 }
 
 bool CChat::OnInput(IInput::CEvent Event)
@@ -178,7 +200,6 @@ bool CChat::OnInput(IInput::CEvent Event)
 			}
 			int max = minimum(i - Begin + 1, (int)sizeof(Line));
 			str_utf8_copy(Line, Text + Begin, max);
-			Begin = i + 1;
 			m_Input.Add(Line);
 		}
 	}
@@ -298,7 +319,7 @@ bool CChat::OnInput(IInput::CEvent Event)
 			for(m_PlaceholderLength = 0; *pCursor && *pCursor != ' '; ++pCursor)
 				++m_PlaceholderLength;
 
-			str_copy(m_aCompletionBuffer, m_Input.GetString() + m_PlaceholderOffset, minimum(static_cast<int>(sizeof(m_aCompletionBuffer)), m_PlaceholderLength + 1));
+			str_truncate(m_aCompletionBuffer, sizeof(m_aCompletionBuffer), m_Input.GetString() + m_PlaceholderOffset, m_PlaceholderLength);
 		}
 
 		if(m_aCompletionBuffer[0] == '/')
@@ -344,7 +365,7 @@ bool CChat::OnInput(IInput::CEvent Event)
 			{
 				char aBuf[256];
 				// add part before the name
-				str_copy(aBuf, m_Input.GetString(), minimum(static_cast<int>(sizeof(aBuf)), m_PlaceholderOffset + 1));
+				str_truncate(aBuf, sizeof(aBuf), m_Input.GetString(), m_PlaceholderOffset);
 
 				// add the command
 				str_append(aBuf, "/", sizeof(aBuf));
@@ -420,7 +441,7 @@ bool CChat::OnInput(IInput::CEvent Event)
 			{
 				char aBuf[256];
 				// add part before the name
-				str_copy(aBuf, m_Input.GetString(), minimum(static_cast<int>(sizeof(aBuf)), m_PlaceholderOffset + 1));
+				str_truncate(aBuf, sizeof(aBuf), m_Input.GetString(), m_PlaceholderOffset);
 
 				// add the name
 				str_append(aBuf, pCompletionString, sizeof(aBuf));
@@ -537,7 +558,7 @@ bool CChat::LineShouldHighlight(const char *pLine, const char *pName)
 	{
 		int Length = str_length(pName);
 
-		if((pLine == pHL || pHL[-1] == ' ') && (pHL[Length] == 0 || pHL[Length] == ' ' || pHL[Length] == '.' || pHL[Length] == '!' || pHL[Length] == ',' || pHL[Length] == '?' || pHL[Length] == ':'))
+		if(Length > 0 && (pLine == pHL || pHL[-1] == ' ') && (pHL[Length] == 0 || pHL[Length] == ' ' || pHL[Length] == '.' || pHL[Length] == '!' || pHL[Length] == ',' || pHL[Length] == '?' || pHL[Length] == ':'))
 			return true;
 	}
 
@@ -563,11 +584,11 @@ void CChat::StoreSave(const char *pText)
 		return;
 
 	char aName[16];
-	str_copy(aName, pStart + 27, minimum(static_cast<size_t>(pMid - pStart - 26), sizeof(aName)));
+	str_truncate(aName, sizeof(aName), pStart + 27, pMid - pStart - 27);
 
 	char aSaveCode[64];
 
-	str_copy(aSaveCode, pMid + 13, minimum(static_cast<size_t>((pOn ? pOn : pEnd) - pMid - 12), sizeof(aSaveCode)));
+	str_truncate(aSaveCode, sizeof(aSaveCode), pMid + 13, (pOn ? pOn : pEnd) - pMid - 13);
 
 	char aTimestamp[20];
 	str_timestamp_format(aTimestamp, sizeof(aTimestamp), FORMAT_SPACE);
@@ -646,6 +667,8 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 	if(*p == 0)
 		return;
 
+	bool IgnoreLine = false;
+
 	while(*p)
 	{
 		Highlighted = false;
@@ -663,17 +686,24 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 		CLine *pCurrentLine = &m_aLines[m_CurrentLine];
 
 		// If it's a client message, m_aText will have ": " prepended so we have to work around it.
-		if(pCurrentLine->m_Team == Team && pCurrentLine->m_ClientID == ClientID &&
-			((ClientID < 0 && str_comp(pCurrentLine->m_aText, pLine) == 0) || (ClientID >= 0 && str_length(pCurrentLine->m_aText) > 2 && str_comp(pCurrentLine->m_aText + 2, pLine) == 0)))
+		if(pCurrentLine->m_Team == Team && pCurrentLine->m_ClientID == ClientID && str_comp(pCurrentLine->m_aText, pLine) == 0)
 		{
 			pCurrentLine->m_TimesRepeated++;
 			if(pCurrentLine->m_TextContainerIndex != -1)
 				TextRender()->DeleteTextContainer(pCurrentLine->m_TextContainerIndex);
 			pCurrentLine->m_TextContainerIndex = -1;
+
+			if(pCurrentLine->m_QuadContainerIndex != -1)
+				Graphics()->DeleteQuadContainer(pCurrentLine->m_QuadContainerIndex);
+			pCurrentLine->m_QuadContainerIndex = -1;
 			pCurrentLine->m_Time = time();
 			pCurrentLine->m_YOffset[0] = -1.f;
 			pCurrentLine->m_YOffset[1] = -1.f;
-			return;
+			// Can't return here because we still want to log the message to console,
+			// even if we ignore it in chat. We will set the new line, fill it out
+			// totally, but then in the end revert back m_CurrentLine after writing
+			// the message to console.
+			IgnoreLine = true;
 		}
 
 		m_CurrentLine = (m_CurrentLine + 1) % MAX_LINES;
@@ -691,10 +721,14 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 			TextRender()->DeleteTextContainer(pCurrentLine->m_TextContainerIndex);
 		pCurrentLine->m_TextContainerIndex = -1;
 
+		if(pCurrentLine->m_QuadContainerIndex != -1)
+			Graphics()->DeleteQuadContainer(pCurrentLine->m_QuadContainerIndex);
+		pCurrentLine->m_QuadContainerIndex = -1;
+
 		// check for highlighted name
 		if(Client()->State() != IClient::STATE_DEMOPLAYBACK)
 		{
-			if(ClientID != m_pClient->m_LocalIDs[0])
+			if(ClientID >= 0 && ClientID != m_pClient->m_LocalIDs[0])
 			{
 				// main character
 				if(LineShouldHighlight(pLine, m_pClient->m_aClients[m_pClient->m_LocalIDs[0]].m_aName))
@@ -752,19 +786,46 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 				Highlighted = true;
 			}
 			else
-				str_copy(pCurrentLine->m_aName, m_pClient->m_aClients[ClientID].m_aName, sizeof(pCurrentLine->m_aName));
+				str_format(pCurrentLine->m_aName, sizeof(pCurrentLine->m_aName), "%s", m_pClient->m_aClients[ClientID].m_aName);
 
-			str_format(pCurrentLine->m_aText, sizeof(pCurrentLine->m_aText), ": %s", pLine);
+			str_format(pCurrentLine->m_aText, sizeof(pCurrentLine->m_aText), "%s", pLine);
 			pCurrentLine->m_Friend = m_pClient->m_aClients[ClientID].m_Friend;
 		}
 
+		pCurrentLine->m_HasRenderTee = false;
+
 		pCurrentLine->m_Friend = ClientID >= 0 ? m_pClient->m_aClients[ClientID].m_Friend : false;
 
+		if(pCurrentLine->m_ClientID >= 0 && pCurrentLine->m_aName[0] != '\0')
+		{
+			if(!g_Config.m_ClChatOld)
+			{
+				pCurrentLine->m_CustomColoredSkin = m_pClient->m_aClients[pCurrentLine->m_ClientID].m_RenderInfo.m_CustomColoredSkin;
+				if(pCurrentLine->m_CustomColoredSkin)
+					pCurrentLine->m_RenderSkin = m_pClient->m_aClients[pCurrentLine->m_ClientID].m_RenderInfo.m_ColorableRenderSkin;
+				else
+					pCurrentLine->m_RenderSkin = m_pClient->m_aClients[pCurrentLine->m_ClientID].m_RenderInfo.m_OriginalRenderSkin;
+
+				str_copy(pCurrentLine->m_aSkinName, m_pClient->m_aClients[pCurrentLine->m_ClientID].m_aSkinName, sizeof(pCurrentLine->m_aSkinName));
+				pCurrentLine->m_ColorBody = m_pClient->m_aClients[pCurrentLine->m_ClientID].m_RenderInfo.m_ColorBody;
+				pCurrentLine->m_ColorFeet = m_pClient->m_aClients[pCurrentLine->m_ClientID].m_RenderInfo.m_ColorFeet;
+
+				pCurrentLine->m_RenderSkinMetrics = m_pClient->m_aClients[pCurrentLine->m_ClientID].m_RenderInfo.m_SkinMetrics;
+				pCurrentLine->m_HasRenderTee = true;
+			}
+		}
+
 		char aBuf[1024];
-		str_format(aBuf, sizeof(aBuf), "%s%s", pCurrentLine->m_aName, pCurrentLine->m_aText);
+		str_format(aBuf, sizeof(aBuf), "%s: %s", pCurrentLine->m_aName, pCurrentLine->m_aText);
 		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, Team >= 2 ? "whisper" : (pCurrentLine->m_Team ? "teamchat" : "chat"), aBuf, Highlighted);
 	}
 
+	if(IgnoreLine)
+	{
+		m_CurrentLine = (m_CurrentLine + MAX_LINES - 1) % MAX_LINES;
+		return;
+	}
+
 	// play sound
 	int64 Now = time();
 	if(ClientID == -1)
@@ -787,7 +848,7 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 		if(Now - m_aLastSoundPlayed[CHAT_HIGHLIGHT] >= time_freq() * 3 / 10)
 		{
 			char aBuf[1024];
-			str_format(aBuf, sizeof(aBuf), "%s%s", m_aLines[m_CurrentLine].m_aName, m_aLines[m_CurrentLine].m_aText);
+			str_format(aBuf, sizeof(aBuf), "%s: %s", m_aLines[m_CurrentLine].m_aName, m_aLines[m_CurrentLine].m_aText);
 			Client()->Notify("DDNet Chat", aBuf);
 			if(g_Config.m_SndHighlight)
 			{
@@ -814,27 +875,69 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 	}
 }
 
+void CChat::RefindSkins()
+{
+	for(int i = 0; i < MAX_LINES; i++)
+	{
+		int r = ((m_CurrentLine - i) + MAX_LINES) % MAX_LINES;
+		if(m_aLines[r].m_TextContainerIndex == -1)
+			continue;
+
+		if(m_aLines[r].m_HasRenderTee)
+		{
+			const CSkin *pSkin = m_pClient->m_pSkins->Get(m_pClient->m_pSkins->Find(m_aLines[r].m_aSkinName));
+			if(m_aLines[r].m_CustomColoredSkin)
+				m_aLines[r].m_RenderSkin = pSkin->m_ColorableSkin;
+			else
+				m_aLines[r].m_RenderSkin = pSkin->m_OriginalSkin;
+
+			m_aLines[r].m_RenderSkinMetrics = pSkin->m_Metrics;
+		}
+	}
+}
+
 void CChat::OnPrepareLines()
 {
 	float x = 5.0f;
 	float y = 300.0f - 28.0f;
-	float FontSize = 6.0f;
+	float FontSize = FONT_SIZE;
 
-	bool ForceRecreate = m_pClient->m_pScoreboard->Active() != m_PrevScoreBoardShowed;
+	float ScreenRatio = Graphics()->ScreenAspect();
+
+	bool IsScoreBoardOpen = m_pClient->m_pScoreboard->Active() && (ScreenRatio > 1.7f); // only assume scoreboard when screen ratio is widescreen(something around 16:9)
+
+	bool ForceRecreate = IsScoreBoardOpen != m_PrevScoreBoardShowed;
 	bool ShowLargeArea = m_Show || g_Config.m_ClShowChat == 2;
+
 	ForceRecreate |= ShowLargeArea != m_PrevShowChat;
-	m_PrevScoreBoardShowed = m_pClient->m_pScoreboard->Active();
+
+	m_PrevScoreBoardShowed = IsScoreBoardOpen;
 	m_PrevShowChat = ShowLargeArea;
 
+	float RealMsgPaddingX = MESSAGE_PADDING_X;
+	float RealMsgPaddingY = MESSAGE_PADDING_Y;
+	float RealMsgPaddingTee = MESSAGE_TEE_SIZE + MESSAGE_TEE_PADDING_RIGHT;
+
+	if(g_Config.m_ClChatOld)
+	{
+		RealMsgPaddingX = 0;
+		RealMsgPaddingY = 0;
+		RealMsgPaddingTee = 0;
+	}
+
 	int64 Now = time();
-	float LineWidth = m_pClient->m_pScoreboard->Active() ? 90.0f : 200.0f;
-	float HeightLimit = m_pClient->m_pScoreboard->Active() ? 230.0f : m_PrevShowChat ? 50.0f : 200.0f;
+	float LineWidth = (IsScoreBoardOpen ? 85.0f : 200.0f) - (RealMsgPaddingX * 1.5f) - RealMsgPaddingTee;
+
+	float HeightLimit = IsScoreBoardOpen ? 180.0f : m_PrevShowChat ? 50.0f : 200.0f;
 	float Begin = x;
+	float TextBegin = Begin + RealMsgPaddingX / 2.0f;
 	CTextCursor Cursor;
-	int OffsetType = m_pClient->m_pScoreboard->Active() ? 1 : 0;
+	int OffsetType = IsScoreBoardOpen ? 1 : 0;
+
 	for(int i = 0; i < MAX_LINES; i++)
 	{
 		int r = ((m_CurrentLine - i) + MAX_LINES) % MAX_LINES;
+
 		if(Now > m_aLines[r].m_Time + 16 * time_freq() && !m_PrevShowChat)
 			break;
 
@@ -846,14 +949,19 @@ void CChat::OnPrepareLines()
 
 		m_aLines[r].m_TextContainerIndex = -1;
 
-		char aName[64] = "";
+		if(m_aLines[r].m_QuadContainerIndex != -1)
+			Graphics()->DeleteQuadContainer(m_aLines[r].m_QuadContainerIndex);
+
+		m_aLines[r].m_QuadContainerIndex = -1;
+
+		char aName[64 + 12] = "";
 
 		if(g_Config.m_ClShowIDs && m_aLines[r].m_ClientID >= 0 && m_aLines[r].m_aName[0] != '\0')
 		{
-			if(m_aLines[r].m_ClientID >= 10)
-				str_format(aName, sizeof(aName), "%d: ", m_aLines[r].m_ClientID);
-			else
+			if(m_aLines[r].m_ClientID < 10)
 				str_format(aName, sizeof(aName), " %d: ", m_aLines[r].m_ClientID);
+			else
+				str_format(aName, sizeof(aName), "%d: ", m_aLines[r].m_ClientID);
 		}
 
 		str_append(aName, m_aLines[r].m_aName, sizeof(aName));
@@ -864,20 +972,49 @@ void CChat::OnPrepareLines()
 		else
 			str_format(aCount, sizeof(aCount), " [%d]", m_aLines[r].m_TimesRepeated + 1);
 
+		if(g_Config.m_ClChatOld)
+		{
+			m_aLines[r].m_HasRenderTee = false;
+		}
+
 		// get the y offset (calculate it if we haven't done that yet)
 		if(m_aLines[r].m_YOffset[OffsetType] < 0.0f)
 		{
-			TextRender()->SetCursor(&Cursor, Begin, 0.0f, FontSize, 0);
+			TextRender()->SetCursor(&Cursor, TextBegin, 0.0f, FontSize, 0);
 			Cursor.m_LineWidth = LineWidth;
 
-			if(g_Config.m_ClMessageFriend)
-				TextRender()->TextEx(&Cursor, "♥ ", -1);
+			if(m_aLines[r].m_ClientID >= 0 && m_aLines[r].m_aName[0] != '\0')
+			{
+				Cursor.m_X += RealMsgPaddingTee;
+
+				if(m_aLines[r].m_Friend && g_Config.m_ClMessageFriend)
+				{
+					TextRender()->TextEx(&Cursor, "♥ ", -1);
+				}
+			}
+
 			TextRender()->TextEx(&Cursor, aName, -1);
 			if(m_aLines[r].m_TimesRepeated > 0)
 				TextRender()->TextEx(&Cursor, aCount, -1);
-			TextRender()->TextEx(&Cursor, m_aLines[r].m_aText, -1);
-			m_aLines[r].m_YOffset[OffsetType] = Cursor.m_Y + Cursor.m_FontSize;
+
+			if(m_aLines[r].m_ClientID >= 0 && m_aLines[r].m_aName[0] != '\0')
+			{
+				TextRender()->TextEx(&Cursor, ": ", -1);
+			}
+
+			CTextCursor AppendCursor = Cursor;
+
+			if(!IsScoreBoardOpen)
+			{
+				AppendCursor.m_StartX = Cursor.m_X;
+				AppendCursor.m_LineWidth -= (Cursor.m_LongestLineWidth - Cursor.m_StartX);
+			}
+
+			TextRender()->TextEx(&AppendCursor, m_aLines[r].m_aText, -1);
+
+			m_aLines[r].m_YOffset[OffsetType] = AppendCursor.m_Y + AppendCursor.m_FontSize + RealMsgPaddingY;
 		}
+
 		y -= m_aLines[r].m_YOffset[OffsetType];
 
 		// cut off if msgs waste too much space
@@ -885,51 +1022,60 @@ void CChat::OnPrepareLines()
 			break;
 
 		// the position the text was created
-		m_aLines[r].m_TextYOffset = y;
+		m_aLines[r].m_TextYOffset = y + RealMsgPaddingY / 2.f;
+
+		int CurRenderFlags = TextRender()->GetRenderFlags();
+		TextRender()->SetRenderFlags(CurRenderFlags | ETextRenderFlags::TEXT_RENDER_FLAG_NO_AUTOMATIC_QUAD_UPLOAD);
 
 		// reset the cursor
-		TextRender()->SetCursor(&Cursor, Begin, y, FontSize, TEXTFLAG_RENDER);
+		TextRender()->SetCursor(&Cursor, TextBegin, m_aLines[r].m_TextYOffset, FontSize, TEXTFLAG_RENDER);
 		Cursor.m_LineWidth = LineWidth;
 
-		if(g_Config.m_ClMessageFriend)
+		// Message is from valid player
+		if(m_aLines[r].m_ClientID >= 0 && m_aLines[r].m_aName[0] != '\0')
 		{
-			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageFriendColor));
-			TextRender()->TextColor(rgb.WithAlpha(m_aLines[r].m_Friend ? 1.f : 0.f)); //Less ugly hack to align messages
-			if(m_aLines[r].m_TextContainerIndex == -1)
-				m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&Cursor, "♥ ");
-			else
-				TextRender()->AppendTextContainer(&Cursor, m_aLines[r].m_TextContainerIndex, "♥ ");
+			Cursor.m_X += RealMsgPaddingTee;
+
+			if(m_aLines[r].m_Friend && g_Config.m_ClMessageFriend)
+			{
+				const char *pHeartStr = "♥ ";
+				ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageFriendColor));
+				TextRender()->TextColor(rgb.WithAlpha(1.f));
+				if(m_aLines[r].m_TextContainerIndex == -1)
+					m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&Cursor, pHeartStr);
+				else
+					TextRender()->AppendTextContainer(&Cursor, m_aLines[r].m_TextContainerIndex, pHeartStr);
+			}
 		}
 
 		// render name
+		ColorRGBA NameColor;
 		if(m_aLines[r].m_ClientID == -1) // system
 		{
-			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageSystemColor));
-			TextRender()->TextColor(rgb);
+			NameColor = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageSystemColor));
 		}
 		else if(m_aLines[r].m_ClientID == -2) // client
 		{
-			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageClientColor));
-			TextRender()->TextColor(rgb);
+			NameColor = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageClientColor));
 		}
 		else if(m_aLines[r].m_Team)
 		{
-			ColorRGBA rgb = CalculateNameColor(ColorHSLA(g_Config.m_ClMessageTeamColor));
-			TextRender()->TextColor(rgb); // team message
+			NameColor = CalculateNameColor(ColorHSLA(g_Config.m_ClMessageTeamColor));
 		}
 		else if(m_aLines[r].m_NameColor == TEAM_RED)
-			TextRender()->TextColor(1.0f, 0.5f, 0.5f, 1.f); // red
+			NameColor = ColorRGBA(1.0f, 0.5f, 0.5f, 1.f); // red
 		else if(m_aLines[r].m_NameColor == TEAM_BLUE)
-			TextRender()->TextColor(0.7f, 0.7f, 1.0f, 1.f); // blue
+			NameColor = ColorRGBA(0.7f, 0.7f, 1.0f, 1.f); // blue
 		else if(m_aLines[r].m_NameColor == TEAM_SPECTATORS)
-			TextRender()->TextColor(0.75f, 0.5f, 0.75f, 1.f); // spectator
+			NameColor = ColorRGBA(0.75f, 0.5f, 0.75f, 1.f); // spectator
 		else if(m_aLines[r].m_ClientID >= 0 && g_Config.m_ClChatTeamColors && m_pClient->m_Teams.Team(m_aLines[r].m_ClientID))
 		{
-			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(m_pClient->m_Teams.Team(m_aLines[r].m_ClientID) / 64.0f, 1.0f, 0.75f));
-			TextRender()->TextColor(rgb);
+			NameColor = color_cast<ColorRGBA>(ColorHSLA(m_pClient->m_Teams.Team(m_aLines[r].m_ClientID) / 64.0f, 1.0f, 0.75f));
 		}
 		else
-			TextRender()->TextColor(0.8f, 0.8f, 0.8f, 1.f);
+			NameColor = ColorRGBA(0.8f, 0.8f, 0.8f, 1.f);
+
+		TextRender()->TextColor(NameColor);
 
 		if(m_aLines[r].m_TextContainerIndex == -1)
 			m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&Cursor, aName);
@@ -945,6 +1091,15 @@ void CChat::OnPrepareLines()
 				TextRender()->AppendTextContainer(&Cursor, m_aLines[r].m_TextContainerIndex, aCount);
 		}
 
+		if(m_aLines[r].m_ClientID >= 0 && m_aLines[r].m_aName[0] != '\0')
+		{
+			TextRender()->TextColor(NameColor);
+			if(m_aLines[r].m_TextContainerIndex == -1)
+				m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&Cursor, ": ");
+			else
+				TextRender()->AppendTextContainer(&Cursor, m_aLines[r].m_TextContainerIndex, ": ");
+		}
+
 		// render line
 		ColorRGBA Color;
 		if(m_aLines[r].m_ClientID == -1) // system
@@ -960,10 +1115,28 @@ void CChat::OnPrepareLines()
 
 		TextRender()->TextColor(Color);
 
+		CTextCursor AppendCursor = Cursor;
+		if(!IsScoreBoardOpen)
+		{
+			AppendCursor.m_LineWidth -= (Cursor.m_LongestLineWidth - Cursor.m_StartX);
+			AppendCursor.m_StartX = Cursor.m_X;
+		}
+
 		if(m_aLines[r].m_TextContainerIndex == -1)
-			m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&Cursor, m_aLines[r].m_aText);
+			m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&AppendCursor, m_aLines[r].m_aText);
 		else
-			TextRender()->AppendTextContainer(&Cursor, m_aLines[r].m_TextContainerIndex, m_aLines[r].m_aText);
+			TextRender()->AppendTextContainer(&AppendCursor, m_aLines[r].m_TextContainerIndex, m_aLines[r].m_aText);
+
+		if(!g_Config.m_ClChatOld && (m_aLines[r].m_aText[0] != '\0' || m_aLines[r].m_aName[0] != '\0'))
+		{
+			float Height = m_aLines[r].m_YOffset[OffsetType];
+			Graphics()->SetColor(1, 1, 1, 1);
+			m_aLines[r].m_QuadContainerIndex = RenderTools()->CreateRoundRectQuadContainer(Begin, y, (AppendCursor.m_LongestLineWidth - TextBegin) + RealMsgPaddingX * 1.5f, Height, MESSAGE_ROUNDING, CUI::CORNER_ALL);
+		}
+
+		TextRender()->SetRenderFlags(CurRenderFlags);
+		if(m_aLines[r].m_TextContainerIndex != -1)
+			TextRender()->UploadTextContainer(m_aLines[r].m_TextContainerIndex);
 	}
 
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
@@ -1012,9 +1185,9 @@ void CChat::OnRender()
 		int EditingCursor = Input()->GetEditingCursor();
 		if(Input()->GetIMEState())
 		{
-			if(str_length(Input()->GetIMECandidate()))
+			if(str_length(Input()->GetIMEEditingText()))
 			{
-				m_Input.Editing(Input()->GetIMECandidate(), EditingCursor);
+				m_Input.Editing(Input()->GetIMEEditingText(), EditingCursor);
 				Editing = true;
 			}
 		}
@@ -1066,9 +1239,22 @@ void CChat::OnRender()
 
 	OnPrepareLines();
 
+	float ScreenRatio = Graphics()->ScreenAspect();
+	bool IsScoreBoardOpen = m_pClient->m_pScoreboard->Active() && (ScreenRatio > 1.7f); // only assume scoreboard when screen ratio is widescreen(something around 16:9)
+
 	int64 Now = time();
-	float HeightLimit = m_pClient->m_pScoreboard->Active() ? 230.0f : m_PrevShowChat ? 50.0f : 200.0f;
-	int OffsetType = m_pClient->m_pScoreboard->Active() ? 1 : 0;
+	float HeightLimit = IsScoreBoardOpen ? 180.0f : m_PrevShowChat ? 50.0f : 200.0f;
+	int OffsetType = IsScoreBoardOpen ? 1 : 0;
+
+	float RealMsgPaddingX = MESSAGE_PADDING_X;
+	float RealMsgPaddingY = MESSAGE_PADDING_Y;
+
+	if(g_Config.m_ClChatOld)
+	{
+		RealMsgPaddingX = 0;
+		RealMsgPaddingY = 0;
+	}
+
 	for(int i = 0; i < MAX_LINES; i++)
 	{
 		int r = ((m_CurrentLine - i) + MAX_LINES) % MAX_LINES;
@@ -1083,11 +1269,47 @@ void CChat::OnRender()
 
 		float Blend = Now > m_aLines[r].m_Time + 14 * time_freq() && !m_PrevShowChat ? 1.0f - (Now - m_aLines[r].m_Time - 14 * time_freq()) / (2.0f * time_freq()) : 1.0f;
 
+		// Draw backgrounds for messages in one batch
+		if(!g_Config.m_ClChatOld)
+		{
+			Graphics()->TextureClear();
+			if(m_aLines[r].m_QuadContainerIndex != -1)
+			{
+				Graphics()->SetColor(0, 0, 0, 0.12f * Blend);
+				Graphics()->RenderQuadContainerEx(m_aLines[r].m_QuadContainerIndex, 0, -1, 0, ((y + RealMsgPaddingY / 2.0f) - m_aLines[r].m_TextYOffset));
+			}
+		}
+
 		if(m_aLines[r].m_TextContainerIndex != -1)
 		{
+			if(!g_Config.m_ClChatOld && m_aLines[r].m_HasRenderTee)
+			{
+				CTeeRenderInfo RenderInfo;
+				RenderInfo.m_CustomColoredSkin = m_aLines[r].m_CustomColoredSkin;
+				if(m_aLines[r].m_CustomColoredSkin)
+					RenderInfo.m_ColorableRenderSkin = m_aLines[r].m_RenderSkin;
+				else
+					RenderInfo.m_OriginalRenderSkin = m_aLines[r].m_RenderSkin;
+				RenderInfo.m_SkinMetrics = m_aLines[r].m_RenderSkinMetrics;
+
+				RenderInfo.m_ColorBody = m_aLines[r].m_ColorBody;
+				RenderInfo.m_ColorFeet = m_aLines[r].m_ColorFeet;
+				RenderInfo.m_Size = MESSAGE_TEE_SIZE;
+
+				float RowHeight = FONT_SIZE + RealMsgPaddingY;
+				float OffsetTeeY = MESSAGE_TEE_SIZE / 2.0f;
+				float FullHeightMinusTee = RowHeight - MESSAGE_TEE_SIZE;
+
+				CAnimState *pIdleState = CAnimState::GetIdle();
+				vec2 OffsetToMid;
+				RenderTools()->GetRenderTeeOffsetToRenderedTee(pIdleState, &RenderInfo, OffsetToMid);
+				vec2 TeeRenderPos(x + (RealMsgPaddingX + MESSAGE_TEE_SIZE) / 2.0f, y + OffsetTeeY + FullHeightMinusTee / 2.0f + OffsetToMid.y);
+				RenderTools()->RenderTee(pIdleState, &RenderInfo, EMOTE_NORMAL, vec2(1, 0.1f), TeeRenderPos, Blend);
+			}
+
 			STextRenderColor TextOutline(0.f, 0.f, 0.f, 0.3f * Blend);
 			STextRenderColor Text(1.f, 1.f, 1.f, Blend);
-			TextRender()->RenderTextContainer(m_aLines[r].m_TextContainerIndex, &Text, &TextOutline, 0, y - m_aLines[r].m_TextYOffset);
+			TextRender()->RenderTextContainer(m_aLines[r].m_TextContainerIndex, &Text, &TextOutline, 0, (y + RealMsgPaddingY / 2.0f) - m_aLines[r].m_TextYOffset);
 		}
 	}
 }
