@@ -10,6 +10,7 @@
 #include <engine/shared/config.h>
 
 #include <game/client/component.h>
+#include <game/client/components/camera.h>
 #include <game/client/components/chat.h>
 #include <game/client/components/menus.h>
 #include <game/client/components/scoreboard.h>
@@ -71,8 +72,8 @@ void CControls::OnReset()
 	m_JoystickFirePressed = false;
 	m_JoystickRunPressed = false;
 	m_JoystickTapTime = 0;
-	for(int i = 0; i < NUM_WEAPONS; i++)
-		m_AmmoCount[i] = 0;
+	for(int &AmmoCount : m_AmmoCount)
+		AmmoCount = 0;
 	m_OldMouseX = m_OldMouseY = 0.0f;
 }
 
@@ -100,8 +101,8 @@ void CControls::OnPlayerDeath()
 {
 	if(g_Config.m_ClResetWantedWeaponOnDeath)
 		m_LastData[g_Config.m_ClDummy].m_WantedWeapon = m_InputData[g_Config.m_ClDummy].m_WantedWeapon = 0;
-	for(int i = 0; i < NUM_WEAPONS; i++)
-		m_AmmoCount[i] = 0;
+	for(int &AmmoCount : m_AmmoCount)
+		AmmoCount = 0;
 	m_JoystickTapTime = 0; // Do not launch hook on first tap
 }
 
@@ -558,6 +559,13 @@ bool CControls::OnMouseMove(float x, float y)
 		x = x * g_Config.m_ClDyncamMousesens / g_Config.m_InpMousesens;
 		y = y * g_Config.m_ClDyncamMousesens / g_Config.m_InpMousesens;
 	}
+
+	if(m_pClient->m_Snap.m_SpecInfo.m_Active && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID < 0)
+	{
+		x = x * m_pClient->m_pCamera->m_Zoom;
+		y = y * m_pClient->m_pCamera->m_Zoom;
+	}
+
 	m_MousePos[g_Config.m_ClDummy] += vec2(x, y); // TODO: ugly
 	ClampMousePos();
 
