@@ -58,7 +58,8 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 		// load the graphic file
 		char aBuf[128];
 		CImageInfo Info;
-		if(g_Config.m_ClLoadCountryFlags)
+		bool LoadCountryFlags = g_Config.m_ClLoadCountryFlags;
+		if(LoadCountryFlags)
 		{
 			str_format(aBuf, sizeof(aBuf), "countryflags/%s.png", aOrigin);
 			if(!Graphics()->LoadPNG(&Info, aBuf, IStorage::TYPE_ALL))
@@ -74,10 +75,10 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 		CCountryFlag CountryFlag;
 		CountryFlag.m_CountryCode = CountryCode;
 		str_copy(CountryFlag.m_aCountryCodeString, aOrigin, sizeof(CountryFlag.m_aCountryCodeString));
-		if(g_Config.m_ClLoadCountryFlags)
+		if(LoadCountryFlags)
 		{
 			CountryFlag.m_Texture = Graphics()->LoadTextureRaw(Info.m_Width, Info.m_Height, Info.m_Format, Info.m_pData, Info.m_Format, 0);
-			free(Info.m_pData);
+			Graphics()->FreePNG(&Info);
 		}
 
 		if(g_Config.m_Debug)
@@ -101,8 +102,8 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 
 	// init LUT
 	if(DefaultIndex != 0)
-		for(int i = 0; i < CODE_RANGE; ++i)
-			m_CodeIndexLUT[i] = DefaultIndex;
+		for(int &CodeIndexLUT : m_CodeIndexLUT)
+			CodeIndexLUT = DefaultIndex;
 	else
 		mem_zero(m_CodeIndexLUT, sizeof(m_CodeIndexLUT));
 	for(int i = 0; i < m_aCountryFlags.size(); ++i)
