@@ -167,12 +167,10 @@ void CEffects::PlayerDeath(vec2 Pos, int ClientID)
 	if(ClientID >= 0)
 	{
 		if(m_pClient->m_aClients[ClientID].m_UseCustomColor)
-			BloodColor = color_cast<ColorRGBA>(ColorHSLA(m_pClient->m_aClients[ClientID].m_ColorBody).UnclampLighting(0.5f));
+			BloodColor = m_pClient->m_aClients[ClientID].m_RenderInfo.m_ColorBody;
 		else
 		{
-			const CSkins::CSkin *s = m_pClient->m_pSkins->Get(m_pClient->m_aClients[ClientID].m_SkinID);
-			if(s)
-				BloodColor = s->m_BloodColor;
+			BloodColor = m_pClient->m_aClients[ClientID].m_RenderInfo.m_BloodColor;
 		}
 	}
 
