@@ -4,6 +4,7 @@
 #include <engine/graphics.h>
 #include <engine/shared/config.h>
 #include <game/generated/client_data.h>
+#include <game/generated/client_data7.h>
 #include <game/generated/protocol.h>
 
 #include <game/client/gameclient.h>
@@ -23,20 +24,22 @@ void CItems::OnReset()
 
 void CItems::RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID)
 {
+	int CurWeapon = clamp(pCurrent->m_Type, 0, NUM_WEAPONS - 1);
+
 	// get positions
 	float Curvature = 0;
 	float Speed = 0;
-	if(pCurrent->m_Type == WEAPON_GRENADE)
+	if(CurWeapon == WEAPON_GRENADE)
 	{
 		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GrenadeCurvature;
 		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GrenadeSpeed;
 	}
-	else if(pCurrent->m_Type == WEAPON_SHOTGUN)
+	else if(CurWeapon == WEAPON_SHOTGUN)
 	{
 		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_ShotgunCurvature;
 		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_ShotgunSpeed;
 	}
-	else if(pCurrent->m_Type == WEAPON_GUN)
+	else if(CurWeapon == WEAPON_GUN)
 	{
 		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GunCurvature;
 		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GunSpeed;
@@ -76,16 +79,11 @@ void CItems::RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID)
 			Alpha = g_Config.m_ClShowOthersAlpha / 100.0f;
 	}
 
-	Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
-	Graphics()->SetColor(1.f, 1.f, 1.f, Alpha);
-
-	int QuadOffset = 2 + 4 + NUM_WEAPONS + clamp(pCurrent->m_Type, 0, NUM_WEAPONS - 1);
-
 	vec2 Vel = Pos - PrevPos;
-	//vec2 pos = mix(vec2(prev->x, prev->y), vec2(current->x, current->y), Client()->IntraGameTick(g_Config.m_ClDummy));
 
 	// add particle for this projectile
-	if(pCurrent->m_Type == WEAPON_GRENADE)
+	// don't check for validity of the projectile for the current weapon here, so particle effects are rendered for mod compability
+	if(CurWeapon == WEAPON_GRENADE)
 	{
 		m_pClient->m_pEffects->SmokeTrail(Pos, Vel * -1, Alpha);
 		static float s_Time = 0.0f;
@@ -111,17 +109,42 @@ void CItems::RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID)
 		m_pClient->m_pEffects->BulletTrail(Pos, Alpha);
 
 		if(length(Vel) > 0.00001f)
-			Graphics()->QuadsSetRotation(GetAngle(Vel));
+			Graphics()->QuadsSetRotation(angle(Vel));
 		else
 			Graphics()->QuadsSetRotation(0);
 	}
 
-	Graphics()->RenderQuadContainerAsSprite(m_ItemsQuadContainerIndex, QuadOffset, Pos.x, Pos.y);
+	if(GameClient()->m_GameSkin.m_SpriteWeaponProjectiles[CurWeapon] != -1)
+	{
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpriteWeaponProjectiles[CurWeapon]);
+		Graphics()->SetColor(1.f, 1.f, 1.f, Alpha);
+
+		int QuadOffset = 2 + 8 + NUM_WEAPONS + CurWeapon;
+
+		Graphics()->RenderQuadContainerAsSprite(m_ItemsQuadContainerIndex, QuadOffset, Pos.x, Pos.y);
+	}
 }
 
 void CItems::RenderPickup(const CNetObj_Pickup *pPrev, const CNetObj_Pickup *pCurrent, bool IsPredicted)
 {
-	Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
+	const int c[] = {
+		SPRITE_PICKUP_HEALTH,
+		SPRITE_PICKUP_ARMOR,
+		SPRITE_PICKUP_WEAPON,
+		SPRITE_PICKUP_NINJA};
+
+	int CurWeapon = clamp(pCurrent->m_Subtype, 0, NUM_WEAPONS - 1);
+
+	if(c[pCurrent->m_Type] == SPRITE_PICKUP_HEALTH)
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpritePickupHealth);
+	else if(c[pCurrent->m_Type] == SPRITE_PICKUP_ARMOR)
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpritePickupArmor);
+	else if(c[pCurrent->m_Type] == SPRITE_PICKUP_WEAPON)
+	{
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpritePickupWeapons[CurWeapon]);
+	}
+	else if(c[pCurrent->m_Type] == SPRITE_PICKUP_NINJA)
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpritePickupNinja);
 	Graphics()->QuadsSetRotation(0);
 	Graphics()->SetColor(1.f, 1.f, 1.f, 1.f);
 
@@ -133,19 +156,15 @@ void CItems::RenderPickup(const CNetObj_Pickup *pPrev, const CNetObj_Pickup *pCu
 	if(pCurrent->m_Type == POWERUP_WEAPON)
 	{
 		Angle = 0; //-pi/6;//-0.25f * pi * 2.0f;
-		QuadOffset += 4 + clamp(pCurrent->m_Subtype, 0, NUM_WEAPONS - 1);
+		QuadOffset += 2 + CurWeapon;
 	}
 	else
 	{
-		const int c[] = {
-			SPRITE_PICKUP_HEALTH,
-			SPRITE_PICKUP_ARMOR,
-			SPRITE_PICKUP_WEAPON,
-			SPRITE_PICKUP_NINJA};
 		QuadOffset += pCurrent->m_Type;
 
 		if(c[pCurrent->m_Type] == SPRITE_PICKUP_NINJA)
 		{
+			QuadOffset = 2 + 8 - 1; // ninja is the last weapon
 			m_pClient->m_pEffects->PowerupShine(Pos, vec2(96, 18));
 			Pos.x -= 10.0f;
 		}
@@ -179,7 +198,10 @@ void CItems::RenderFlag(const CNetObj_Flag *pPrev, const CNetObj_Flag *pCurrent,
 	float Angle = 0.0f;
 	float Size = 42.0f;
 
-	Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
+	if(pCurrent->m_Team == TEAM_RED)
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpriteFlagRed);
+	else
+		Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpriteFlagBlue);
 	Graphics()->QuadsSetRotation(0);
 	Graphics()->SetColor(1.f, 1.f, 1.f, 1.f);
 	int QuadOffset = 0;
@@ -213,56 +235,63 @@ void CItems::RenderLaser(const struct CNetObj_Laser *pCurrent, bool IsPredicted)
 	ColorRGBA RGB;
 	vec2 Pos = vec2(pCurrent->m_X, pCurrent->m_Y);
 	vec2 From = vec2(pCurrent->m_FromX, pCurrent->m_FromY);
-	vec2 Dir = normalize(Pos - From);
-
-	float Ticks;
-	if(IsPredicted)
-		Ticks = (float)(Client()->PredGameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) + Client()->PredIntraGameTick(g_Config.m_ClDummy);
-	else
-		Ticks = (float)(Client()->GameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) + Client()->IntraGameTick(g_Config.m_ClDummy);
-	float Ms = (Ticks / 50.0f) * 1000.0f;
-	float a = Ms / m_pClient->m_Tuning[g_Config.m_ClDummy].m_LaserBounceDelay;
-	a = clamp(a, 0.0f, 1.0f);
-	float Ia = 1 - a;
-
-	vec2 Out, Border;
-
-	Graphics()->TextureClear();
-	Graphics()->QuadsBegin();
-
-	// do outline
+	float Len = distance(Pos, From);
 	RGB = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClLaserOutlineColor));
 	ColorRGBA OuterColor(RGB.r, RGB.g, RGB.b, 1.0f);
-	Graphics()->SetColor(OuterColor.r, OuterColor.g, OuterColor.b, 1.0f);
-	Out = vec2(Dir.y, -Dir.x) * (7.0f * Ia);
-
-	IGraphics::CFreeformItem Freeform(
-		From.x - Out.x, From.y - Out.y,
-		From.x + Out.x, From.y + Out.y,
-		Pos.x - Out.x, Pos.y - Out.y,
-		Pos.x + Out.x, Pos.y + Out.y);
-	Graphics()->QuadsDrawFreeform(&Freeform, 1);
-
-	// do inner
 	RGB = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClLaserInnerColor));
 	ColorRGBA InnerColor(RGB.r, RGB.g, RGB.b, 1.0f);
-	Out = vec2(Dir.y, -Dir.x) * (5.0f * Ia);
-	Graphics()->SetColor(InnerColor.r, InnerColor.g, InnerColor.b, 1.0f); // center
 
-	Freeform = IGraphics::CFreeformItem(
-		From.x - Out.x, From.y - Out.y,
-		From.x + Out.x, From.y + Out.y,
-		Pos.x - Out.x, Pos.y - Out.y,
-		Pos.x + Out.x, Pos.y + Out.y);
-	Graphics()->QuadsDrawFreeform(&Freeform, 1);
+	vec2 Dir;
+	if(Len > 0)
+	{
+		Dir = normalize_pre_length(Pos - From, Len);
 
-	Graphics()->QuadsEnd();
+		float Ticks;
+		if(IsPredicted)
+			Ticks = (float)(Client()->PredGameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) + Client()->PredIntraGameTick(g_Config.m_ClDummy);
+		else
+			Ticks = (float)(Client()->GameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) + Client()->IntraGameTick(g_Config.m_ClDummy);
+		float Ms = (Ticks / 50.0f) * 1000.0f;
+		float a = Ms / m_pClient->m_Tuning[g_Config.m_ClDummy].m_LaserBounceDelay;
+		a = clamp(a, 0.0f, 1.0f);
+		float Ia = 1 - a;
+
+		vec2 Out;
+
+		Graphics()->TextureClear();
+		Graphics()->QuadsBegin();
+
+		// do outline
+		Graphics()->SetColor(OuterColor.r, OuterColor.g, OuterColor.b, 1.0f);
+		Out = vec2(Dir.y, -Dir.x) * (7.0f * Ia);
+
+		IGraphics::CFreeformItem Freeform(
+			From.x - Out.x, From.y - Out.y,
+			From.x + Out.x, From.y + Out.y,
+			Pos.x - Out.x, Pos.y - Out.y,
+			Pos.x + Out.x, Pos.y + Out.y);
+		Graphics()->QuadsDrawFreeform(&Freeform, 1);
+
+		// do inner
+		Out = vec2(Dir.y, -Dir.x) * (5.0f * Ia);
+		Graphics()->SetColor(InnerColor.r, InnerColor.g, InnerColor.b, 1.0f); // center
+
+		Freeform = IGraphics::CFreeformItem(
+			From.x - Out.x, From.y - Out.y,
+			From.x + Out.x, From.y + Out.y,
+			Pos.x - Out.x, Pos.y - Out.y,
+			Pos.x + Out.x, Pos.y + Out.y);
+		Graphics()->QuadsDrawFreeform(&Freeform, 1);
+
+		Graphics()->QuadsEnd();
+	}
 
 	// render head
 	{
-		int QuadOffset = 2 + 4 + NUM_WEAPONS * 2 + (Client()->GameTick(g_Config.m_ClDummy) % 3);
+		int CurParticle = (Client()->GameTick(g_Config.m_ClDummy) % 3);
+		int QuadOffset = 2 + 8 + NUM_WEAPONS * 2 + CurParticle;
 
-		Graphics()->TextureSet(g_pData->m_aImages[IMAGE_PARTICLES].m_Id);
+		Graphics()->TextureSet(GameClient()->m_ParticlesSkin.m_SpriteParticleSplat[CurParticle]);
 		Graphics()->QuadsSetRotation(Client()->GameTick(g_Config.m_ClDummy));
 		Graphics()->SetColor(OuterColor.r, OuterColor.g, OuterColor.b, 1.0f);
 		Graphics()->RenderQuadContainerAsSprite(m_ItemsQuadContainerIndex, QuadOffset, Pos.x, Pos.y);
@@ -394,38 +423,56 @@ void CItems::OnInit()
 
 	m_ItemsQuadContainerIndex = Graphics()->CreateQuadContainer();
 
-	RenderTools()->SelectSprite(SPRITE_FLAG_RED);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
 	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, -21.f, -42.f, 42.f, 84.f);
-	RenderTools()->SelectSprite(SPRITE_FLAG_BLUE);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
 	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, -21.f, -42.f, 42.f, 84.f);
 
-	RenderTools()->SelectSprite(SPRITE_PICKUP_HEALTH);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 64.f);
-	RenderTools()->SelectSprite(SPRITE_PICKUP_ARMOR);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 64.f);
-	RenderTools()->SelectSprite(SPRITE_PICKUP_WEAPON);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 64.f);
-	RenderTools()->SelectSprite(SPRITE_PICKUP_NINJA);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 128.f);
+	float ScaleX, ScaleY;
+	RenderTools()->GetSpriteScale(SPRITE_PICKUP_HEALTH, ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 64.f * ScaleX, 64.f * ScaleY);
+	RenderTools()->GetSpriteScale(SPRITE_PICKUP_ARMOR, ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 64.f * ScaleX, 64.f * ScaleY);
+	RenderTools()->GetSpriteScale(&client_data7::g_pData->m_aSprites[client_data7::SPRITE_PICKUP_HAMMER], ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, client_data7::g_pData->m_Weapons.m_aId[WEAPON_HAMMER].m_VisualSize * ScaleX, client_data7::g_pData->m_Weapons.m_aId[WEAPON_HAMMER].m_VisualSize * ScaleY);
+	RenderTools()->GetSpriteScale(&client_data7::g_pData->m_aSprites[client_data7::SPRITE_PICKUP_GUN], ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, client_data7::g_pData->m_Weapons.m_aId[WEAPON_GUN].m_VisualSize * ScaleX, client_data7::g_pData->m_Weapons.m_aId[WEAPON_GUN].m_VisualSize * ScaleY);
+	RenderTools()->GetSpriteScale(&client_data7::g_pData->m_aSprites[client_data7::SPRITE_PICKUP_SHOTGUN], ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, client_data7::g_pData->m_Weapons.m_aId[WEAPON_SHOTGUN].m_VisualSize * ScaleX, client_data7::g_pData->m_Weapons.m_aId[WEAPON_SHOTGUN].m_VisualSize * ScaleY);
+	RenderTools()->GetSpriteScale(&client_data7::g_pData->m_aSprites[client_data7::SPRITE_PICKUP_GRENADE], ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, client_data7::g_pData->m_Weapons.m_aId[WEAPON_GRENADE].m_VisualSize * ScaleX, client_data7::g_pData->m_Weapons.m_aId[WEAPON_GRENADE].m_VisualSize * ScaleY);
+	RenderTools()->GetSpriteScale(&client_data7::g_pData->m_aSprites[client_data7::SPRITE_PICKUP_LASER], ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, client_data7::g_pData->m_Weapons.m_aId[WEAPON_LASER].m_VisualSize * ScaleX, client_data7::g_pData->m_Weapons.m_aId[WEAPON_LASER].m_VisualSize * ScaleY);
+	RenderTools()->GetSpriteScale(SPRITE_PICKUP_NINJA, ScaleX, ScaleY);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 128.f * ScaleX, 128.f * ScaleY);
 
 	for(int i = 0; i < NUM_WEAPONS; ++i)
 	{
-		RenderTools()->SelectSprite(g_pData->m_Weapons.m_aId[i].m_pSpriteBody);
-		RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, g_pData->m_Weapons.m_aId[i].m_VisualSize);
+		RenderTools()->GetSpriteScale(g_pData->m_Weapons.m_aId[i].m_pSpriteBody, ScaleX, ScaleY);
+		Graphics()->QuadsSetSubset(0, 0, 1, 1);
+		RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, g_pData->m_Weapons.m_aId[i].m_VisualSize * ScaleX, g_pData->m_Weapons.m_aId[i].m_VisualSize * ScaleY);
 	}
 
 	for(int i = 0; i < NUM_WEAPONS; ++i)
 	{
-		RenderTools()->SelectSprite(g_pData->m_Weapons.m_aId[i].m_pSpriteProj);
-		RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 32.f, false);
+		Graphics()->QuadsSetSubset(0, 0, 1, 1);
+		RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 32.f);
 	}
 
-	RenderTools()->SelectSprite(SPRITE_PART_SPLAT01);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 24.f, false);
-	RenderTools()->SelectSprite(SPRITE_PART_SPLAT02);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 24.f, false);
-	RenderTools()->SelectSprite(SPRITE_PART_SPLAT03);
-	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 24.f, false);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 24.f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 24.f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	RenderTools()->QuadContainerAddSprite(m_ItemsQuadContainerIndex, 24.f);
 }
 
 void CItems::AddExtraProjectile(CNetObj_Projectile *pProj)
