@@ -155,7 +155,9 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 	EMapImageModType EntitiesModType = MAP_IMAGE_MOD_TYPE_DDNET;
 	bool EntitesAreMasked = !GameClient()->m_GameInfo.m_DontMaskEntities;
 
-	if(GameClient()->m_GameInfo.m_EntitiesDDNet)
+	if(GameClient()->m_GameInfo.m_EntitiesFDDrace)
+		EntitiesModType = MAP_IMAGE_MOD_TYPE_FDDRACE;
+	else if(GameClient()->m_GameInfo.m_EntitiesDDNet)
 		EntitiesModType = MAP_IMAGE_MOD_TYPE_DDNET;
 	else if(GameClient()->m_GameInfo.m_EntitiesDDRace)
 		EntitiesModType = MAP_IMAGE_MOD_TYPE_DDRACE;
@@ -217,7 +219,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 
 		if(ImagePNGLoaded && ImgInfo.m_Width > 0 && ImgInfo.m_Height > 0)
 		{
-			int ColorChannelCount = 0;
+			int ColorChannelCount = 4;
 			if(ImgInfo.m_Format == CImageInfo::FORMAT_ALPHA)
 				ColorChannelCount = 1;
 			else if(ImgInfo.m_Format == CImageInfo::FORMAT_RGB)
@@ -330,6 +332,8 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 			}
 
 			free(pBuildImgData);
+
+			Graphics()->FreePNG(&ImgInfo);
 		}
 	}
 
@@ -465,6 +469,7 @@ void CMapImages::UpdateEntityLayerText(void *pTexBuffer, int ImageColorChannelCo
 void CMapImages::InitOverlayTextures()
 {
 	int TextureSize = 64 * m_TextureScale / 100;
+	TextureSize = clamp(TextureSize, 2, 64);
 	int TextureToVerticalCenterOffset = (64 - TextureSize) / 2; // should be used to move texture to the center of 64 pixels area
 
 	if(m_OverlayBottomTexture == -1)
