@@ -59,7 +59,7 @@ void CMapLayers::MapScreenToGroup(float CenterX, float CenterY, CMapItemGroup *p
 	Graphics()->MapScreen(Points[0], Points[1], Points[2], Points[3]);
 }
 
-void CMapLayers::EnvelopeEval(float TimeOffset, int Env, float *pChannels, void *pUser)
+void CMapLayers::EnvelopeEval(int TimeOffsetMillis, int Env, float *pChannels, void *pUser)
 {
 	CMapLayers *pThis = (CMapLayers *)pUser;
 	pChannels[0] = 0;
@@ -84,8 +84,10 @@ void CMapLayers::EnvelopeEval(float TimeOffset, int Env, float *pChannels, void
 
 	CMapItemEnvelope *pItem = (CMapItemEnvelope *)pThis->m_pLayers->Map()->GetItem(Start + Env, 0, 0);
 
-	static float s_Time = 0.0f;
-	static float s_LastLocalTime = pThis->LocalTime();
+	const int64 TickToMicroSeconds = (1000000ll / (int64)pThis->Client()->GameTickSpeed());
+
+	static int64 s_Time = 0;
+	static int64 s_LastLocalTime = time_get_microseconds();
 	if(pThis->Client()->State() == IClient::STATE_DEMOPLAYBACK)
 	{
 		const IDemoPlayer::CInfo *pInfo = pThis->DemoPlayer()->BaseInfo();
@@ -99,18 +101,27 @@ void CMapLayers::EnvelopeEval(float TimeOffset, int Env, float *pChannels, void
 			}
 			if(pItem->m_Version < 2 || pItem->m_Synchronized)
 			{
-				s_Time = mix((pThis->Client()->PrevGameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick) / (float)pThis->Client()->GameTickSpeed(),
-					(pThis->Client()->GameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick) / (float)pThis->Client()->GameTickSpeed(),
-					pThis->Client()->IntraGameTick(g_Config.m_ClDummy));
+				// get the lerp of the current tick and prev
+				int MinTick = pThis->Client()->PrevGameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
+				int CurTick = pThis->Client()->GameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
+				s_Time = (int64)(mix<double>(
+							 0,
+							 (CurTick - MinTick),
+							 pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
+						 TickToMicroSeconds) +
+					 MinTick * TickToMicroSeconds;
 			}
 			else
 			{
-				s_Time = mix(pThis->m_LastLocalTick / (float)pThis->Client()->GameTickSpeed(),
-					pThis->m_CurrentLocalTick / (float)pThis->Client()->GameTickSpeed(),
-					pThis->Client()->IntraGameTick(g_Config.m_ClDummy));
+				int MinTick = pThis->m_LastLocalTick;
+				s_Time = (int64)(mix<double>(0,
+							 pThis->m_CurrentLocalTick - MinTick,
+							 pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
+						 TickToMicroSeconds) +
+					 MinTick * TickToMicroSeconds;
 			}
 		}
-		pThis->RenderTools()->RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + TimeOffset, pChannels);
+		pThis->RenderTools()->RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + (int64)TimeOffsetMillis * 1000ll, pChannels);
 	}
 	else
 	{
@@ -118,17 +129,24 @@ void CMapLayers::EnvelopeEval(float TimeOffset, int Env, float *pChannels, void
 		{
 			if(pThis->m_pClient->m_Snap.m_pGameInfoObj) // && !(pThis->m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_PAUSED))
 			{
-				s_Time = mix((pThis->Client()->PrevGameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick) / (float)pThis->Client()->GameTickSpeed(),
-					(pThis->Client()->GameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick) / (float)pThis->Client()->GameTickSpeed(),
-					pThis->Client()->IntraGameTick(g_Config.m_ClDummy));
+				// get the lerp of the current tick and prev
+				int MinTick = pThis->Client()->PrevGameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
+				int CurTick = pThis->Client()->GameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
+				s_Time = (int64)(mix<double>(
+							 0,
+							 (CurTick - MinTick),
+							 pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
+						 TickToMicroSeconds) +
+					 MinTick * TickToMicroSeconds;
 			}
 		}
 		else
 		{
-			s_Time += pThis->LocalTime() - s_LastLocalTime;
-			s_LastLocalTime = pThis->LocalTime();
+			int64 CurTime = time_get_microseconds();
+			s_Time += CurTime - s_LastLocalTime;
+			s_LastLocalTime = CurTime;
 		}
-		pThis->RenderTools()->RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + TimeOffset, pChannels);
+		pThis->RenderTools()->RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + (int64)TimeOffsetMillis * 1000ll, pChannels);
 	}
 }
 
@@ -585,7 +603,7 @@ void CMapLayers::OnMapLoad()
 
 						if(!DoTextureCoords)
 						{
-							tmpTiles.reserve((size_t)(pTMap->m_Width * pTMap->m_Height));
+							tmpTiles.reserve((size_t)pTMap->m_Width * pTMap->m_Height);
 							tmpBorderTopTiles.reserve((size_t)pTMap->m_Width);
 							tmpBorderBottomTiles.reserve((size_t)pTMap->m_Width);
 							tmpBorderLeftTiles.reserve((size_t)pTMap->m_Height);
@@ -594,7 +612,7 @@ void CMapLayers::OnMapLoad()
 						}
 						else
 						{
-							tmpTileTexCoords.reserve((size_t)(pTMap->m_Width * pTMap->m_Height));
+							tmpTileTexCoords.reserve((size_t)pTMap->m_Width * pTMap->m_Height);
 							tmpBorderTopTilesTexCoords.reserve((size_t)pTMap->m_Width);
 							tmpBorderBottomTilesTexCoords.reserve((size_t)pTMap->m_Width);
 							tmpBorderLeftTilesTexCoords.reserve((size_t)pTMap->m_Height);
@@ -820,7 +838,7 @@ void CMapLayers::OnMapLoad()
 						size_t UploadDataSize = tmpTileTexCoords.size() * sizeof(SGraphicTileTexureCoords) + tmpTiles.size() * sizeof(SGraphicTile);
 						if(UploadDataSize > 0)
 						{
-							char *pUploadData = new char[UploadDataSize];
+							char *pUploadData = (char *)malloc(sizeof(char) * UploadDataSize);
 
 							mem_copy_special(pUploadData, pTmpTiles, sizeof(vec2), tmpTiles.size() * 4, (DoTextureCoords ? sizeof(vec3) : 0));
 							if(DoTextureCoords)
@@ -829,8 +847,7 @@ void CMapLayers::OnMapLoad()
 							}
 
 							// first create the buffer object
-							int BufferObjectIndex = Graphics()->CreateBufferObject(UploadDataSize, pUploadData);
-							delete[] pUploadData;
+							int BufferObjectIndex = Graphics()->CreateBufferObject(UploadDataSize, pUploadData, true);
 
 							// then create the buffer container
 							SBufferContainerInfo ContainerInfo;
@@ -989,7 +1006,7 @@ void CMapLayers::RenderTileLayer(int LayerIndex, ColorRGBA *pColor, CMapItemLaye
 	if(pTileLayer->m_ColorEnv >= 0)
 	{
 		float aChannels[4];
-		EnvelopeEval(pTileLayer->m_ColorEnvOffset / 1000.0f, pTileLayer->m_ColorEnv, aChannels, this);
+		EnvelopeEval(pTileLayer->m_ColorEnvOffset, pTileLayer->m_ColorEnv, aChannels, this);
 		r = aChannels[0];
 		g = aChannels[1];
 		b = aChannels[2];
@@ -1371,6 +1388,8 @@ void CMapLayers::RenderQuadLayer(int LayerIndex, CMapItemLayerQuads *pQuadLayer,
 	static std::vector<SQuadRenderInfo> s_QuadRenderInfo;
 
 	s_QuadRenderInfo.resize(pQuadLayer->m_NumQuads);
+	size_t QuadsRenderCount = 0;
+	size_t CurQuadOffset = 0;
 	for(int i = 0; i < pQuadLayer->m_NumQuads; ++i)
 	{
 		CQuad *q = &pQuads[i];
@@ -1379,7 +1398,7 @@ void CMapLayers::RenderQuadLayer(int LayerIndex, CMapItemLayerQuads *pQuadLayer,
 		aColor[0] = aColor[1] = aColor[2] = aColor[3] = 1.f;
 		if(q->m_ColorEnv >= 0)
 		{
-			EnvelopeEval(q->m_ColorEnvOffset / 1000.0f, q->m_ColorEnv, aColor, this);
+			EnvelopeEval(q->m_ColorEnvOffset, q->m_ColorEnv, aColor, this);
 		}
 
 		float OffsetX = 0;
@@ -1389,19 +1408,30 @@ void CMapLayers::RenderQuadLayer(int LayerIndex, CMapItemLayerQuads *pQuadLayer,
 		if(q->m_PosEnv >= 0)
 		{
 			float aChannels[4];
-			EnvelopeEval(q->m_PosEnvOffset / 1000.0f, q->m_PosEnv, aChannels, this);
+			EnvelopeEval(q->m_PosEnvOffset, q->m_PosEnv, aChannels, this);
 			OffsetX = aChannels[0];
 			OffsetY = aChannels[1];
 			Rot = aChannels[2] / 180.0f * pi;
 		}
 
-		SQuadRenderInfo &QInfo = s_QuadRenderInfo[i];
-		mem_copy(QInfo.m_aColor, aColor, sizeof(aColor));
-		QInfo.m_aOffsets[0] = OffsetX;
-		QInfo.m_aOffsets[1] = OffsetY;
-		QInfo.m_Rotation = Rot;
+		if(aColor[3] > 0)
+		{
+			SQuadRenderInfo &QInfo = s_QuadRenderInfo[QuadsRenderCount++];
+			mem_copy(QInfo.m_aColor, aColor, sizeof(aColor));
+			QInfo.m_aOffsets[0] = OffsetX;
+			QInfo.m_aOffsets[1] = OffsetY;
+			QInfo.m_Rotation = Rot;
+		}
+		else
+		{
+			// render quads of the current offset directly(cancel batching)
+			Graphics()->RenderQuadLayer(Visuals.m_BufferContainerIndex, &s_QuadRenderInfo[0], QuadsRenderCount, CurQuadOffset);
+			QuadsRenderCount = 0;
+			// since this quad is ignored, the offset is the next quad
+			CurQuadOffset = i + 1;
+		}
 	}
-	Graphics()->RenderQuadLayer(Visuals.m_BufferContainerIndex, &s_QuadRenderInfo[0], pQuadLayer->m_NumQuads);
+	Graphics()->RenderQuadLayer(Visuals.m_BufferContainerIndex, &s_QuadRenderInfo[0], QuadsRenderCount, CurQuadOffset);
 }
 
 void CMapLayers::LayersOfGroupCount(CMapItemGroup *pGroup, int &TileLayerCount, int &QuadLayerCount, bool &PassedGameLayer)
@@ -1721,7 +1751,7 @@ void CMapLayers::OnRender()
 					CTile *pTiles = (CTile *)m_pLayers->Map()->GetData(pTMap->m_Data);
 					unsigned int Size = m_pLayers->Map()->GetDataSize(pTMap->m_Data);
 
-					if(Size >= pTMap->m_Width * pTMap->m_Height * sizeof(CTile))
+					if(Size >= (size_t)pTMap->m_Width * pTMap->m_Height * sizeof(CTile))
 					{
 						ColorRGBA Color = ColorRGBA(pTMap->m_Color.r / 255.0f, pTMap->m_Color.g / 255.0f, pTMap->m_Color.b / 255.0f, pTMap->m_Color.a / 255.0f);
 						if(IsGameLayer && EntityOverlayVal)
@@ -1819,7 +1849,7 @@ void CMapLayers::OnRender()
 				CTile *pFrontTiles = (CTile *)m_pLayers->Map()->GetData(pTMap->m_Front);
 				unsigned int Size = m_pLayers->Map()->GetDataSize(pTMap->m_Front);
 
-				if(Size >= pTMap->m_Width * pTMap->m_Height * sizeof(CTile))
+				if(Size >= (size_t)pTMap->m_Width * pTMap->m_Height * sizeof(CTile))
 				{
 					ColorRGBA Color = ColorRGBA(pTMap->m_Color.r / 255.0f, pTMap->m_Color.g / 255.0f, pTMap->m_Color.b / 255.0f, pTMap->m_Color.a / 255.0f * EntityOverlayVal / 100.0f);
 					if(!Graphics()->IsTileBufferingEnabled())
@@ -1846,7 +1876,7 @@ void CMapLayers::OnRender()
 				CSwitchTile *pSwitchTiles = (CSwitchTile *)m_pLayers->Map()->GetData(pTMap->m_Switch);
 				unsigned int Size = m_pLayers->Map()->GetDataSize(pTMap->m_Switch);
 
-				if(Size >= pTMap->m_Width * pTMap->m_Height * sizeof(CSwitchTile))
+				if(Size >= (size_t)pTMap->m_Width * pTMap->m_Height * sizeof(CSwitchTile))
 				{
 					ColorRGBA Color = ColorRGBA(pTMap->m_Color.r / 255.0f, pTMap->m_Color.g / 255.0f, pTMap->m_Color.b / 255.0f, pTMap->m_Color.a / 255.0f * EntityOverlayVal / 100.0f);
 					if(!Graphics()->IsTileBufferingEnabled())
@@ -1879,7 +1909,7 @@ void CMapLayers::OnRender()
 				CTeleTile *pTeleTiles = (CTeleTile *)m_pLayers->Map()->GetData(pTMap->m_Tele);
 				unsigned int Size = m_pLayers->Map()->GetDataSize(pTMap->m_Tele);
 
-				if(Size >= pTMap->m_Width * pTMap->m_Height * sizeof(CTeleTile))
+				if(Size >= (size_t)pTMap->m_Width * pTMap->m_Height * sizeof(CTeleTile))
 				{
 					ColorRGBA Color = ColorRGBA(pTMap->m_Color.r / 255.0f, pTMap->m_Color.g / 255.0f, pTMap->m_Color.b / 255.0f, pTMap->m_Color.a / 255.0f * EntityOverlayVal / 100.0f);
 					if(!Graphics()->IsTileBufferingEnabled())
@@ -1910,7 +1940,7 @@ void CMapLayers::OnRender()
 				CSpeedupTile *pSpeedupTiles = (CSpeedupTile *)m_pLayers->Map()->GetData(pTMap->m_Speedup);
 				unsigned int Size = m_pLayers->Map()->GetDataSize(pTMap->m_Speedup);
 
-				if(Size >= pTMap->m_Width * pTMap->m_Height * sizeof(CSpeedupTile))
+				if(Size >= (size_t)pTMap->m_Width * pTMap->m_Height * sizeof(CSpeedupTile))
 				{
 					ColorRGBA Color = ColorRGBA(pTMap->m_Color.r / 255.0f, pTMap->m_Color.g / 255.0f, pTMap->m_Color.b / 255.0f, pTMap->m_Color.a / 255.0f * EntityOverlayVal / 100.0f);
 					if(!Graphics()->IsTileBufferingEnabled())
@@ -1948,7 +1978,7 @@ void CMapLayers::OnRender()
 				CTuneTile *pTuneTiles = (CTuneTile *)m_pLayers->Map()->GetData(pTMap->m_Tune);
 				unsigned int Size = m_pLayers->Map()->GetDataSize(pTMap->m_Tune);
 
-				if(Size >= pTMap->m_Width * pTMap->m_Height * sizeof(CTuneTile))
+				if(Size >= (size_t)pTMap->m_Width * pTMap->m_Height * sizeof(CTuneTile))
 				{
 					ColorRGBA Color = ColorRGBA(pTMap->m_Color.r / 255.0f, pTMap->m_Color.g / 255.0f, pTMap->m_Color.b / 255.0f, pTMap->m_Color.a / 255.0f * EntityOverlayVal / 100.0f);
 					if(!Graphics()->IsTileBufferingEnabled())
