@@ -21,6 +21,8 @@ CMenuBackground::CMenuBackground() :
 	m_Camera.m_PrevCenter = vec2(0.0f, 0.0f);
 	m_MenuCenter = vec2(0.0f, 0.0f);
 
+	m_ChangedPosition = false;
+
 	ResetPositions();
 
 	m_CurrentPosition = -1;
@@ -70,7 +72,7 @@ void CMenuBackground::ResetPositions()
 	m_Positions[POS_SETTINGS_DDNET] = vec2(1200.0f, 200.0f);
 	m_Positions[POS_SETTINGS_ASSETS] = vec2(500.0f, 500.0f);
 	for(int i = 0; i < POS_BROWSER_CUSTOM_NUM; ++i)
-		m_Positions[POS_BROWSER_CUSTOM0 + i] = vec2(500.0f + (75.0f * (float)i), 1250.0f - (75.0f * (float)i));
+		m_Positions[POS_BROWSER_CUSTOM0 + i] = vec2(500.0f + (75.0f * (float)i), 650.0f - (75.0f * (float)i));
 	for(int i = 0; i < POS_SETTINGS_RESERVED_NUM; ++i)
 		m_Positions[POS_SETTINGS_RESERVED0 + i] = vec2(0, 0);
 	for(int i = 0; i < POS_RESERVED_NUM; ++i)
@@ -106,14 +108,14 @@ int CMenuBackground::ThemeScan(const char *pName, int IsDir, int DirType, void *
 		return 0;
 
 	// try to edit an existing theme
-	for(int i = 0; i < (int)pSelf->m_lThemes.size(); i++)
+	for(auto &Theme : pSelf->m_lThemes)
 	{
-		if(str_comp(pSelf->m_lThemes[i].m_Name, aThemeName) == 0)
+		if(str_comp(Theme.m_Name, aThemeName) == 0)
 		{
 			if(IsDay)
-				pSelf->m_lThemes[i].m_HasDay = true;
+				Theme.m_HasDay = true;
 			if(IsNight)
-				pSelf->m_lThemes[i].m_HasNight = true;
+				Theme.m_HasNight = true;
 			return 0;
 		}
 	}
@@ -155,6 +157,7 @@ int CMenuBackground::ThemeIconScan(const char *pName, int IsDir, int DirType, vo
 			pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
 
 			Theme.m_IconTexture = pSelf->Graphics()->LoadTextureRaw(Info.m_Width, Info.m_Height, Info.m_Format, Info.m_pData, Info.m_Format, 0);
+			pSelf->Graphics()->FreePNG(&Info);
 			return 0;
 		}
 	}
@@ -199,6 +202,9 @@ void CMenuBackground::LoadMenuBackground(bool HasDayHint, bool HasNightHint)
 			case SEASON_WINTER:
 				pMenuMap = "winter";
 				break;
+			case SEASON_NEWYEAR:
+				pMenuMap = "newyear";
+				break;
 			}
 		}
 		else if(str_comp(pMenuMap, "rand") == 0)
@@ -308,7 +314,8 @@ bool CMenuBackground::Render()
 	m_Camera.m_Zoom = 0.7f;
 	static vec2 Dir = vec2(1.0f, 0.0f);
 
-	if(distance(m_Camera.m_Center, m_RotationCenter) <= (float)g_Config.m_ClRotationRadius + 0.5f)
+	float DistToCenter = distance(m_Camera.m_Center, m_RotationCenter);
+	if(!m_ChangedPosition && absolute(DistToCenter - (float)g_Config.m_ClRotationRadius) <= 0.5f)
 	{
 		// do little rotation
 		float RotPerTick = 360.0f / (float)g_Config.m_ClRotationSpeed * clamp(Client()->RenderFrameTime(), 0.0f, 0.1f);
@@ -318,9 +325,17 @@ bool CMenuBackground::Render()
 	else
 	{
 		// positions for the animation
-		Dir = normalize(m_AnimationStartPos - m_RotationCenter);
-		vec2 TargetPos = m_RotationCenter + Dir * (float)g_Config.m_ClRotationRadius;
+		vec2 DirToCenter;
+		if(DistToCenter > 0.5f)
+			DirToCenter = normalize(m_AnimationStartPos - m_RotationCenter);
+		else
+			DirToCenter = vec2(1, 0);
+		vec2 TargetPos = m_RotationCenter + DirToCenter * (float)g_Config.m_ClRotationRadius;
 		float Distance = distance(m_AnimationStartPos, TargetPos);
+		if(Distance > 0.001f)
+			Dir = normalize(m_AnimationStartPos - TargetPos);
+		else
+			Dir = vec2(1, 0);
 
 		// move time
 		m_MoveTime += clamp(Client()->RenderFrameTime(), 0.0f, 0.1f) * g_Config.m_ClCameraSpeed / 10.0f;
@@ -328,10 +343,19 @@ bool CMenuBackground::Render()
 		XVal = pow(XVal, 7.0f);
 
 		m_Camera.m_Center = TargetPos + Dir * (XVal * Distance);
+		if(m_CurrentPosition < 0)
+		{
+			m_AnimationStartPos = m_Camera.m_Center;
+			m_MoveTime = 0.0f;
+		}
+
+		m_ChangedPosition = false;
 	}
 
 	CMapLayers::OnRender();
 
+	m_CurrentPosition = -1;
+
 	return true;
 }
 
@@ -352,6 +376,8 @@ void CMenuBackground::ChangePosition(int PositionNumber)
 		{
 			m_CurrentPosition = POS_START;
 		}
+
+		m_ChangedPosition = true;
 	}
 	m_AnimationStartPos = m_Camera.m_Center;
 	m_RotationCenter = m_Positions[m_CurrentPosition];
