@@ -10,6 +10,7 @@
 #include <engine/friends.h>
 #include <engine/shared/config.h>
 #include <engine/shared/linereader.h>
+#include <engine/textrender.h>
 #include <game/client/components/mapimages.h>
 
 #include <game/client/component.h>
@@ -23,6 +24,22 @@ struct CServerProcess
 	CLineReader LineReader;
 };
 
+struct SColorPicker
+{
+public:
+	const float ms_Width = 160.0f;
+	const float ms_Height = 186.0f;
+
+	float m_X;
+	float m_Y;
+
+	bool m_Active;
+
+	CUIRect m_AttachedRect;
+	unsigned int *m_pColor;
+	unsigned int m_HSVColor;
+};
+
 // compnent to fetch keypresses, override all other input
 class CMenusKeyBinder : public CComponent
 {
@@ -48,24 +65,34 @@ class CMenus : public CComponent
 	static ColorRGBA ms_ColorTabbarActive;
 	static ColorRGBA ms_ColorTabbarHover;
 
+	static SColorPicker ms_ColorPicker;
+	static bool ms_ValueSelectorTextMode;
+
+	char m_aLocalStringHelper[1024];
+
+	float ButtonColorMulActive() { return 0.5f; }
+	float ButtonColorMulHot() { return 1.5f; }
+	float ButtonColorMulDefault() { return 1.0f; }
 	float ButtonColorMul(const void *pID);
 
 	int DoButton_DemoPlayer(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
 	int DoButton_Sprite(const void *pID, int ImageID, int SpriteID, int Checked, const CUIRect *pRect, int Corners);
 	int DoButton_Toggle(const void *pID, int Checked, const CUIRect *pRect, bool Active);
-	int DoButton_Menu(const void *pID, const char *pText, int Checked, const CUIRect *pRect, const char *pImageName = 0, int Corners = CUI::CORNER_ALL, float r = 5.0f, float FontFactor = 0.0f, vec4 ColorHot = vec4(1.0f, 1.0f, 1.0f, 0.75f), vec4 Color = vec4(1, 1, 1, 0.5f));
-	int DoButton_MenuTab(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Corners, const ColorRGBA *pDefaultColor = NULL, const ColorRGBA *pActiveColor = NULL, const ColorRGBA *pHoverColor = NULL, float EdgeRounding = 10);
+	int DoButton_Menu(const void *pID, const char *pText, int Checked, const CUIRect *pRect, const char *pImageName = 0, int Corners = CUI::CORNER_ALL, float r = 5.0f, float FontFactor = 0.0f, vec4 ColorHot = vec4(1.0f, 1.0f, 1.0f, 0.75f), vec4 Color = vec4(1, 1, 1, 0.5f), int AlignVertically = 1, bool CheckForActiveColorPicker = false);
+	int DoButton_MenuTab(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Corners, SUIAnimator *pAnimator = NULL, const ColorRGBA *pDefaultColor = NULL, const ColorRGBA *pActiveColor = NULL, const ColorRGBA *pHoverColor = NULL, float EdgeRounding = 10, int AlignVertically = 1);
 
 	int DoButton_CheckBox_Common(const void *pID, const char *pText, const char *pBoxText, const CUIRect *pRect);
 	int DoButton_CheckBox(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
+	int DoButton_CheckBoxAutoVMarginAndSet(const void *pID, const char *pText, int *pValue, CUIRect *pRect, float VMargin);
 	int DoButton_CheckBox_Number(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
-
+	ColorHSLA DoLine_ColorPicker(int *pResetID, const float LineSize, const float WantedPickerPosition, const float LabelSize, const float BottomMargin, CUIRect *pMainRect, const char *pText, unsigned int *pColorValue, const ColorRGBA DefaultColor, bool CheckBoxSpacing = true, bool UseCheckBox = false, int *pCheckBoxValue = NULL);
+	void DoLaserPreview(const CUIRect *pRect, const ColorHSLA OutlineColor, const ColorHSLA InnerColor);
 	/*static void ui_draw_menu_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
 	static void ui_draw_keyselect_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
 	static void ui_draw_menu_tab_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
 	static void ui_draw_settings_tab_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
 	*/
-
+	int DoValueSelector(void *pID, CUIRect *pRect, const char *pLabel, bool UseScroll, int Current, int Min, int Max, int Step, float Scale, bool IsHex, float Round, ColorRGBA *Color);
 	int DoButton_Icon(int ImageId, int SpriteId, const CUIRect *pRect);
 	int DoButton_GridHeader(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
 
@@ -88,6 +115,97 @@ class CMenus : public CComponent
 	//static int ui_do_key_reader(void *id, const CUIRect *rect, int key);
 	void UiDoGetButtons(int Start, int Stop, CUIRect View, CUIRect ScopeView);
 
+	void RenderColorPicker();
+
+	// new gui with gui elements
+	template<typename T>
+	int DoButtonMenu(CUIElement &UIElement, const void *pID, T &&GetTextLambda, int Checked, const CUIRect *pRect, bool HintRequiresStringCheck, bool HintCanChangePositionOrSize = false, int Corners = CUI::CORNER_ALL, float r = 5.0f, float FontFactor = 0.0f, vec4 ColorHot = vec4(1.0f, 1.0f, 1.0f, 0.75f), vec4 Color = vec4(1, 1, 1, 0.5f), int AlignVertically = 1)
+	{
+		CUIRect Text = *pRect;
+		Text.HMargin(pRect->h >= 20.0f ? 2.0f : 1.0f, &Text);
+		Text.HMargin((Text.h * FontFactor) / 2.0f, &Text);
+
+		if(UIElement.Size() != 3 || HintRequiresStringCheck || HintCanChangePositionOrSize)
+		{
+			bool NeedsRecalc = UIElement.Size() != 3;
+			if(HintCanChangePositionOrSize)
+			{
+				if(UIElement.Size() == 3)
+				{
+					if(UIElement.Get(0)->m_X != pRect->x || UIElement.Get(0)->m_Y != pRect->y || UIElement.Get(0)->m_Width != pRect->w || UIElement.Get(0)->m_Y != pRect->h)
+					{
+						NeedsRecalc = true;
+					}
+				}
+			}
+			const char *pText = NULL;
+			if(HintRequiresStringCheck)
+			{
+				if(UIElement.Size() == 3)
+				{
+					pText = GetTextLambda();
+					if(str_comp(UIElement.Get(0)->m_Text.c_str(), pText) != 0)
+					{
+						NeedsRecalc = true;
+					}
+				}
+			}
+			if(NeedsRecalc)
+			{
+				if(UIElement.Size() > 0)
+				{
+					UI()->ResetUIElement(UIElement);
+				}
+
+				vec4 RealColor = Color;
+				for(int i = 0; i < 3; ++i)
+				{
+					Color.a = RealColor.a;
+					if(i == 0)
+						Color.a *= ButtonColorMulActive();
+					else if(i == 1)
+						Color.a *= ButtonColorMulHot();
+					else if(i == 2)
+						Color.a *= ButtonColorMulDefault();
+					Graphics()->SetColor(Color);
+
+					CUIElement::SUIElementRect NewRect;
+					NewRect.m_UIRectQuadContainer = RenderTools()->CreateRoundRectQuadContainer(pRect->x, pRect->y, pRect->w, pRect->h, r, Corners);
+
+					NewRect.m_X = pRect->x;
+					NewRect.m_Y = pRect->y;
+					NewRect.m_Width = pRect->w;
+					NewRect.m_Height = pRect->h;
+
+					if(i == 0)
+					{
+						if(pText == NULL)
+							pText = GetTextLambda();
+						NewRect.m_Text = pText;
+						UI()->DoLabel(NewRect, &Text, pText, Text.h * ms_FontmodHeight, 0, -1, AlignVertically);
+					}
+					UIElement.Add(NewRect);
+				}
+				Graphics()->SetColor(1, 1, 1, 1);
+			}
+		}
+
+		// render
+		size_t Index = 2;
+		if(UI()->ActiveItem() == pID)
+			Index = 0;
+		else if(UI()->HotItem() == pID)
+			Index = 1;
+		Graphics()->TextureClear();
+		Graphics()->RenderQuadContainer(UIElement.Get(Index)->m_UIRectQuadContainer, -1);
+		STextRenderColor ColorText(TextRender()->DefaultTextColor());
+		STextRenderColor ColorTextOutline(TextRender()->DefaultTextOutlineColor());
+		if(UIElement.Get(0)->m_UITextContainer != -1)
+			TextRender()->RenderTextContainer(UIElement.Get(0)->m_UITextContainer, &ColorText, &ColorTextOutline);
+
+		return UI()->DoButtonLogic(pID, Checked, pRect);
+	}
+
 	struct CListboxItem
 	{
 		int m_Visible;
@@ -186,6 +304,9 @@ protected:
 	char m_aMessageBody[512];
 	char m_aMessageButton[512];
 
+	CUIElement m_RefreshButton;
+	CUIElement m_ConnectButton;
+
 	void PopupMessage(const char *pTopic, const char *pBody, const char *pButton);
 
 	// TODO: this is a bit ugly but.. well.. yeah
@@ -316,7 +437,7 @@ protected:
 		const CFriendInfo *m_pFriendInfo;
 		int m_NumFound;
 
-		bool operator<(const CFriendItem &Other)
+		bool operator<(const CFriendItem &Other) const
 		{
 			if(m_NumFound && !Other.m_NumFound)
 				return true;
@@ -440,6 +561,8 @@ public:
 		PAGE_NETWORK,
 		PAGE_GHOST,
 
+		PAGE_LENGTH,
+
 		SETTINGS_LANGUAGE = 0,
 		SETTINGS_GENERAL,
 		SETTINGS_PLAYER,
@@ -450,8 +573,33 @@ public:
 		SETTINGS_SOUND,
 		SETTINGS_DDNET,
 		SETTINGS_ASSETS,
+
+		SETTINGS_LENGTH,
+
+		BIG_TAB_NEWS = 0,
+		BIG_TAB_INTERNET,
+		BIG_TAB_LAN,
+		BIG_TAB_FAVORITES,
+		BIG_TAB_DDNET,
+		BIG_TAB_KOG,
+		BIG_TAB_DEMOS,
+
+		BIG_TAB_LENGTH,
+
+		SMALL_TAB_HOME = 0,
+		SMALL_TAB_QUIT,
+		SMALL_TAB_SETTINGS,
+		SMALL_TAB_EDITOR,
+		SMALL_TAB_DEMOBUTTON,
+		SMALL_TAB_SERVER,
+
+		SMALL_TAB_LENGTH,
 	};
 
+	SUIAnimator m_aAnimatorsBigPage[BIG_TAB_LENGTH];
+	SUIAnimator m_aAnimatorsSmallPage[SMALL_TAB_LENGTH];
+	SUIAnimator m_aAnimatorsSettingsTab[SETTINGS_LENGTH];
+
 	// DDRace
 	int DoButton_CheckBox_DontCare(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
 	sorted_array<CDemoItem> m_lDemos;
@@ -473,7 +621,7 @@ public:
 		CGhostItem() :
 			m_Slot(-1), m_Own(false) { m_aFilename[0] = 0; }
 
-		bool operator<(const CGhostItem &Other) { return m_Time < Other.m_Time; }
+		bool operator<(const CGhostItem &Other) const { return m_Time < Other.m_Time; }
 
 		bool Active() const { return m_Slot != -1; }
 		bool HasFile() const { return m_aFilename[0]; }
@@ -502,6 +650,7 @@ public:
 	{
 		POPUP_NONE = 0,
 		POPUP_FIRST_LAUNCH,
+		POPUP_POINTS,
 		POPUP_CONNECTING,
 		POPUP_MESSAGE,
 		POPUP_DISCONNECTED,
@@ -528,6 +677,7 @@ public:
 private:
 	static int GhostlistFetchCallback(const char *pName, int IsDir, int StorageType, void *pUser);
 	void SetMenuPage(int NewPage);
+	bool HandleListInputs(const CUIRect &View, float &ScrollValue, float ScrollAmount, int *pScrollOffset, float ElemHeight, int &SelectedIndex, int NumElems);
 
 	// found in menus_ingame.cpp
 	void RenderInGameNetwork(CUIRect MainView);
@@ -536,6 +686,7 @@ private:
 	// found in menus_settings.cpp
 	void RenderSettingsDDNet(CUIRect MainView);
 	void RenderSettingsHUD(CUIRect MainView);
+	ColorHSLA RenderHSLColorPicker(const CUIRect *pRect, unsigned int *pColor, bool Alpha);
 	ColorHSLA RenderHSLScrollbars(CUIRect *pRect, unsigned int *pColor, bool Alpha = false);
 
 	CServerProcess m_ServerProcess;
