@@ -30,8 +30,8 @@ void CParticles::OnReset()
 	m_aParticles[MAX_PARTICLES - 1].m_NextPart = -1;
 	m_FirstFree = 0;
 
-	for(int i = 0; i < NUM_GROUPS; i++)
-		m_aFirstPart[i] = -1;
+	for(int &FirstPart : m_aFirstPart)
+		FirstPart = -1;
 }
 
 void CParticles::Add(int Group, CParticle *pPart, float TimePassed)
@@ -89,9 +89,9 @@ void CParticles::Update(float TimePassed)
 		FrictionFraction -= 0.05f;
 	}
 
-	for(int g = 0; g < NUM_GROUPS; g++)
+	for(int &FirstPart : m_aFirstPart)
 	{
-		int i = m_aFirstPart[g];
+		int i = FirstPart;
 		while(i != -1)
 		{
 			int Next = m_aParticles[i].m_NextPart;
@@ -116,7 +116,7 @@ void CParticles::Update(float TimePassed)
 				if(m_aParticles[i].m_PrevPart != -1)
 					m_aParticles[m_aParticles[i].m_PrevPart].m_NextPart = m_aParticles[i].m_NextPart;
 				else
-					m_aFirstPart[g] = m_aParticles[i].m_NextPart;
+					FirstPart = m_aParticles[i].m_NextPart;
 
 				if(m_aParticles[i].m_NextPart != -1)
 					m_aParticles[m_aParticles[i].m_NextPart].m_PrevPart = m_aParticles[i].m_PrevPart;
@@ -167,15 +167,28 @@ void CParticles::OnInit()
 
 	for(int i = 0; i <= (SPRITE_PART9 - SPRITE_PART_SLICE); ++i)
 	{
-		RenderTools()->SelectSprite(i);
-		RenderTools()->QuadContainerAddSprite(m_ParticleQuadContainerIndex, 1.f, false);
+		Graphics()->QuadsSetSubset(0, 0, 1, 1);
+		RenderTools()->QuadContainerAddSprite(m_ParticleQuadContainerIndex, 1.f);
 	}
 }
 
+bool CParticles::ParticleIsVisibleOnScreen(const vec2 &CurPos, float CurSize)
+{
+	float ScreenX0, ScreenY0, ScreenX1, ScreenY1;
+	Graphics()->GetScreen(&ScreenX0, &ScreenY0, &ScreenX1, &ScreenY1);
+
+	// for simplicity assume the worst case rotation, that increases the bounding box around the particle by its diagonal
+	const float SqrtOf2 = sqrtf(2);
+	CurSize = SqrtOf2 * CurSize;
+
+	// always uses the mid of the particle
+	float SizeHalf = CurSize / 2;
+
+	return CurPos.x + SizeHalf >= ScreenX0 && CurPos.x - SizeHalf <= ScreenX1 && CurPos.y + SizeHalf >= ScreenY0 && CurPos.y - SizeHalf <= ScreenY1;
+}
+
 void CParticles::RenderGroup(int Group)
 {
-	Graphics()->TextureSet(g_pData->m_aImages[IMAGE_PARTICLES].m_Id);
-
 	// don't use the buffer methods here, else the old renderer gets many draw calls
 	if(Graphics()->IsQuadContainerBufferingEnabled())
 	{
@@ -212,35 +225,41 @@ void CParticles::RenderGroup(int Group)
 			vec2 p = m_aParticles[i].m_Pos;
 			float Size = mix(m_aParticles[i].m_StartSize, m_aParticles[i].m_EndSize, a);
 
-			if(LastColor[0] != m_aParticles[i].m_Color.r || LastColor[1] != m_aParticles[i].m_Color.g || LastColor[2] != m_aParticles[i].m_Color.b || LastColor[3] != m_aParticles[i].m_Color.a || LastQuadOffset != QuadOffset)
+			// the current position, respecting the size, is inside the viewport, render it, else ignore
+			if(ParticleIsVisibleOnScreen(p, Size))
 			{
-				Graphics()->RenderQuadContainerAsSpriteMultiple(m_ParticleQuadContainerIndex, LastQuadOffset, CurParticleRenderCount, s_aParticleRenderInfo);
-				CurParticleRenderCount = 0;
-				LastQuadOffset = QuadOffset;
+				if(LastColor[0] != m_aParticles[i].m_Color.r || LastColor[1] != m_aParticles[i].m_Color.g || LastColor[2] != m_aParticles[i].m_Color.b || LastColor[3] != m_aParticles[i].m_Color.a || LastQuadOffset != QuadOffset)
+				{
+					Graphics()->TextureSet(GameClient()->m_ParticlesSkin.m_SpriteParticles[LastQuadOffset - SPRITE_PART_SLICE]);
+					Graphics()->RenderQuadContainerAsSpriteMultiple(m_ParticleQuadContainerIndex, LastQuadOffset, CurParticleRenderCount, s_aParticleRenderInfo);
+					CurParticleRenderCount = 0;
+					LastQuadOffset = QuadOffset;
 
-				Graphics()->SetColor(
-					m_aParticles[i].m_Color.r,
-					m_aParticles[i].m_Color.g,
-					m_aParticles[i].m_Color.b,
-					m_aParticles[i].m_Color.a);
+					Graphics()->SetColor(
+						m_aParticles[i].m_Color.r,
+						m_aParticles[i].m_Color.g,
+						m_aParticles[i].m_Color.b,
+						m_aParticles[i].m_Color.a);
 
-				LastColor[0] = m_aParticles[i].m_Color.r;
-				LastColor[1] = m_aParticles[i].m_Color.g;
-				LastColor[2] = m_aParticles[i].m_Color.b;
-				LastColor[3] = m_aParticles[i].m_Color.a;
+					LastColor[0] = m_aParticles[i].m_Color.r;
+					LastColor[1] = m_aParticles[i].m_Color.g;
+					LastColor[2] = m_aParticles[i].m_Color.b;
+					LastColor[3] = m_aParticles[i].m_Color.a;
+				}
+
+				s_aParticleRenderInfo[CurParticleRenderCount].m_Pos[0] = p.x;
+				s_aParticleRenderInfo[CurParticleRenderCount].m_Pos[1] = p.y;
+
+				s_aParticleRenderInfo[CurParticleRenderCount].m_Scale = Size;
+				s_aParticleRenderInfo[CurParticleRenderCount].m_Rotation = m_aParticles[i].m_Rot;
+
+				++CurParticleRenderCount;
 			}
 
-			s_aParticleRenderInfo[CurParticleRenderCount].m_Pos[0] = p.x;
-			s_aParticleRenderInfo[CurParticleRenderCount].m_Pos[1] = p.y;
-
-			s_aParticleRenderInfo[CurParticleRenderCount].m_Scale = Size;
-			s_aParticleRenderInfo[CurParticleRenderCount].m_Rotation = m_aParticles[i].m_Rot;
-
-			++CurParticleRenderCount;
-
 			i = m_aParticles[i].m_NextPart;
 		}
 
+		Graphics()->TextureSet(GameClient()->m_ParticlesSkin.m_SpriteParticles[LastQuadOffset - SPRITE_PART_SLICE]);
 		Graphics()->RenderQuadContainerAsSpriteMultiple(m_ParticleQuadContainerIndex, LastQuadOffset, CurParticleRenderCount, s_aParticleRenderInfo);
 	}
 	else
@@ -248,31 +267,36 @@ void CParticles::RenderGroup(int Group)
 		int i = m_aFirstPart[Group];
 
 		Graphics()->BlendNormal();
-		//gfx_blend_additive();
-		Graphics()->TextureSet(g_pData->m_aImages[IMAGE_PARTICLES].m_Id);
-		Graphics()->QuadsBegin();
+		Graphics()->WrapClamp();
 
 		while(i != -1)
 		{
-			RenderTools()->SelectSprite(m_aParticles[i].m_Spr);
 			float a = m_aParticles[i].m_Life / m_aParticles[i].m_LifeSpan;
 			vec2 p = m_aParticles[i].m_Pos;
 			float Size = mix(m_aParticles[i].m_StartSize, m_aParticles[i].m_EndSize, a);
 
-			Graphics()->QuadsSetRotation(m_aParticles[i].m_Rot);
+			// the current position, respecting the size, is inside the viewport, render it, else ignore
+			if(ParticleIsVisibleOnScreen(p, Size))
+			{
+				Graphics()->TextureSet(GameClient()->m_ParticlesSkin.m_SpriteParticles[m_aParticles[i].m_Spr - SPRITE_PART_SLICE]);
+				Graphics()->QuadsBegin();
 
-			Graphics()->SetColor(
-				m_aParticles[i].m_Color.r,
-				m_aParticles[i].m_Color.g,
-				m_aParticles[i].m_Color.b,
-				m_aParticles[i].m_Color.a); // pow(a, 0.75f) *
+				Graphics()->QuadsSetRotation(m_aParticles[i].m_Rot);
 
-			IGraphics::CQuadItem QuadItem(p.x, p.y, Size, Size);
-			Graphics()->QuadsDraw(&QuadItem, 1);
+				Graphics()->SetColor(
+					m_aParticles[i].m_Color.r,
+					m_aParticles[i].m_Color.g,
+					m_aParticles[i].m_Color.b,
+					m_aParticles[i].m_Color.a); // pow(a, 0.75f) *
+
+				IGraphics::CQuadItem QuadItem(p.x, p.y, Size, Size);
+				Graphics()->QuadsDraw(&QuadItem, 1);
+				Graphics()->QuadsEnd();
+			}
 
 			i = m_aParticles[i].m_NextPart;
 		}
-		Graphics()->QuadsEnd();
+		Graphics()->WrapNormal();
 		Graphics()->BlendNormal();
 	}
 }
