@@ -120,9 +120,8 @@ void CScoreboard::RenderSpectators(float x, float y, float w)
 	Cursor.m_LineWidth = w - 20.0f;
 	Cursor.m_MaxLines = 4;
 
-	for(int i = 0; i < MAX_CLIENTS; ++i)
+	for(const auto *pInfo : m_pClient->m_Snap.m_paInfoByName)
 	{
-		const CNetObj_PlayerInfo *pInfo = m_pClient->m_Snap.m_paInfoByName[i];
 		if(!pInfo || pInfo->m_Team != TEAM_SPECTATORS)
 			continue;
 
@@ -148,7 +147,7 @@ void CScoreboard::RenderSpectators(float x, float y, float w)
 	}
 }
 
-void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const char *pTitle)
+void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const char *pTitle, int NumPlayers)
 {
 	if(Team == TEAM_SPECTATORS)
 		return;
@@ -176,6 +175,9 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 	if(Team < -1)
 		Team = 0;
 
+	if(NumPlayers < 0)
+		NumPlayers = m_pClient->m_Snap.m_aTeamSize[Team];
+
 	float h = 760.0f;
 
 	// background
@@ -195,13 +197,16 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 
 	// render title
 	float TitleFontsize = 40.0f;
+	int TitleWidth = (lower32 || lower24 || lower16) ? 1140 : 440;
 	if(!pTitle)
 	{
 		if(m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags & GAMESTATEFLAG_GAMEOVER)
 			pTitle = Localize("Game over");
 		else
 		{
-			str_utf8_truncate(aBuf, sizeof(aBuf), Client()->GetCurrentMap(), 16);
+			str_utf8_copy(aBuf, Client()->GetCurrentMap(), sizeof(aBuf));
+			while(TextRender()->TextWidth(0, TitleFontsize, aBuf, -1, -1.0f) > TitleWidth)
+				aBuf[str_length(aBuf) - 1] = '\0';
 			if(str_comp(aBuf, Client()->GetCurrentMap()))
 				str_append(aBuf, "…", sizeof(aBuf));
 			pTitle = aBuf;
@@ -235,9 +240,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 	if(m_pClient->m_GameInfo.m_TimeScore && g_Config.m_ClDDRaceScoreBoard)
 	{
 		if(m_ServerRecord > 0)
-		{
-			str_format(aBuf, sizeof(aBuf), "%02d:%02d", ((int)m_ServerRecord) / 60, ((int)m_ServerRecord) % 60);
-		}
+			str_time_float(m_ServerRecord, TIME_HOURS, aBuf, sizeof(aBuf));
 		else
 			aBuf[0] = 0;
 	}
@@ -256,28 +259,31 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 	float TeeSizeMod = 1.0f;
 	float Spacing = 16.0f;
 	float RoundRadius = 15.0f;
-	if(m_pClient->m_Snap.m_aTeamSize[Team] > 48)
+	float FontSize = 24.0f;
+	if(NumPlayers > 48)
 	{
 		LineHeight = 20.0f;
 		TeeSizeMod = 0.4f;
 		Spacing = 0.0f;
 		RoundRadius = 5.0f;
+		FontSize = 16.0f;
 	}
-	else if(m_pClient->m_Snap.m_aTeamSize[Team] > 32)
+	else if(NumPlayers > 32)
 	{
 		LineHeight = 27.0f;
 		TeeSizeMod = 0.6f;
 		Spacing = 0.0f;
 		RoundRadius = 5.0f;
+		FontSize = 20.0f;
 	}
-	else if(m_pClient->m_Snap.m_aTeamSize[Team] > 12)
+	else if(NumPlayers > 12)
 	{
 		LineHeight = 40.0f;
 		TeeSizeMod = 0.8f;
 		Spacing = 0.0f;
 		RoundRadius = 15.0f;
 	}
-	else if(m_pClient->m_Snap.m_aTeamSize[Team] > 8)
+	else if(NumPlayers > 8)
 	{
 		LineHeight = 50.0f;
 		TeeSizeMod = 0.9f;
@@ -285,12 +291,6 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		RoundRadius = 15.0f;
 	}
 
-	float FontSize = 24.0f;
-	if(m_pClient->m_Snap.m_aTeamSize[Team] > 48)
-		FontSize = 16.0f;
-	else if(m_pClient->m_Snap.m_aTeamSize[Team] > 32)
-		FontSize = 20.0f;
-
 	float ScoreOffset = x + 10.0f, ScoreLength = TextRender()->TextWidth(0, FontSize, "00:00:00", -1, -1.0f);
 	float TeeOffset = ScoreOffset + ScoreLength + 15.0f, TeeLength = 60 * TeeSizeMod;
 	float NameOffset = TeeOffset + TeeLength, NameLength = 300.0f - TeeLength;
@@ -421,13 +421,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 			if(pInfo->m_Score == -9999)
 				aBuf[0] = 0;
 			else
-			{
-				int Time = abs(pInfo->m_Score);
-				if(Time / 3600)
-					str_format(aBuf, sizeof(aBuf), "%02d:%02d:%02d", Time / 3600, (Time / 60) % 60, Time % 60);
-				else
-					str_format(aBuf, sizeof(aBuf), "%02d:%02d", Time / 60, Time % 60);
-			}
+				str_time((int64)abs(pInfo->m_Score) * 100, TIME_HOURS, aBuf, sizeof(aBuf));
 		}
 		else
 			str_format(aBuf, sizeof(aBuf), "%d", clamp(pInfo->m_Score, -999, 99999));
@@ -440,10 +434,13 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 			m_pClient->m_Snap.m_pGameDataObj && (m_pClient->m_Snap.m_pGameDataObj->m_FlagCarrierRed == pInfo->m_ClientID || m_pClient->m_Snap.m_pGameDataObj->m_FlagCarrierBlue == pInfo->m_ClientID))
 		{
 			Graphics()->BlendNormal();
-			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
-			Graphics()->QuadsBegin();
+			if(m_pClient->m_Snap.m_pGameDataObj->m_FlagCarrierBlue == pInfo->m_ClientID)
+				Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpriteFlagBlue);
+			else
+				Graphics()->TextureSet(GameClient()->m_GameSkin.m_SpriteFlagRed);
 
-			RenderTools()->SelectSprite(m_pClient->m_Snap.m_pGameDataObj->m_FlagCarrierBlue == pInfo->m_ClientID ? SPRITE_FLAG_BLUE : SPRITE_FLAG_RED, SPRITE_FLAG_FLIP_X);
+			Graphics()->QuadsBegin();
+			Graphics()->QuadsSetSubset(1, 0, 0, 1);
 
 			float Size = LineHeight;
 			IGraphics::CQuadItem QuadItem(TeeOffset + 0.0f, y - 5.0f - Spacing / 2.0f, Size / 2.0f, Size);
@@ -467,7 +464,14 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		if(g_Config.m_ClShowIDs)
 		{
 			char aId[64] = "";
-			str_format(aId, sizeof(aId), "%2d: %s", pInfo->m_ClientID, m_pClient->m_aClients[pInfo->m_ClientID].m_aName);
+			if(pInfo->m_ClientID < 10)
+			{
+				str_format(aId, sizeof(aId), " %d: %s", pInfo->m_ClientID, m_pClient->m_aClients[pInfo->m_ClientID].m_aName);
+			}
+			else
+			{
+				str_format(aId, sizeof(aId), "%d: %s", pInfo->m_ClientID, m_pClient->m_aClients[pInfo->m_ClientID].m_aName);
+			}
 			Cursor.m_LineWidth = NameLength;
 			TextRender()->TextEx(&Cursor, aId, -1);
 		}
@@ -545,30 +549,30 @@ void CScoreboard::RenderRecordingNotification(float x)
 	//draw the text
 	char aBuf[64] = "\0";
 	char aBuf2[64];
-	int Seconds;
+	char aTime[32];
 
 	if(m_pClient->DemoRecorder(RECORDER_MANUAL)->IsRecording())
 	{
-		Seconds = m_pClient->DemoRecorder(RECORDER_MANUAL)->Length();
-		str_format(aBuf2, sizeof(aBuf2), Localize("Manual %3d:%02d  "), Seconds / 60, Seconds % 60);
+		str_time((int64)m_pClient->DemoRecorder(RECORDER_MANUAL)->Length() * 100, TIME_HOURS, aTime, sizeof(aTime));
+		str_format(aBuf2, sizeof(aBuf2), "%s %s  ", Localize("Manual"), aTime);
 		str_append(aBuf, aBuf2, sizeof(aBuf));
 	}
 	if(m_pClient->DemoRecorder(RECORDER_RACE)->IsRecording())
 	{
-		Seconds = m_pClient->DemoRecorder(RECORDER_RACE)->Length();
-		str_format(aBuf2, sizeof(aBuf2), Localize("Race %3d:%02d  "), Seconds / 60, Seconds % 60);
+		str_time((int64)m_pClient->DemoRecorder(RECORDER_RACE)->Length() * 100, TIME_HOURS, aTime, sizeof(aTime));
+		str_format(aBuf2, sizeof(aBuf2), "%s %s  ", Localize("Race"), aTime);
 		str_append(aBuf, aBuf2, sizeof(aBuf));
 	}
 	if(m_pClient->DemoRecorder(RECORDER_AUTO)->IsRecording())
 	{
-		Seconds = m_pClient->DemoRecorder(RECORDER_AUTO)->Length();
-		str_format(aBuf2, sizeof(aBuf2), Localize("Auto %3d:%02d  "), Seconds / 60, Seconds % 60);
+		str_time((int64)m_pClient->DemoRecorder(RECORDER_AUTO)->Length() * 100, TIME_HOURS, aTime, sizeof(aTime));
+		str_format(aBuf2, sizeof(aBuf2), "%s %s  ", Localize("Auto"), aTime);
 		str_append(aBuf, aBuf2, sizeof(aBuf));
 	}
 	if(m_pClient->DemoRecorder(RECORDER_REPLAYS)->IsRecording())
 	{
-		Seconds = m_pClient->DemoRecorder(RECORDER_REPLAYS)->Length();
-		str_format(aBuf2, sizeof(aBuf2), Localize("Replay %3d:%02d  "), Seconds / 60, Seconds % 60);
+		str_time((int64)m_pClient->DemoRecorder(RECORDER_REPLAYS)->Length() * 100, TIME_HOURS, aTime, sizeof(aTime));
+		str_format(aBuf2, sizeof(aBuf2), "%s %s  ", Localize("Replay"), aTime);
 		str_append(aBuf, aBuf2, sizeof(aBuf));
 	}
 
@@ -613,18 +617,18 @@ void CScoreboard::OnRender()
 		{
 			if(m_pClient->m_Snap.m_aTeamSize[0] > 48)
 			{
-				RenderScoreboard(Width / 2 - w, 150.0f, w, -4, 0);
 				RenderScoreboard(Width / 2, 150.0f, w, -5, "");
+				RenderScoreboard(Width / 2 - w, 150.0f, w, -4, 0);
 			}
 			else if(m_pClient->m_Snap.m_aTeamSize[0] > 32)
 			{
-				RenderScoreboard(Width / 2 - w, 150.0f, w, -7, 0);
 				RenderScoreboard(Width / 2, 150.0f, w, -8, "");
+				RenderScoreboard(Width / 2 - w, 150.0f, w, -7, 0);
 			}
 			else if(m_pClient->m_Snap.m_aTeamSize[0] > 16)
 			{
-				RenderScoreboard(Width / 2 - w, 150.0f, w, -6, 0);
 				RenderScoreboard(Width / 2, 150.0f, w, -3, "");
+				RenderScoreboard(Width / 2 - w, 150.0f, w, -6, 0);
 			}
 			else
 			{
@@ -660,8 +664,9 @@ void CScoreboard::OnRender()
 				TextRender()->Text(0, Width / 2 - w / 2, 39, 86.0f, aText, -1.0f);
 			}
 
-			RenderScoreboard(Width / 2 - w - 5.0f, 150.0f, w, TEAM_RED, pRedClanName ? pRedClanName : Localize("Red team"));
-			RenderScoreboard(Width / 2 + 5.0f, 150.0f, w, TEAM_BLUE, pBlueClanName ? pBlueClanName : Localize("Blue team"));
+			int NumPlayers = maximum(m_pClient->m_Snap.m_aTeamSize[TEAM_RED], m_pClient->m_Snap.m_aTeamSize[TEAM_BLUE]);
+			RenderScoreboard(Width / 2 - w - 5.0f, 150.0f, w, TEAM_RED, pRedClanName ? pRedClanName : Localize("Red team"), NumPlayers);
+			RenderScoreboard(Width / 2 + 5.0f, 150.0f, w, TEAM_BLUE, pBlueClanName ? pBlueClanName : Localize("Blue team"), NumPlayers);
 		}
 	}
 
@@ -697,9 +702,8 @@ const char *CScoreboard::GetClanName(int Team)
 {
 	int ClanPlayers = 0;
 	const char *pClanName = 0;
-	for(int i = 0; i < MAX_CLIENTS; i++)
+	for(const auto *pInfo : m_pClient->m_Snap.m_paInfoByScore)
 	{
-		const CNetObj_PlayerInfo *pInfo = m_pClient->m_Snap.m_paInfoByScore[i];
 		if(!pInfo || pInfo->m_Team != Team)
 			continue;
 
