@@ -7,38 +7,37 @@
 #include <base/vmath.h>
 #include <engine/client/http.h>
 #include <game/client/component.h>
+#include <game/client/skin.h>
 
 class CSkins : public CComponent
 {
 public:
-	// do this better and nicer
-	struct CSkin
+	class CGetPngFile : public CGetFile
 	{
-		bool m_IsVanilla;
-		IGraphics::CTextureHandle m_OrgTexture;
-		IGraphics::CTextureHandle m_ColorTexture;
-		char m_aName[24];
-		ColorRGBA m_BloodColor;
+		CSkins *m_pSkins;
 
-		bool operator<(const CSkin &Other) { return str_comp(m_aName, Other.m_aName) < 0; }
+	protected:
+		virtual int OnCompletion(int State);
 
-		bool operator<(const char *pOther) { return str_comp(m_aName, pOther) < 0; }
-		bool operator==(const char *pOther) { return !str_comp(m_aName, pOther); }
+	public:
+		CGetPngFile(CSkins *pSkins, IStorage *pStorage, const char *pUrl, const char *pDest, int StorageType = -2, CTimeout Timeout = CTimeout{4000, 500, 5}, bool LogProgress = true);
+		CImageInfo m_Info;
 	};
 
 	struct CDownloadSkin
 	{
-		std::shared_ptr<CGetFile> m_pTask;
+		std::shared_ptr<CSkins::CGetPngFile> m_pTask;
 		char m_aPath[MAX_PATH_LENGTH];
 		char m_aName[24];
 
-		bool operator<(const CDownloadSkin &Other) { return str_comp(m_aName, Other.m_aName) < 0; }
-		bool operator<(const char *pOther) { return str_comp(m_aName, pOther) < 0; }
-		bool operator==(const char *pOther) { return !str_comp(m_aName, pOther); }
+		bool operator<(const CDownloadSkin &Other) const { return str_comp(m_aName, Other.m_aName) < 0; }
+		bool operator<(const char *pOther) const { return str_comp(m_aName, pOther) < 0; }
+		bool operator==(const char *pOther) const { return !str_comp(m_aName, pOther); }
 	};
 
 	void OnInit();
 
+	void Refresh();
 	int Num();
 	const CSkin *Get(int Index);
 	int Find(const char *pName);
@@ -48,7 +47,9 @@ private:
 	sorted_array<CDownloadSkin> m_aDownloadSkins;
 	char m_EventSkinPrefix[24];
 
+	bool LoadSkinPNG(CImageInfo &Info, const char *pName, const char *pPath, int DirType);
 	int LoadSkin(const char *pName, const char *pPath, int DirType);
+	int LoadSkin(const char *pName, CImageInfo &Info);
 	int FindImpl(const char *pName);
 	static int SkinScan(const char *pName, int IsDir, int DirType, void *pUser);
 };
