@@ -4,6 +4,7 @@
 
 #include "voting.h"
 #include <base/vmath.h>
+#include <game/client/components/sounds.h>
 #include <game/client/render.h>
 #include <game/generated/protocol.h>
 
@@ -188,15 +189,21 @@ void CVoting::OnMessage(int MsgType, void *pRawMsg)
 	if(MsgType == NETMSGTYPE_SV_VOTESET)
 	{
 		CNetMsg_Sv_VoteSet *pMsg = (CNetMsg_Sv_VoteSet *)pRawMsg;
+		OnReset();
 		if(pMsg->m_Timeout)
 		{
-			OnReset();
 			str_copy(m_aDescription, pMsg->m_pDescription, sizeof(m_aDescription));
 			str_copy(m_aReason, pMsg->m_pReason, sizeof(m_aReason));
 			m_Closetime = time() + time_freq() * pMsg->m_Timeout;
+
+			if(Client()->RconAuthed())
+			{
+				char aBuf[512];
+				str_format(aBuf, sizeof(aBuf), "%s (%s)", m_aDescription, m_aReason);
+				Client()->Notify("DDNet Vote", aBuf);
+				m_pClient->m_pSounds->Play(CSounds::CHN_GUI, SOUND_CHAT_HIGHLIGHT, 0);
+			}
 		}
-		else
-			OnReset();
 	}
 	else if(MsgType == NETMSGTYPE_SV_VOTESTATUS)
 	{
