@@ -49,6 +49,7 @@ public:
 	bool m_EntitiesFNG;
 	bool m_EntitiesVanilla;
 	bool m_EntitiesBW;
+	bool m_EntitiesFDDrace;
 
 	bool m_Race;
 
@@ -83,6 +84,7 @@ class CGameClient : public IGameClient
 	class ITextRender *m_pTextRender;
 	class IClient *m_pClient;
 	class ISound *m_pSound;
+	class CConfig *m_pConfig;
 	class IConsole *m_pConsole;
 	class IStorage *m_pStorage;
 	class IDemoPlayer *m_pDemoPlayer;
@@ -102,14 +104,14 @@ class CGameClient : public IGameClient
 	void UpdatePositions();
 
 	int m_PredictedTick;
-	int m_LastNewPredictedTick[2];
+	int m_LastNewPredictedTick[NUM_DUMMIES];
 
 	int m_LastRoundStartTick;
 
 	int m_LastFlagCarrierRed;
 	int m_LastFlagCarrierBlue;
 
-	int m_CheckInfo[2];
+	int m_CheckInfo[NUM_DUMMIES];
 
 	char m_aDDNetVersionStr[64];
 
@@ -134,6 +136,7 @@ public:
 	class ISound *Sound() const { return m_pSound; }
 	class IInput *Input() const { return m_pInput; }
 	class IStorage *Storage() const { return m_pStorage; }
+	class CConfig *Config() const { return m_pConfig; }
 	class IConsole *Console() { return m_pConsole; }
 	class ITextRender *TextRender() const { return m_pTextRender; }
 	class IDemoPlayer *DemoPlayer() const { return m_pDemoPlayer; }
@@ -164,7 +167,7 @@ public:
 	int m_FlagDropTick[2];
 
 	// TODO: move this
-	CTuningParams m_Tuning[2];
+	CTuningParams m_Tuning[NUM_DUMMIES];
 
 	enum
 	{
@@ -247,7 +250,6 @@ public:
 		char m_aClan[MAX_CLAN_LENGTH];
 		int m_Country;
 		char m_aSkinName[64];
-		int m_SkinID;
 		int m_SkinColor;
 		int m_Team;
 		int m_Emoticon;
@@ -379,6 +381,8 @@ public:
 	void OnWindowResize();
 	static void OnWindowResizeCB(void *pUser);
 
+	void OnLanguageChange();
+
 	virtual const char *GetItemName(int Type);
 	virtual const char *Version();
 	virtual const char *NetVersion();
@@ -422,7 +426,7 @@ public:
 
 	// DDRace
 
-	int m_LocalIDs[2];
+	int m_LocalIDs[NUM_DUMMIES];
 	CNetObj_PlayerInput m_DummyInput;
 	CNetObj_PlayerInput m_HammerInput;
 	int m_DummyFire;
@@ -441,7 +445,7 @@ public:
 	bool AntiPingWeapons() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingWeapons && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK; }
 	bool AntiPingGunfire() { return AntiPingGrenade() && AntiPingWeapons() && g_Config.m_ClAntiPingGunfire; }
 	bool Predict() { return g_Config.m_ClPredict && !(m_Snap.m_pGameInfoObj && m_Snap.m_pGameInfoObj->m_GameStateFlags & GAMESTATEFLAG_GAMEOVER) && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK && m_Snap.m_pLocalCharacter; }
-	bool PredictDummy() { return g_Config.m_ClPredictDummy && Client()->DummyConnected() && m_Snap.m_LocalClientID >= 0 && m_PredictedDummyID >= 0; }
+	bool PredictDummy() { return g_Config.m_ClPredictDummy && Client()->DummyConnected() && m_Snap.m_LocalClientID >= 0 && m_PredictedDummyID >= 0 && !m_aClients[m_PredictedDummyID].m_Paused; }
 
 	CGameWorld m_GameWorld;
 	CGameWorld m_PredictedWorld;
@@ -456,9 +460,119 @@ public:
 	void LoadEmoticonsSkin(const char *pPath, bool AsDir = false);
 	void LoadParticlesSkin(const char *pPath, bool AsDir = false);
 
+	void RefindSkins();
+
+	struct SClientGameSkin
+	{
+		// health armor hud
+		IGraphics::CTextureHandle m_SpriteHealthFull;
+		IGraphics::CTextureHandle m_SpriteHealthEmpty;
+		IGraphics::CTextureHandle m_SpriteArmorFull;
+		IGraphics::CTextureHandle m_SpriteArmorEmpty;
+
+		// cursors
+		IGraphics::CTextureHandle m_SpriteWeaponHammerCursor;
+		IGraphics::CTextureHandle m_SpriteWeaponGunCursor;
+		IGraphics::CTextureHandle m_SpriteWeaponShotgunCursor;
+		IGraphics::CTextureHandle m_SpriteWeaponGrenadeCursor;
+		IGraphics::CTextureHandle m_SpriteWeaponNinjaCursor;
+		IGraphics::CTextureHandle m_SpriteWeaponLaserCursor;
+
+		IGraphics::CTextureHandle m_SpriteWeaponCursors[6];
+
+		// weapons and hook
+		IGraphics::CTextureHandle m_SpriteHookChain;
+		IGraphics::CTextureHandle m_SpriteHookHead;
+		IGraphics::CTextureHandle m_SpriteWeaponHammer;
+		IGraphics::CTextureHandle m_SpriteWeaponGun;
+		IGraphics::CTextureHandle m_SpriteWeaponShotgun;
+		IGraphics::CTextureHandle m_SpriteWeaponGrenade;
+		IGraphics::CTextureHandle m_SpriteWeaponNinja;
+		IGraphics::CTextureHandle m_SpriteWeaponLaser;
+
+		IGraphics::CTextureHandle m_SpriteWeapons[6];
+
+		// particles
+		IGraphics::CTextureHandle m_SpriteParticles[9];
+
+		// stars
+		IGraphics::CTextureHandle m_SpriteStars[3];
+
+		// projectiles
+		IGraphics::CTextureHandle m_SpriteWeaponGunProjectile;
+		IGraphics::CTextureHandle m_SpriteWeaponShotgunProjectile;
+		IGraphics::CTextureHandle m_SpriteWeaponGrenadeProjectile;
+		IGraphics::CTextureHandle m_SpriteWeaponHammerProjectile;
+		IGraphics::CTextureHandle m_SpriteWeaponNinjaProjectile;
+		IGraphics::CTextureHandle m_SpriteWeaponLaserProjectile;
+
+		IGraphics::CTextureHandle m_SpriteWeaponProjectiles[6];
+
+		// muzzles
+		IGraphics::CTextureHandle m_SpriteWeaponGunMuzzles[3];
+		IGraphics::CTextureHandle m_SpriteWeaponShotgunMuzzles[3];
+		IGraphics::CTextureHandle m_SpriteWeaponNinjaMuzzles[3];
+
+		IGraphics::CTextureHandle m_SpriteWeaponsMuzzles[6][3];
+
+		// pickups
+		IGraphics::CTextureHandle m_SpritePickupHealth;
+		IGraphics::CTextureHandle m_SpritePickupArmor;
+		IGraphics::CTextureHandle m_SpritePickupGrenade;
+		IGraphics::CTextureHandle m_SpritePickupShotgun;
+		IGraphics::CTextureHandle m_SpritePickupLaser;
+		IGraphics::CTextureHandle m_SpritePickupNinja;
+		IGraphics::CTextureHandle m_SpritePickupGun;
+		IGraphics::CTextureHandle m_SpritePickupHammer;
+
+		IGraphics::CTextureHandle m_SpritePickupWeapons[6];
+
+		// flags
+		IGraphics::CTextureHandle m_SpriteFlagBlue;
+		IGraphics::CTextureHandle m_SpriteFlagRed;
+
+		// ninja bar (0.7)
+		IGraphics::CTextureHandle m_SpriteNinjaBarFullLeft;
+		IGraphics::CTextureHandle m_SpriteNinjaBarFull;
+		IGraphics::CTextureHandle m_SpriteNinjaBarEmpty;
+		IGraphics::CTextureHandle m_SpriteNinjaBarEmptyRight;
+
+		bool IsSixup()
+		{
+			return m_SpriteNinjaBarFullLeft != -1;
+		}
+	};
+
+	SClientGameSkin m_GameSkin;
+	bool m_GameSkinLoaded;
+
+	struct SClientParticlesSkin
+	{
+		IGraphics::CTextureHandle m_SpriteParticleSlice;
+		IGraphics::CTextureHandle m_SpriteParticleBall;
+		IGraphics::CTextureHandle m_SpriteParticleSplat[3];
+		IGraphics::CTextureHandle m_SpriteParticleSmoke;
+		IGraphics::CTextureHandle m_SpriteParticleShell;
+		IGraphics::CTextureHandle m_SpriteParticleExpl;
+		IGraphics::CTextureHandle m_SpriteParticleAirJump;
+		IGraphics::CTextureHandle m_SpriteParticleHit;
+		IGraphics::CTextureHandle m_SpriteParticles[10];
+	};
+
+	SClientParticlesSkin m_ParticlesSkin;
+	bool m_ParticlesSkinLoaded;
+
+	struct SClientEmoticonsSkin
+	{
+		IGraphics::CTextureHandle m_SpriteEmoticons[16];
+	};
+
+	SClientEmoticonsSkin m_EmoticonsSkin;
+	bool m_EmoticonsSkinLoaded;
+
 private:
-	bool m_DDRaceMsgSent[2];
-	int m_ShowOthers[2];
+	bool m_DDRaceMsgSent[NUM_DUMMIES];
+	int m_ShowOthers[NUM_DUMMIES];
 
 	void UpdatePrediction();
 	void UpdateRenderedCharacters();
@@ -477,6 +591,10 @@ private:
 	void LoadMapSettings();
 	CTuningParams m_aTuningList[NUM_TUNEZONES];
 	CTuningParams *TuningList() { return m_aTuningList; }
+
+	float m_LastZoom;
+	float m_LastScreenAspect;
+	float m_LastDummyConnected;
 };
 
 ColorRGBA CalculateNameColor(ColorHSLA TextColorHSL);
