@@ -390,7 +390,7 @@ void CCharacter::FireWeapon()
 			for(int i = -ShotSpread; i <= ShotSpread; ++i)
 			{
 				float Spreading[] = {-0.185f, -0.070f, 0, 0.070f, 0.185f};
-				float a = GetAngle(Direction);
+				float a = angle(Direction);
 				a += Spreading[i + 2];
 				float v = 1 - (absolute(i) / (float)ShotSpread);
 				float Speed = mix((float)Tuning()->m_ShotgunSpeeddiff, 1.0f, v);
@@ -613,7 +613,7 @@ void CCharacter::HandleSkippableTiles(int Index)
 	// handle speedup tiles
 	if(Collision()->IsSpeedup(Index))
 	{
-		vec2 Direction, MaxVel, TempVel = m_Core.m_Vel;
+		vec2 Direction, TempVel = m_Core.m_Vel;
 		int Force, MaxSpeed = 0;
 		float TeeAngle, SpeederAngle, DiffAngle, SpeedLeft, TeeSpeed;
 		Collision()->GetSpeedup(Index, &Direction, &Force, &MaxSpeed);
@@ -917,8 +917,8 @@ void CCharacter::DDRacePostCoreTick()
 	// handle Anti-Skip tiles
 	std::list<int> Indices = Collision()->GetMapIndices(m_PrevPos, m_Pos);
 	if(!Indices.empty())
-		for(std::list<int>::iterator i = Indices.begin(); i != Indices.end(); i++)
-			HandleTiles(*i);
+		for(int Index : Indices)
+			HandleTiles(Index);
 	else
 	{
 		HandleTiles(CurrentIndex);
@@ -1023,6 +1023,12 @@ CCharacter::CCharacter(CGameWorld *pGameWorld, int ID, CNetObj_Character *pChar,
 	m_TeleCheckpoint = 0;
 	m_StrongWeakID = 0;
 
+	// never intilize both to zero
+	m_Input.m_TargetX = 0;
+	m_Input.m_TargetY = -1;
+
+	m_LatestPrevInput = m_LatestInput = m_PrevInput = m_SavedInput = m_Input;
+
 	ResetPrediction();
 	Read(pChar, pExtended, false);
 
@@ -1221,8 +1227,8 @@ void CCharacter::Read(CNetObj_Character *pChar, CNetObj_DDNetCharacter *pExtende
 		mem_zero(&m_SavedInput, sizeof(m_SavedInput));
 		m_Input.m_Direction = m_SavedInput.m_Direction = m_Core.m_Direction;
 		m_Input.m_Hook = m_SavedInput.m_Hook = (m_Core.m_HookState != HOOK_IDLE);
-		m_Input.m_TargetX = cosf(pChar->m_Angle / 256.0f);
-		m_Input.m_TargetY = sinf(pChar->m_Angle / 256.0f);
+		m_Input.m_TargetX = m_SavedInput.m_TargetX = cosf(pChar->m_Angle / 256.0f) * 256.0f;
+		m_Input.m_TargetY = m_SavedInput.m_TargetY = sinf(pChar->m_Angle / 256.0f) * 256.0f;
 	}
 
 	// in most cases the reload timer can be determined from the last attack tick
