@@ -104,7 +104,7 @@ void CProjectile::Tick()
 			GameWorld()->CreateExplosion(ColPos, m_Owner, m_Type, m_Owner == -1, (!pTargetChr ? -1 : pTargetChr->Team()),
 				(m_Owner != -1) ? TeamMask : -1LL);
 		}
-		else if(pTargetChr && m_Freeze && ((m_Layer == LAYER_SWITCH && Collision()->m_pSwitchers[m_Number].m_Status[pTargetChr->Team()]) || m_Layer != LAYER_SWITCH))
+		else if(pTargetChr && m_Freeze && ((m_Layer == LAYER_SWITCH && m_Number > 0 && Collision()->m_pSwitchers[m_Number].m_Status[pTargetChr->Team()]) || m_Layer != LAYER_SWITCH))
 			pTargetChr->Freeze();
 		if(Collide && m_Bouncing != 0)
 		{
@@ -234,9 +234,3 @@ bool CProjectile::Match(CProjectile *pProj)
 		return false;
 	return true;
 }
-
-int CProjectile::NetworkClipped(vec2 ViewPos)
-{
-	float Ct = (GameWorld()->GameTick() - m_StartTick) / (float)GameWorld()->GameTickSpeed();
-	return ((CEntity *)this)->NetworkClipped(GetPos(Ct), ViewPos);
-}
