@@ -33,24 +33,6 @@ CEntity::~CEntity()
 		GameWorld()->RemoveEntity(this);
 }
 
-int CEntity::NetworkClipped(vec2 ViewPos)
-{
-	return NetworkClipped(m_Pos, ViewPos);
-}
-
-int CEntity::NetworkClipped(vec2 CheckPos, vec2 ViewPos)
-{
-	float dx = ViewPos.x - CheckPos.x;
-	float dy = ViewPos.y - CheckPos.y;
-
-	if(absolute(dx) > 1000.0f || absolute(dy) > 800.0f)
-		return 1;
-
-	if(distance(ViewPos, CheckPos) > 4000.0f)
-		return 1;
-	return 0;
-}
-
 bool CEntity::GameLayerClipped(vec2 CheckPos)
 {
 	return round_to_int(CheckPos.x) / 32 < -200 || round_to_int(CheckPos.x) / 32 > Collision()->GetWidth() + 200 ||
