@@ -13,6 +13,7 @@ class CCharacter;
 class CGameWorld
 {
 	friend class CCharacter;
+
 public:
 	enum
 	{
@@ -59,7 +60,7 @@ public:
 	class CCharacter *GetCharacterByID(int ID) { return (ID >= 0 && ID < MAX_CLIENTS) ? m_apCharacters[ID] : 0; }
 
 	// from gamecontext
-	void CreateExplosion(vec2 Pos, int Owner, int Weapon, bool NoDamage, int ActivatedTeam, int64_t Mask);
+	void CreateExplosion(vec2 Pos, int Owner, int Weapon, bool NoDamage, int ActivatedTeam, int64 Mask);
 
 	// for client side prediction
 	struct
