@@ -9,7 +9,10 @@
 #include <engine/graphics.h>
 #include <engine/map.h>
 #include <engine/shared/config.h>
+#include <game/client/components/skins.h>
+#include <game/client/gameclient.h>
 #include <game/generated/client_data.h>
+#include <game/generated/client_data7.h>
 #include <game/generated/protocol.h>
 #include <game/layers.h>
 
@@ -36,32 +39,33 @@ static void layershot_end()
 	config.cl_layershot++;
 }*/
 
-void CRenderTools::Init(IGraphics *pGraphics, CUI *pUI)
+void CRenderTools::Init(IGraphics *pGraphics, CUI *pUI, CGameClient *pGameClient)
 {
 	m_pGraphics = pGraphics;
 	m_pUI = pUI;
+	m_pGameClient = (CGameClient *)pGameClient;
 	m_TeeQuadContainerIndex = Graphics()->CreateQuadContainer();
 	Graphics()->SetColor(1.f, 1.f, 1.f, 1.f);
 
-	SelectSprite(SPRITE_TEE_BODY, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f, false);
-	SelectSprite(SPRITE_TEE_BODY_OUTLINE, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f, false);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f);
 
-	SelectSprite(SPRITE_TEE_EYE_PAIN, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f, false);
-	SelectSprite(SPRITE_TEE_EYE_HAPPY, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f, false);
-	SelectSprite(SPRITE_TEE_EYE_SURPRISE, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f, false);
-	SelectSprite(SPRITE_TEE_EYE_ANGRY, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f, false);
-	SelectSprite(SPRITE_TEE_EYE_NORMAL, 0, 0, 0);
-	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f, false);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
+	QuadContainerAddSprite(m_TeeQuadContainerIndex, 64.f * 0.4f);
 
-	SelectSprite(SPRITE_TEE_FOOT_OUTLINE, 0, 0, 0);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
 	QuadContainerAddSprite(m_TeeQuadContainerIndex, -32.f, -16.f, 64.f, 32.f);
-	SelectSprite(SPRITE_TEE_FOOT, 0, 0, 0);
+	Graphics()->QuadsSetSubset(0, 0, 1, 1);
 	QuadContainerAddSprite(m_TeeQuadContainerIndex, -32.f, -16.f, 64.f, 32.f);
 }
 
@@ -74,9 +78,7 @@ void CRenderTools::SelectSprite(CDataSprite *pSpr, int Flags, int sx, int sy)
 	int cx = pSpr->m_pSet->m_Gridx;
 	int cy = pSpr->m_pSet->m_Gridy;
 
-	float f = sqrtf(h * h + w * w);
-	gs_SpriteWScale = w / f;
-	gs_SpriteHScale = h / f;
+	GetSpriteScaleImpl(w, h, gs_SpriteWScale, gs_SpriteHScale);
 
 	float x1 = x / (float)cx + 0.5f / (float)(cx * 32);
 	float x2 = (x + w) / (float)cx - 0.5f / (float)(cx * 32);
@@ -108,38 +110,60 @@ void CRenderTools::SelectSprite(int Id, int Flags, int sx, int sy)
 	SelectSprite(&g_pData->m_aSprites[Id], Flags, sx, sy);
 }
 
+void CRenderTools::GetSpriteScale(client_data7::CDataSprite *pSprite, float &ScaleX, float &ScaleY)
+{
+	int w = pSprite->m_W;
+	int h = pSprite->m_H;
+	GetSpriteScaleImpl(w, h, ScaleX, ScaleY);
+}
+
+void CRenderTools::GetSpriteScale(struct CDataSprite *pSprite, float &ScaleX, float &ScaleY)
+{
+	int w = pSprite->m_W;
+	int h = pSprite->m_H;
+	GetSpriteScaleImpl(w, h, ScaleX, ScaleY);
+}
+
+void CRenderTools::GetSpriteScale(int id, float &ScaleX, float &ScaleY)
+{
+	GetSpriteScale(&g_pData->m_aSprites[id], ScaleX, ScaleY);
+}
+
+void CRenderTools::GetSpriteScaleImpl(int Width, int Height, float &ScaleX, float &ScaleY)
+{
+	float f = sqrtf(Height * Height + Width * Width);
+	ScaleX = Width / f;
+	ScaleY = Height / f;
+}
+
 void CRenderTools::DrawSprite(float x, float y, float Size)
 {
 	IGraphics::CQuadItem QuadItem(x, y, Size * gs_SpriteWScale, Size * gs_SpriteHScale);
 	Graphics()->QuadsDraw(&QuadItem, 1);
 }
 
-void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float x, float y, float Size, bool DoSpriteScale)
+void CRenderTools::DrawSprite(float x, float y, float ScaledWidth, float ScaledHeight)
 {
-	if(DoSpriteScale)
-	{
-		IGraphics::CQuadItem QuadItem(x, y, Size * gs_SpriteWScale, Size * gs_SpriteHScale);
-		Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
-	}
-	else
-	{
-		IGraphics::CQuadItem QuadItem(x, y, Size, Size);
-		Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
-	}
+	IGraphics::CQuadItem QuadItem(x, y, ScaledWidth, ScaledHeight);
+	Graphics()->QuadsDraw(&QuadItem, 1);
 }
 
-void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float Size, bool DoSpriteScale)
+void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float x, float y, float Size)
 {
-	if(DoSpriteScale)
-	{
-		IGraphics::CQuadItem QuadItem(-(Size * gs_SpriteWScale) / 2.f, -(Size * gs_SpriteHScale) / 2.f, (Size * gs_SpriteWScale), (Size * gs_SpriteHScale));
-		Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
-	}
-	else
-	{
-		IGraphics::CQuadItem QuadItem(-(Size) / 2.f, -(Size) / 2.f, (Size), (Size));
-		Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
-	}
+	IGraphics::CQuadItem QuadItem(x, y, Size, Size);
+	Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
+}
+
+void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float Size)
+{
+	IGraphics::CQuadItem QuadItem(-(Size) / 2.f, -(Size) / 2.f, (Size), (Size));
+	Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
+}
+
+void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float Width, float Height)
+{
+	IGraphics::CQuadItem QuadItem(-(Width) / 2.f, -(Height) / 2.f, (Width), (Height));
+	Graphics()->QuadContainerAddQuads(QuadContainerIndex, &QuadItem, 1);
 }
 
 void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float X, float Y, float Width, float Height)
@@ -150,9 +174,11 @@ void CRenderTools::QuadContainerAddSprite(int QuadContainerIndex, float X, float
 
 void CRenderTools::DrawRoundRectExt(float x, float y, float w, float h, float r, int Corners)
 {
-	IGraphics::CFreeformItem ArrayF[32];
 	int NumItems = 0;
-	int Num = 8;
+	const int Num = 8;
+
+	IGraphics::CFreeformItem ArrayF[Num * 4];
+
 	for(int i = 0; i < Num; i += 2)
 	{
 		float a1 = i / (float)Num * pi / 2;
@@ -432,6 +458,31 @@ int CRenderTools::CreateRoundRectQuadContainer(float x, float y, float w, float
 	return ContainerIndex;
 }
 
+void CRenderTools::DrawUIElRect(CUIElement::SUIElementRect &ElUIRect, const CUIRect *pRect, ColorRGBA Color, int Corners, float Rounding)
+{
+	bool NeedsRecreate = false;
+	if(ElUIRect.m_UIRectQuadContainer == -1 || ElUIRect.m_X != pRect->x || ElUIRect.m_Y != pRect->y || ElUIRect.m_Width != pRect->w || ElUIRect.m_Height != pRect->h)
+	{
+		if(ElUIRect.m_UIRectQuadContainer != -1)
+			Graphics()->DeleteQuadContainer(ElUIRect.m_UIRectQuadContainer);
+		NeedsRecreate = true;
+	}
+	if(NeedsRecreate)
+	{
+		ElUIRect.m_X = pRect->x;
+		ElUIRect.m_Y = pRect->y;
+		ElUIRect.m_Width = pRect->w;
+		ElUIRect.m_Height = pRect->h;
+
+		Graphics()->SetColor(Color);
+		ElUIRect.m_UIRectQuadContainer = CreateRoundRectQuadContainer(pRect->x, pRect->y, pRect->w, pRect->h, Rounding, Corners);
+		Graphics()->SetColor(1, 1, 1, 1);
+	}
+
+	Graphics()->TextureClear();
+	Graphics()->RenderQuadContainer(ElUIRect.m_UIRectQuadContainer, -1);
+}
+
 void CRenderTools::DrawRoundRect(float x, float y, float w, float h, float r)
 {
 	DrawRoundRectExt(x, y, w, h, r, 0xf);
@@ -457,6 +508,19 @@ void CRenderTools::DrawUIRect4(const CUIRect *r, vec4 ColorTopLeft, vec4 ColorTo
 	Graphics()->QuadsEnd();
 }
 
+void CRenderTools::DrawUIRect4NoRounding(const CUIRect *r, vec4 ColorTopLeft, vec4 ColorTopRight, vec4 ColorBottomLeft, vec4 ColorBottomRight)
+{
+	Graphics()->TextureClear();
+
+	Graphics()->QuadsBegin();
+
+	Graphics()->SetColor4(ColorTopLeft, ColorTopRight, ColorBottomLeft, ColorBottomRight);
+	IGraphics::CQuadItem ItemQ = IGraphics::CQuadItem(r->x, r->y, r->w, r->h);
+	Graphics()->QuadsDrawTL(&ItemQ, 1);
+
+	Graphics()->QuadsEnd();
+}
+
 void CRenderTools::DrawCircle(float x, float y, float r, int Segments)
 {
 	IGraphics::CFreeformItem Array[32];
@@ -489,13 +553,106 @@ void CRenderTools::DrawCircle(float x, float y, float r, int Segments)
 		Graphics()->QuadsDrawFreeform(Array, NumItems);
 }
 
+void CRenderTools::GetRenderTeeAnimScaleAndBaseSize(CAnimState *pAnim, CTeeRenderInfo *pInfo, float &AnimScale, float &BaseSize)
+{
+	AnimScale = pInfo->m_Size * 1.0f / 64.0f;
+	BaseSize = pInfo->m_Size;
+}
+
+void CRenderTools::GetRenderTeeBodyScale(float BaseSize, float &BodyScale)
+{
+	BodyScale = g_Config.m_ClFatSkins ? BaseSize * 1.3f : BaseSize;
+	BodyScale /= 64.0f;
+}
+
+void CRenderTools::GetRenderTeeFeetScale(float BaseSize, float &FeetScaleWidth, float &FeetScaleHeight)
+{
+	FeetScaleWidth = BaseSize / 64.0f;
+	FeetScaleHeight = (BaseSize / 2) / 32.0f;
+}
+
+void CRenderTools::GetRenderTeeBodySize(CAnimState *pAnim, CTeeRenderInfo *pInfo, vec2 &BodyOffset, float &Width, float &Height)
+{
+	float AnimScale, BaseSize;
+	GetRenderTeeAnimScaleAndBaseSize(pAnim, pInfo, AnimScale, BaseSize);
+
+	float BodyScale;
+	GetRenderTeeBodyScale(BaseSize, BodyScale);
+
+	Width = pInfo->m_SkinMetrics.m_Body.WidthNormalized() * 64.0f * BodyScale;
+	Height = pInfo->m_SkinMetrics.m_Body.HeightNormalized() * 64.0f * BodyScale;
+	BodyOffset.x = pInfo->m_SkinMetrics.m_Body.OffsetXNormalized() * 64.0f * BodyScale;
+	BodyOffset.y = pInfo->m_SkinMetrics.m_Body.OffsetYNormalized() * 64.0f * BodyScale;
+}
+
+void CRenderTools::GetRenderTeeFeetSize(CAnimState *pAnim, CTeeRenderInfo *pInfo, vec2 &FeetOffset, float &Width, float &Height)
+{
+	float AnimScale, BaseSize;
+	GetRenderTeeAnimScaleAndBaseSize(pAnim, pInfo, AnimScale, BaseSize);
+
+	float FeetScaleWidth, FeetScaleHeight;
+	GetRenderTeeFeetScale(BaseSize, FeetScaleWidth, FeetScaleHeight);
+
+	Width = pInfo->m_SkinMetrics.m_Feet.WidthNormalized() * 64.0f * FeetScaleWidth;
+	Height = pInfo->m_SkinMetrics.m_Feet.HeightNormalized() * 32.0f * FeetScaleHeight;
+	FeetOffset.x = pInfo->m_SkinMetrics.m_Feet.OffsetXNormalized() * 64.0f * FeetScaleWidth;
+	FeetOffset.y = pInfo->m_SkinMetrics.m_Feet.OffsetYNormalized() * 32.0f * FeetScaleHeight;
+}
+
+void CRenderTools::GetRenderTeeOffsetToRenderedTee(CAnimState *pAnim, CTeeRenderInfo *pInfo, vec2 &TeeOffsetToMid)
+{
+	float AnimScale, BaseSize;
+	GetRenderTeeAnimScaleAndBaseSize(pAnim, pInfo, AnimScale, BaseSize);
+	vec2 BodyPos = vec2(pAnim->GetBody()->m_X, pAnim->GetBody()->m_Y) * AnimScale;
+
+	float AssumedScale = BaseSize / 64.0f;
+
+	// just use the lowest feet
+	vec2 FeetPos;
+	CAnimKeyframe *pFoot = pAnim->GetFrontFoot();
+	FeetPos = vec2(pFoot->m_X * AnimScale, pFoot->m_Y * AnimScale);
+	pFoot = pAnim->GetBackFoot();
+	FeetPos = vec2(FeetPos.x, maximum(FeetPos.y, pFoot->m_Y * AnimScale));
+
+	vec2 BodyOffset;
+	float BodyWidth, BodyHeight;
+	GetRenderTeeBodySize(pAnim, pInfo, BodyOffset, BodyWidth, BodyHeight);
+
+	// -32 is the assumed min relative position for the quad
+	float MinY = -32.0f * AssumedScale;
+	// the body pos shifts the body away from center
+	MinY += BodyPos.y;
+	// the actual body is smaller tho, bcs it doesnt use the full skin image in most cases
+	MinY += BodyOffset.y;
+
+	vec2 FeetOffset;
+	float FeetWidth, FeetHeight;
+	GetRenderTeeFeetSize(pAnim, pInfo, FeetOffset, FeetWidth, FeetHeight);
+
+	// MaxY builds up from the MinY
+	float MaxY = MinY + BodyHeight;
+	// if the body is smaller than the total feet offset, use feet
+	// since feets are smaller in height, respect the assumed relative position
+	MaxY = maximum(MaxY, (-16.0f * AssumedScale + FeetPos.y) + FeetOffset.y + FeetHeight);
+
+	// now we got the full rendered size
+	float FullHeight = (MaxY - MinY);
+
+	// next step is to calculate the offset that was created compared to the assumed relative positon
+	float MidOfRendered = MinY + FullHeight / 2.0f;
+
+	// TODO: x coordinate is ignored for now, bcs it's not really used yet anyway
+	TeeOffsetToMid.x = 0;
+	// negative value, because the calculation that uses this offset should work with addition.
+	TeeOffsetToMid.y = -MidOfRendered;
+}
+
 void CRenderTools::RenderTee(CAnimState *pAnim, CTeeRenderInfo *pInfo, int Emote, vec2 Dir, vec2 Pos, float Alpha)
 {
 	vec2 Direction = Dir;
 	vec2 Position = Pos;
 
-	//Graphics()->TextureSet(data->images[IMAGE_CHAR_DEFAULT].id);
-	Graphics()->TextureSet(pInfo->m_Texture);
+	const CSkin::SSkinTextures *pSkinTextures = pInfo->m_CustomColoredSkin ? &pInfo->m_ColorableRenderSkin : &pInfo->m_OriginalRenderSkin;
 
 	// first pass we draw the outline
 	// second pass we draw the filling
@@ -505,8 +662,8 @@ void CRenderTools::RenderTee(CAnimState *pAnim, CTeeRenderInfo *pInfo, int Emote
 
 		for(int f = 0; f < 2; f++)
 		{
-			float AnimScale = pInfo->m_Size * 1.0f / 64.0f;
-			float BaseSize = pInfo->m_Size;
+			float AnimScale, BaseSize;
+			GetRenderTeeAnimScaleAndBaseSize(pAnim, pInfo, AnimScale, BaseSize);
 			if(f == 1)
 			{
 				Graphics()->QuadsSetRotation(pAnim->GetBody()->m_Angle * pi * 2);
@@ -514,27 +671,34 @@ void CRenderTools::RenderTee(CAnimState *pAnim, CTeeRenderInfo *pInfo, int Emote
 				// draw body
 				Graphics()->SetColor(pInfo->m_ColorBody.r, pInfo->m_ColorBody.g, pInfo->m_ColorBody.b, Alpha);
 				vec2 BodyPos = Position + vec2(pAnim->GetBody()->m_X, pAnim->GetBody()->m_Y) * AnimScale;
-				float BodySize = g_Config.m_ClFatSkins ? BaseSize * 1.3f : BaseSize;
-				Graphics()->RenderQuadContainerAsSprite(m_TeeQuadContainerIndex, OutLine, BodyPos.x, BodyPos.y, BodySize / 64.f, BodySize / 64.f);
+				float BodyScale;
+				GetRenderTeeBodyScale(BaseSize, BodyScale);
+				Graphics()->TextureSet(OutLine == 1 ? pSkinTextures->m_BodyOutline : pSkinTextures->m_Body);
+				Graphics()->RenderQuadContainerAsSprite(m_TeeQuadContainerIndex, OutLine, BodyPos.x, BodyPos.y, BodyScale, BodyScale);
 
 				// draw eyes
 				if(p == 1)
 				{
 					int QuadOffset = 2;
 					int EyeQuadOffset = 0;
+					int TeeEye = 0;
 					switch(Emote)
 					{
 					case EMOTE_PAIN:
 						EyeQuadOffset = 0;
+						TeeEye = SPRITE_TEE_EYE_PAIN - SPRITE_TEE_EYE_NORMAL;
 						break;
 					case EMOTE_HAPPY:
 						EyeQuadOffset = 1;
+						TeeEye = SPRITE_TEE_EYE_HAPPY - SPRITE_TEE_EYE_NORMAL;
 						break;
 					case EMOTE_SURPRISE:
 						EyeQuadOffset = 2;
+						TeeEye = SPRITE_TEE_EYE_SURPRISE - SPRITE_TEE_EYE_NORMAL;
 						break;
 					case EMOTE_ANGRY:
 						EyeQuadOffset = 3;
+						TeeEye = SPRITE_TEE_EYE_ANGRY - SPRITE_TEE_EYE_NORMAL;
 						break;
 					default:
 						EyeQuadOffset = 4;
@@ -546,6 +710,7 @@ void CRenderTools::RenderTee(CAnimState *pAnim, CTeeRenderInfo *pInfo, int Emote
 					float EyeSeparation = (0.075f - 0.010f * absolute(Direction.x)) * BaseSize;
 					vec2 Offset = vec2(Direction.x * 0.125f, -0.05f + Direction.y * 0.10f) * BaseSize;
 
+					Graphics()->TextureSet(pSkinTextures->m_Eyes[TeeEye]);
 					Graphics()->RenderQuadContainerAsSprite(m_TeeQuadContainerIndex, QuadOffset + EyeQuadOffset, BodyPos.x - EyeSeparation + Offset.x, BodyPos.y + Offset.y, EyeScale / (64.f * 0.4f), h / (64.f * 0.4f));
 					Graphics()->RenderQuadContainerAsSprite(m_TeeQuadContainerIndex, QuadOffset + EyeQuadOffset, BodyPos.x + EyeSeparation + Offset.x, BodyPos.y + Offset.y, -EyeScale / (64.f * 0.4f), h / (64.f * 0.4f));
 				}
@@ -573,6 +738,7 @@ void CRenderTools::RenderTee(CAnimState *pAnim, CTeeRenderInfo *pInfo, int Emote
 
 			Graphics()->SetColor(pInfo->m_ColorFeet.r * cs, pInfo->m_ColorFeet.g * cs, pInfo->m_ColorFeet.b * cs, Alpha);
 
+			Graphics()->TextureSet(OutLine == 1 ? pSkinTextures->m_FeetOutline : pSkinTextures->m_Feet);
 			Graphics()->RenderQuadContainerAsSprite(m_TeeQuadContainerIndex, QuadOffset, Position.x + pFoot->m_X * AnimScale, Position.y + pFoot->m_Y * AnimScale, w / 64.f, h / 32.f);
 		}
 	}
