@@ -638,13 +638,13 @@ int CEditor::PopupQuad(CEditor *pEditor, CUIRect View, void *pContext)
 	{
 		if(Prop == PROP_POS_X)
 		{
-			for(int k = 0; k < 5; ++k)
-				lQuads[i]->m_aPoints[k].x += OffsetX;
+			for(auto &Point : lQuads[i]->m_aPoints)
+				Point.x += OffsetX;
 		}
 		if(Prop == PROP_POS_Y)
 		{
-			for(int k = 0; k < 5; ++k)
-				lQuads[i]->m_aPoints[k].y += OffsetY;
+			for(auto &Point : lQuads[i]->m_aPoints)
+				Point.y += OffsetY;
 		}
 		if(Prop == PROP_POS_ENV)
 		{
@@ -1474,8 +1474,6 @@ int CEditor::PopupSelectConfigAutoMap(CEditor *pEditor, CUIRect View, void *pCon
 				if(pEditor->Input()->KeyPress(KEY_MOUSE_WHEEL_DOWN))
 					s_ScrollValue = clamp(s_ScrollValue + 1.0f / ScrollNum, 0.0f, 1.0f);
 			}
-			else
-				ScrollNum = 0;
 		}
 
 		// Margin for scrollbar
@@ -1492,7 +1490,7 @@ int CEditor::PopupSelectConfigAutoMap(CEditor *pEditor, CUIRect View, void *pCon
 
 	pEditor->UI()->ClipEnable(&View);
 
-	for(int i = -1; i < pAutoMapper->ConfigNamesNum(); i++)
+	for(int i = 0; i < pAutoMapper->ConfigNamesNum() + 1; i++)
 	{
 		if(ListCur > ListStopAt)
 			break;
@@ -1501,8 +1499,10 @@ int CEditor::PopupSelectConfigAutoMap(CEditor *pEditor, CUIRect View, void *pCon
 		{
 			View.HSplitTop(ButtonMargin, 0, &View);
 			View.HSplitTop(ButtonHeight, &Button, &View);
-			if(pEditor->DoButton_MenuItem(&s_AutoMapperConfigButtons[i], i == -1 ? "None" : pAutoMapper->GetConfigName(i), i == s_AutoMapConfigCurrent, &Button, 0, 0))
+			if(pEditor->DoButton_MenuItem(&s_AutoMapperConfigButtons[i], i == 0 ? "None" : pAutoMapper->GetConfigName(i - 1), i == s_AutoMapConfigCurrent, &Button, 0, 0))
+			{
 				s_AutoMapConfigSelected = i;
+			}
 		}
 		ListCur += ButtonHeight + ButtonMargin;
 	}
@@ -1532,7 +1532,7 @@ int CEditor::PopupSelectConfigAutoMapResult()
 
 	s_AutoMapConfigCurrent = s_AutoMapConfigSelected;
 	s_AutoMapConfigSelected = -100;
-	return s_AutoMapConfigCurrent;
+	return s_AutoMapConfigCurrent - 1;
 }
 
 // DDRace
