@@ -56,20 +56,6 @@ float VelocityRamp(float Value, float Start, float Range, float Curvature)
 	return 1.0f / powf(Curvature, (Value - Start) / Range);
 }
 
-void CCharacterCore::Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore *pTeams)
-{
-	m_pWorld = pWorld;
-	m_pCollision = pCollision;
-	m_pTeleOuts = NULL;
-
-	m_pTeams = pTeams;
-	m_Id = -1;
-	m_Hook = true;
-	m_Collision = true;
-	m_JumpedTotal = 0;
-	m_Jumps = 2;
-}
-
 void CCharacterCore::Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore *pTeams, std::map<int, std::vector<vec2>> *pTeleOuts)
 {
 	m_pWorld = pWorld;
@@ -78,10 +64,7 @@ void CCharacterCore::Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore
 
 	m_pTeams = pTeams;
 	m_Id = -1;
-	m_Hook = true;
-	m_Collision = true;
-	m_JumpedTotal = 0;
-	m_Jumps = 2;
+	Reset();
 }
 
 void CCharacterCore::Reset()
@@ -118,6 +101,10 @@ void CCharacterCore::Reset()
 	m_HasTelegunLaser = false;
 	m_FreezeEnd = 0;
 	m_DeepFrozen = false;
+
+	// never initialize both to 0
+	m_Input.m_TargetX = 0;
+	m_Input.m_TargetY = -1;
 }
 
 void CCharacterCore::Tick(bool UseInput)
@@ -274,18 +261,21 @@ void CCharacterCore::Tick(bool UseInput)
 			for(int i = 0; i < MAX_CLIENTS; i++)
 			{
 				CCharacterCore *pCharCore = m_pWorld->m_apCharacters[i];
-				if(!pCharCore || pCharCore == this || (!(m_Super || pCharCore->m_Super) && (!m_pTeams->CanCollide(i, m_Id) || pCharCore->m_Solo || m_Solo)))
+				if(!pCharCore || pCharCore == this || (!(m_Super || pCharCore->m_Super) && ((m_Id != -1 && !m_pTeams->CanCollide(i, m_Id)) || pCharCore->m_Solo || m_Solo)))
 					continue;
 
-				vec2 ClosestPoint = closest_point_on_line(m_HookPos, NewPos, pCharCore->m_Pos);
-				if(distance(pCharCore->m_Pos, ClosestPoint) < PhysSize + 2.0f)
+				vec2 ClosestPoint;
+				if(closest_point_on_line(m_HookPos, NewPos, pCharCore->m_Pos, ClosestPoint))
 				{
-					if(m_HookedPlayer == -1 || distance(m_HookPos, pCharCore->m_Pos) < Distance)
+					if(distance(pCharCore->m_Pos, ClosestPoint) < PhysSize + 2.0f)
 					{
-						m_TriggeredEvents |= COREEVENT_HOOK_ATTACH_PLAYER;
-						m_HookState = HOOK_GRABBED;
-						m_HookedPlayer = i;
-						Distance = distance(m_HookPos, pCharCore->m_Pos);
+						if(m_HookedPlayer == -1 || distance(m_HookPos, pCharCore->m_Pos) < Distance)
+						{
+							m_TriggeredEvents |= COREEVENT_HOOK_ATTACH_PLAYER;
+							m_HookState = HOOK_GRABBED;
+							m_HookedPlayer = i;
+							Distance = distance(m_HookPos, pCharCore->m_Pos);
+						}
 					}
 				}
 			}
@@ -328,7 +318,7 @@ void CCharacterCore::Tick(bool UseInput)
 		if(m_HookedPlayer != -1)
 		{
 			CCharacterCore *pCharCore = m_pWorld->m_apCharacters[m_HookedPlayer];
-			if(pCharCore && m_pTeams->CanKeepHook(m_Id, pCharCore->m_Id))
+			if(pCharCore && m_Id != -1 && m_pTeams->CanKeepHook(m_Id, pCharCore->m_Id))
 				m_HookPos = pCharCore->m_Pos;
 			else
 			{
@@ -395,41 +385,44 @@ void CCharacterCore::Tick(bool UseInput)
 
 			// handle player <-> player collision
 			float Distance = distance(m_Pos, pCharCore->m_Pos);
-			vec2 Dir = normalize(m_Pos - pCharCore->m_Pos);
-
-			bool CanCollide = (m_Super || pCharCore->m_Super) || (pCharCore->m_Collision && m_Collision && !m_NoCollision && !pCharCore->m_NoCollision && m_pWorld->m_Tuning[g_Config.m_ClDummy].m_PlayerCollision);
-
-			if(CanCollide && Distance < PhysSize * 1.25f && Distance > 0.0f)
+			if(Distance > 0)
 			{
-				float a = (PhysSize * 1.45f - Distance);
-				float Velocity = 0.5f;
+				vec2 Dir = normalize(m_Pos - pCharCore->m_Pos);
 
-				// make sure that we don't add excess force by checking the
-				// direction against the current velocity. if not zero.
-				if(length(m_Vel) > 0.0001)
-					Velocity = 1 - (dot(normalize(m_Vel), Dir) + 1) / 2;
+				bool CanCollide = (m_Super || pCharCore->m_Super) || (pCharCore->m_Collision && m_Collision && !m_NoCollision && !pCharCore->m_NoCollision && m_pWorld->m_Tuning[g_Config.m_ClDummy].m_PlayerCollision);
 
-				m_Vel += Dir * a * (Velocity * 0.75f);
-				m_Vel *= 0.85f;
-			}
-
-			// handle hook influence
-			if(m_Hook && m_HookedPlayer == i && m_pWorld->m_Tuning[g_Config.m_ClDummy].m_PlayerHooking)
-			{
-				if(Distance > PhysSize * 1.50f) // TODO: fix tweakable variable
+				if(CanCollide && Distance < PhysSize * 1.25f && Distance > 0.0f)
 				{
-					float Accel = m_pWorld->m_Tuning[g_Config.m_ClDummy].m_HookDragAccel * (Distance / m_pWorld->m_Tuning[g_Config.m_ClDummy].m_HookLength);
-					float DragSpeed = m_pWorld->m_Tuning[g_Config.m_ClDummy].m_HookDragSpeed;
+					float a = (PhysSize * 1.45f - Distance);
+					float Velocity = 0.5f;
 
-					vec2 Temp;
-					// add force to the hooked player
-					Temp.x = SaturatedAdd(-DragSpeed, DragSpeed, pCharCore->m_Vel.x, Accel * Dir.x * 1.5f);
-					Temp.y = SaturatedAdd(-DragSpeed, DragSpeed, pCharCore->m_Vel.y, Accel * Dir.y * 1.5f);
-					pCharCore->m_Vel = ClampVel(pCharCore->m_MoveRestrictions, Temp);
-					// add a little bit force to the guy who has the grip
-					Temp.x = SaturatedAdd(-DragSpeed, DragSpeed, m_Vel.x, -Accel * Dir.x * 0.25f);
-					Temp.y = SaturatedAdd(-DragSpeed, DragSpeed, m_Vel.y, -Accel * Dir.y * 0.25f);
-					m_Vel = ClampVel(m_MoveRestrictions, Temp);
+					// make sure that we don't add excess force by checking the
+					// direction against the current velocity. if not zero.
+					if(length(m_Vel) > 0.0001)
+						Velocity = 1 - (dot(normalize(m_Vel), Dir) + 1) / 2;
+
+					m_Vel += Dir * a * (Velocity * 0.75f);
+					m_Vel *= 0.85f;
+				}
+
+				// handle hook influence
+				if(m_Hook && m_HookedPlayer == i && m_pWorld->m_Tuning[g_Config.m_ClDummy].m_PlayerHooking)
+				{
+					if(Distance > PhysSize * 1.50f) // TODO: fix tweakable variable
+					{
+						float Accel = m_pWorld->m_Tuning[g_Config.m_ClDummy].m_HookDragAccel * (Distance / m_pWorld->m_Tuning[g_Config.m_ClDummy].m_HookLength);
+						float DragSpeed = m_pWorld->m_Tuning[g_Config.m_ClDummy].m_HookDragSpeed;
+
+						vec2 Temp;
+						// add force to the hooked player
+						Temp.x = SaturatedAdd(-DragSpeed, DragSpeed, pCharCore->m_Vel.x, Accel * Dir.x * 1.5f);
+						Temp.y = SaturatedAdd(-DragSpeed, DragSpeed, pCharCore->m_Vel.y, Accel * Dir.y * 1.5f);
+						pCharCore->m_Vel = ClampVel(pCharCore->m_MoveRestrictions, Temp);
+						// add a little bit force to the guy who has the grip
+						Temp.x = SaturatedAdd(-DragSpeed, DragSpeed, m_Vel.x, -Accel * Dir.x * 0.25f);
+						Temp.y = SaturatedAdd(-DragSpeed, DragSpeed, m_Vel.y, -Accel * Dir.y * 0.25f);
+						m_Vel = ClampVel(m_MoveRestrictions, Temp);
+					}
 				}
 			}
 		}
@@ -473,30 +466,33 @@ void CCharacterCore::Move()
 	{
 		// check player collision
 		float Distance = distance(m_Pos, NewPos);
-		int End = Distance + 1;
-		vec2 LastPos = m_Pos;
-		for(int i = 0; i < End; i++)
+		if(Distance > 0)
 		{
-			float a = i / Distance;
-			vec2 Pos = mix(m_Pos, NewPos, a);
-			for(int p = 0; p < MAX_CLIENTS; p++)
+			int End = Distance + 1;
+			vec2 LastPos = m_Pos;
+			for(int i = 0; i < End; i++)
 			{
-				CCharacterCore *pCharCore = m_pWorld->m_apCharacters[p];
-				if(!pCharCore || pCharCore == this)
-					continue;
-				if((!(pCharCore->m_Super || m_Super) && (m_Solo || pCharCore->m_Solo || !pCharCore->m_Collision || pCharCore->m_NoCollision || (m_Id != -1 && !m_pTeams->CanCollide(m_Id, p)))))
-					continue;
-				float D = distance(Pos, pCharCore->m_Pos);
-				if(D < 28.0f && D >= 0.0f)
+				float a = i / Distance;
+				vec2 Pos = mix(m_Pos, NewPos, a);
+				for(int p = 0; p < MAX_CLIENTS; p++)
 				{
-					if(a > 0.0f)
-						m_Pos = LastPos;
-					else if(distance(NewPos, pCharCore->m_Pos) > D)
-						m_Pos = NewPos;
-					return;
+					CCharacterCore *pCharCore = m_pWorld->m_apCharacters[p];
+					if(!pCharCore || pCharCore == this)
+						continue;
+					if((!(pCharCore->m_Super || m_Super) && (m_Solo || pCharCore->m_Solo || !pCharCore->m_Collision || pCharCore->m_NoCollision || (m_Id != -1 && !m_pTeams->CanCollide(m_Id, p)))))
+						continue;
+					float D = distance(Pos, pCharCore->m_Pos);
+					if(D < 28.0f && D >= 0.0f)
+					{
+						if(a > 0.0f)
+							m_Pos = LastPos;
+						else if(distance(NewPos, pCharCore->m_Pos) > D)
+							m_Pos = NewPos;
+						return;
+					}
 				}
+				LastPos = Pos;
 			}
-			LastPos = Pos;
 		}
 	}
 
@@ -552,6 +548,9 @@ void CCharacterCore::ReadDDNet(const CNetObj_DDNetCharacter *pObjDDNet)
 	m_NoHookHit = pObjDDNet->m_Flags & CHARACTERFLAG_NO_HOOK;
 	m_Super = pObjDDNet->m_Flags & CHARACTERFLAG_SUPER;
 
+	m_Hook = !m_NoHookHit;
+	m_Collision = !m_NoCollision;
+
 	// Endless
 	m_EndlessHook = pObjDDNet->m_Flags & CHARACTERFLAG_ENDLESS_HOOK;
 	m_EndlessJump = pObjDDNet->m_Flags & CHARACTERFLAG_ENDLESS_JUMP;
@@ -577,11 +576,21 @@ void CCharacterCore::Quantize()
 
 // DDRace
 
+void CCharacterCore::SetTeamsCore(CTeamsCore *pTeams)
+{
+	m_pTeams = pTeams;
+}
+
+void CCharacterCore::SetTeleOuts(std::map<int, std::vector<vec2>> *pTeleOuts)
+{
+	m_pTeleOuts = pTeleOuts;
+}
+
 bool CCharacterCore::IsSwitchActiveCb(int Number, void *pUser)
 {
 	CCharacterCore *pThis = (CCharacterCore *)pUser;
 	if(pThis->Collision()->m_pSwitchers)
-		if(pThis->m_pTeams->Team(pThis->m_Id) != (pThis->m_pTeams->m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER))
+		if(pThis->m_Id != -1 && pThis->m_pTeams->Team(pThis->m_Id) != (pThis->m_pTeams->m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER))
 			return pThis->Collision()->m_pSwitchers[Number].m_Status[pThis->m_pTeams->Team(pThis->m_Id)];
 	return false;
 }
