@@ -65,27 +65,6 @@ public:
 	bool Get(const char *pName, float *pValue);
 };
 
-inline vec2 GetDirection(int Angle)
-{
-	float a = Angle / 256.0f;
-	return vec2(cosf(a), sinf(a));
-}
-
-inline vec2 GetDir(float Angle)
-{
-	return vec2(cosf(Angle), sinf(Angle));
-}
-
-inline float GetAngle(vec2 Dir)
-{
-	if(Dir.x == 0 && Dir.y == 0)
-		return 0.0f;
-	float a = atanf(Dir.y / Dir.x);
-	if(Dir.x < 0)
-		a = a + pi;
-	return a;
-}
-
 inline void StrToInts(int *pInts, int Num, const char *pStr)
 {
 	int Index = 0;
@@ -247,8 +226,7 @@ public:
 
 	int m_TriggeredEvents;
 
-	void Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore *pTeams);
-	void Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore *pTeams, std::map<int, std::vector<vec2>> *pTeleOuts);
+	void Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore *pTeams = nullptr, std::map<int, std::vector<vec2>> *pTeleOuts = nullptr);
 	void Reset();
 	void Tick(bool UseInput);
 	void Move();
@@ -267,7 +245,9 @@ public:
 	int m_Colliding;
 	bool m_LeftWall;
 
-	// DDnet Character
+	// DDNet Character
+	void SetTeamsCore(CTeamsCore *pTeams);
+	void SetTeleOuts(std::map<int, std::vector<vec2>> *pTeleOuts);
 	void ReadDDNet(const CNetObj_DDNetCharacter *pObjDDNet);
 	bool m_Solo;
 	bool m_Jetpack;
