@@ -5,23 +5,23 @@
 #include <game/server/entity.h>
 class CCharacter;
 
-class CDragger: public CEntity
+class CDragger : public CEntity
 {
 	vec2 m_Core;
 	float m_Strength;
 	int m_EvalTick;
 	void Move();
 	void Drag();
-	CCharacter * m_Target;
+	CCharacter *m_Target;
 	bool m_NW;
 	int m_CaughtTeam;
 
-	CCharacter * m_SoloEnts[MAX_CLIENTS];
+	CCharacter *m_SoloEnts[MAX_CLIENTS];
 	int m_SoloIDs[MAX_CLIENTS];
-public:
 
+public:
 	CDragger(CGameWorld *pGameWorld, vec2 Pos, float Strength, bool NW,
-			int CaughtTeam, int Layer = 0, int Number = 0);
+		int CaughtTeam, int Layer = 0, int Number = 0);
 
 	virtual void Reset();
 	virtual void Tick();
@@ -30,12 +30,10 @@ public:
 
 class CDraggerTeam
 {
-	CDragger * m_Draggers[MAX_CLIENTS];
+	CDragger *m_Draggers[MAX_CLIENTS];
 
 public:
-
-	CDraggerTeam(CGameWorld *pGameWorld, vec2 Pos, float Strength, bool NW =
-			false, int Layer = 0, int Number = 0);
+	CDraggerTeam(CGameWorld *pGameWorld, vec2 Pos, float Strength, bool NW = false, int Layer = 0, int Number = 0);
 	//~CDraggerTeam();
 };
 
