@@ -29,12 +29,16 @@ class IGameController
 	int m_aNumSpawnPoints[3];
 
 	class CGameContext *m_pGameServer;
+	class CConfig *m_pConfig;
 	class IServer *m_pServer;
 
 protected:
 	CGameContext *GameServer() const { return m_pGameServer; }
+	CConfig *Config() { return m_pConfig; }
 	IServer *Server() const { return m_pServer; }
 
+	void DoActivityCheck();
+
 	struct CSpawnEval
 	{
 		CSpawnEval()
@@ -74,6 +78,48 @@ public:
 	IGameController(class CGameContext *pGameServer);
 	virtual ~IGameController();
 
+	// event
+	/*
+		Function: OnCharacterDeath
+			Called when a CCharacter in the world dies.
+
+		Arguments:
+			victim - The CCharacter that died.
+			killer - The player that killed it.
+			weapon - What weapon that killed it. Can be -1 for undefined
+				weapon when switching team or player suicides.
+	*/
+	virtual int OnCharacterDeath(class CCharacter *pVictim, class CPlayer *pKiller, int Weapon);
+	/*
+		Function: OnCharacterSpawn
+			Called when a CCharacter spawns into the game world.
+
+		Arguments:
+			chr - The CCharacter that was spawned.
+	*/
+	virtual void OnCharacterSpawn(class CCharacter *pChr);
+
+	virtual void HandleCharacterTiles(class CCharacter *pChr, int MapIndex);
+
+	/*
+		Function: OnEntity
+			Called when the map is loaded to process an entity
+			in the map.
+
+		Arguments:
+			index - Entity index.
+			pos - Where the entity is located in the world.
+
+		Returns:
+			bool?
+	*/
+	virtual bool OnEntity(int Index, vec2 Pos, int Layer, int Flags, int Number = 0);
+
+	virtual void OnPlayerDisconnect(class CPlayer *pPlayer, const char *pReason);
+
+	void OnReset();
+
+	// game
 	void DoWarmup(int Seconds);
 
 	void StartRound();
@@ -93,45 +139,10 @@ public:
 
 	virtual void Snap(int SnappingClient);
 
-	/*
-		Function: on_entity
-			Called when the map is loaded to process an entity
-			in the map.
-
-		Arguments:
-			index - Entity index.
-			pos - Where the entity is located in the world.
-
-		Returns:
-			bool?
-	*/
-	//virtual bool OnEntity(int Index, vec2 Pos);
-	virtual bool OnEntity(int Index, vec2 Pos, int Layer, int Flags, int Number = 0);
-
-	/*
-		Function: on_CCharacter_spawn
-			Called when a CCharacter spawns into the game world.
-
-		Arguments:
-			chr - The CCharacter that was spawned.
-	*/
-	virtual void OnCharacterSpawn(class CCharacter *pChr);
-
-	/*
-		Function: on_CCharacter_death
-			Called when a CCharacter in the world dies.
-
-		Arguments:
-			victim - The CCharacter that died.
-			killer - The player that killed it.
-			weapon - What weapon that killed it. Can be -1 for undefined
-				weapon when switching team or player suicides.
-	*/
-	virtual int OnCharacterDeath(class CCharacter *pVictim, class CPlayer *pKiller, int Weapon);
-
-	//
+	//spawn
 	virtual bool CanSpawn(int Team, vec2 *pPos);
 
+	virtual void DoTeamChange(class CPlayer *pPlayer, int Team, bool DoChatMsg = true);
 	/*
 
 	*/
@@ -140,7 +151,7 @@ public:
 	virtual bool CanJoinTeam(int Team, int NotThisID);
 	int ClampTeam(int Team);
 
-	virtual void PostReset();
+	virtual int64 GetMaskForPlayerWorldEvent(int Asker, int ExceptID = -1);
 
 	// DDRace
 
