@@ -15,14 +15,26 @@ public:
 	CGameControllerDDRace(class CGameContext *pGameServer);
 	~CGameControllerDDRace();
 
+	void OnCharacterSpawn(class CCharacter *pChr) override;
+	void HandleCharacterTiles(class CCharacter *pChr, int MapIndex) override;
+
+	void OnPlayerDisconnect(class CPlayer *pPlayer, const char *pReason) override;
+
+	void Tick() override;
+
+	void DoTeamChange(class CPlayer *pPlayer, int Team, bool DoChatMsg = true) override;
+
+	int64 GetMaskForPlayerWorldEvent(int Asker, int ExceptID = -1) override;
+
+	void InitTeleporter();
+
+	int GetPlayerTeam(int ClientID) const;
+
 	CGameTeams m_Teams;
 
 	std::map<int, std::vector<vec2>> m_TeleOuts;
 	std::map<int, std::vector<vec2>> m_TeleCheckOuts;
 
-	void InitTeleporter();
-	virtual void Tick();
-
 	std::shared_ptr<CScoreInitResult> m_pInitResult;
 };
 #endif // GAME_SERVER_GAMEMODES_DDRACE_H
