@@ -3,7 +3,9 @@
 #include <cstdio>
 #include <new>
 
+#include "entities/character.h"
 #include "gamemodes/DDRace.h"
+#include "player.h"
 #include "teams.h"
 #include <engine/shared/config.h>
 
@@ -18,7 +20,7 @@ CSaveTee::~CSaveTee()
 void CSaveTee::save(CCharacter *pChr)
 {
 	m_ClientID = pChr->m_pPlayer->GetCID();
-	str_copy(m_aName, pChr->m_pPlayer->Server()->ClientName(m_ClientID), sizeof(m_aName));
+	str_copy(m_aName, pChr->Server()->ClientName(m_ClientID), sizeof(m_aName));
 
 	m_Alive = pChr->m_Alive;
 	m_Paused = abs(pChr->m_pPlayer->IsPaused());
@@ -320,7 +322,7 @@ int CSaveTee::FromString(const char *String)
 		"%f\t%f\t%f\t%f\t%f\t"
 		"%d\t"
 		"%d\t%d\t%d\t"
-		"%*s\t" // discard the game uuid
+		"%36s\t"
 		"%d\t%d"
 		"%d\t%d\t%d\t%d\t"
 		"%d",
@@ -351,6 +353,7 @@ int CSaveTee::FromString(const char *String)
 		&m_aCpCurrent[20], &m_aCpCurrent[21], &m_aCpCurrent[22], &m_aCpCurrent[23], &m_aCpCurrent[24],
 		&m_NotEligibleForFinish,
 		&m_HasTelegunGun, &m_HasTelegunLaser, &m_HasTelegunGrenade,
+		m_aGameUuid,
 		&m_HookedPlayer, &m_NewHook,
 		&m_InputDirection, &m_InputJump, &m_InputFire, &m_InputHook,
 		&m_ReloadTimer);
@@ -363,8 +366,9 @@ int CSaveTee::FromString(const char *String)
 		m_HasTelegunGrenade = 0;
 		m_HasTelegunLaser = 0;
 		m_HasTelegunGun = 0;
+		FormatUuid(CalculateUuid("game-uuid-nonexistent@ddnet.tw"), m_aGameUuid, sizeof(m_aGameUuid));
 		// fall through
-	case 100:
+	case 101:
 		m_HookedPlayer = -1;
 		m_NewHook = false;
 		if(m_HookState == HOOK_GRABBED)
@@ -375,7 +379,7 @@ int CSaveTee::FromString(const char *String)
 		m_InputHook = 0;
 		m_ReloadTimer = 0;
 		// fall through
-	case 107:
+	case 108:
 		return 0;
 	default:
 		dbg_msg("load", "failed to load tee-string");
@@ -391,6 +395,11 @@ void CSaveTee::LoadHookedPlayer(const CSaveTeam *pTeam)
 	m_HookedPlayer = pTeam->m_pSavedTees[m_HookedPlayer].GetClientID();
 }
 
+bool CSaveTee::IsHooking() const
+{
+	return m_HookState == HOOK_GRABBED || m_HookState == HOOK_FLYING;
+}
+
 CSaveTeam::CSaveTeam(IGameController *Controller)
 {
 	m_pController = Controller;
@@ -722,7 +731,7 @@ bool CSaveTeam::MatchPlayers(const char (*paNames)[MAX_NAME_LENGTH], const int *
 		}
 		if(!Found)
 		{
-			str_format(pMessage, MessageLen, "'%s' don't belong to this team", paNames[i]);
+			str_format(pMessage, MessageLen, "'%s' doesn't belong to this team", paNames[i]);
 			return false;
 		}
 	}
