@@ -4,6 +4,9 @@
 #include "teehistorian.h"
 #include <engine/shared/config.h>
 
+#include "entities/character.h"
+#include "player.h"
+
 CGameTeams::CGameTeams(CGameContext *pGameContext) :
 	m_pGameContext(pGameContext)
 {
@@ -226,44 +229,41 @@ void CGameTeams::CheckTeamFinished(int Team)
 
 			for(unsigned int i = 0; i < PlayersCount; ++i)
 				OnFinish(TeamPlayers[i], Time, aTimestamp);
-			ChangeTeamState(Team, TEAMSTATE_FINISHED); //TODO: Make it better
-			//ChangeTeamState(Team, TEAMSTATE_OPEN);
+			ChangeTeamState(Team, TEAMSTATE_FINISHED); // TODO: Make it better
 			OnTeamFinish(TeamPlayers, PlayersCount, Time, aTimestamp);
 		}
 	}
 }
 
-bool CGameTeams::SetCharacterTeam(int ClientID, int Team)
+const char *CGameTeams::SetCharacterTeam(int ClientID, int Team)
 {
-	//Check on wrong parameters. +1 for TEAM_SUPER
-	if(ClientID < 0 || ClientID >= MAX_CLIENTS || Team < 0 || Team >= MAX_CLIENTS + 1)
-		return false;
-	//You can join to TEAM_SUPER at any time, but any other group you cannot if it started
+	if(ClientID < 0 || ClientID >= MAX_CLIENTS)
+		return "Invalid client ID";
+	if(Team < 0 || Team >= MAX_CLIENTS + 1)
+		return "Invalid team number";
 	if(Team != TEAM_SUPER && m_TeamState[Team] > TEAMSTATE_OPEN)
-		return false;
-	//No need to switch team if you there
+		return "This team started already";
 	if(m_Core.Team(ClientID) == Team)
-		return false;
+		return "You are in this team already";
 	if(!Character(ClientID))
-		return false;
-	//You cannot be in TEAM_SUPER if you not super
+		return "Your character is not valid";
 	if(Team == TEAM_SUPER && !Character(ClientID)->m_Super)
-		return false;
-	//if you begin race
-	if(Character(ClientID)->m_DDRaceState != DDRACE_NONE && Team != TEAM_SUPER)
-		return false;
-	//No cheating through noob filter with practice and then leaving team
+		return "You can't join super team if you don't have super rights";
+	if(Team != TEAM_SUPER && Character(ClientID)->m_DDRaceState != DDRACE_NONE)
+		return "You have started racing already";
+	// No cheating through noob filter with practice and then leaving team
 	if(m_Practice[m_Core.Team(ClientID)])
-		return false;
+		return "You have used practice mode already";
+
+	// you can not join a team which is currently in the process of saving,
+	// because the save-process can fail and then the team is reset into the game
+	if(Team != TEAM_SUPER && GetSaving(Team))
+		return "Your team is currently saving";
+	if(m_Core.Team(ClientID) != TEAM_SUPER && GetSaving(m_Core.Team(ClientID)))
+		return "This team is currently saving";
 
-	//you can not join a team which is currently in the process of saving,
-	//because the save-process can fail and then the team is reset into the game
-	if((Team != TEAM_SUPER && GetSaving(Team)) || (m_Core.Team(ClientID) != TEAM_SUPER && GetSaving(m_Core.Team(ClientID))))
-		return false;
 	SetForceCharacterTeam(ClientID, Team);
-
-	//GameServer()->CreatePlayerSpawn(Character(id)->m_Core.m_Pos, TeamMask());
-	return true;
+	return nullptr;
 }
 
 void CGameTeams::SetForceCharacterTeam(int ClientID, int Team)
@@ -306,7 +306,7 @@ void CGameTeams::ForceLeaveTeam(int ClientID)
 		for(int i = 0; i < MAX_CLIENTS; ++i)
 			if(i != ClientID && m_Core.Team(ClientID) == m_Core.Team(i))
 			{
-				NoOneInOldTeam = false; //all good exists someone in old team
+				NoOneInOldTeam = false; // all good exists someone in old team
 				break;
 			}
 		if(NoOneInOldTeam)
@@ -338,21 +338,7 @@ int CGameTeams::Count(int Team) const
 
 void CGameTeams::ChangeTeamState(int Team, int State)
 {
-	int OldState = m_TeamState[Team];
 	m_TeamState[Team] = State;
-	onChangeTeamState(Team, State, OldState);
-}
-
-void CGameTeams::onChangeTeamState(int Team, int State, int OldState)
-{
-	if(OldState != State && State == TEAMSTATE_STARTED)
-	{
-		// OnTeamStateStarting
-	}
-	if(OldState != State && State == TEAMSTATE_FINISHED)
-	{
-		// OnTeamStateFinishing
-	}
 }
 
 bool CGameTeams::TeamFinished(int Team)
@@ -537,12 +523,13 @@ void CGameTeams::OnFinish(CPlayer *Player, float Time, const char *pTimestamp)
 {
 	if(!Player || !Player->IsPlaying())
 		return;
-	//TODO:DDRace:btd: this ugly
+	// TODO:DDRace:btd: this ugly
 	const int ClientID = Player->GetCID();
 	CPlayerData *pData = GameServer()->Score()->PlayerData(ClientID);
 
 	char aBuf[128];
 	SetCpActive(Player, -2);
+	// Note that the "finished in" message is parsed by the client
 	str_format(aBuf, sizeof(aBuf),
 		"%s finished in: %d minute(s) %5.2f second(s)",
 		Server()->ClientName(ClientID), (int)Time / 60,
@@ -588,7 +575,7 @@ void CGameTeams::OnFinish(CPlayer *Player, float Time, const char *pTimestamp)
 				str_format(aBuf, sizeof(aBuf),
 					"%5.2f second(s) worse, better luck next time.",
 					Diff);
-			GameServer()->SendChatTarget(ClientID, aBuf, CGameContext::CHAT_SIX); //this is private, sent only to the tee
+			GameServer()->SendChatTarget(ClientID, aBuf, CGameContext::CHAT_SIX); // this is private, sent only to the tee
 		}
 	}
 	else
@@ -618,7 +605,6 @@ void CGameTeams::OnFinish(CPlayer *Player, float Time, const char *pTimestamp)
 		if(g_Config.m_SvNamelessScore || !str_startswith(Server()->ClientName(ClientID), "nameless tee"))
 		{
 			GameServer()->m_pController->m_CurrentRecord = Time;
-			//dbg_msg("character", "Finish");
 			NeedToSendNewRecord = true;
 		}
 	}
@@ -680,7 +666,7 @@ void CGameTeams::ProcessSaveTeam()
 {
 	for(int Team = 0; Team < MAX_CLIENTS; Team++)
 	{
-		if(m_pSaveTeamResult[Team] == nullptr || m_pSaveTeamResult[Team].use_count() != 1)
+		if(m_pSaveTeamResult[Team] == nullptr || !m_pSaveTeamResult[Team]->m_Completed)
 			continue;
 		if(m_pSaveTeamResult[Team]->m_aBroadcast[0] != '\0')
 			GameServer()->SendBroadcast(m_pSaveTeamResult[Team]->m_aBroadcast, -1);
@@ -697,6 +683,15 @@ void CGameTeams::ProcessSaveTeam()
 					m_pSaveTeamResult[Team]->m_SaveID,
 					m_pSaveTeamResult[Team]->m_SavedTeam.GetString());
 			}
+			for(int i = 0; i < m_pSaveTeamResult[Team]->m_SavedTeam.GetMembersCount(); i++)
+			{
+				if(m_pSaveTeamResult[Team]->m_SavedTeam.m_pSavedTees->IsHooking())
+				{
+					int ClientID = m_pSaveTeamResult[Team]->m_SavedTeam.m_pSavedTees->GetClientID();
+					if(GameServer()->m_apPlayers[ClientID] != nullptr)
+						GameServer()->SendChatTarget(ClientID, "Start holding the hook before loading the savegame to keep the hook");
+				}
+			}
 			ResetSavedTeam(m_pSaveTeamResult[Team]->m_RequestingPlayer, Team);
 			char aSaveID[UUID_MAXSTRSIZE];
 			FormatUuid(m_pSaveTeamResult[Team]->m_SaveID, aSaveID, UUID_MAXSTRSIZE);
@@ -852,6 +847,7 @@ void CGameTeams::ResetSavedTeam(int ClientID, int Team)
 	{
 		ChangeTeamState(Team, CGameTeams::TEAMSTATE_OPEN);
 		ResetSwitchers(Team);
+		m_Practice[Team] = false;
 	}
 	else
 	{
