@@ -7,6 +7,6 @@ TEST(GitRevision, ExistsOrNull)
 {
 	if(GIT_SHORTREV_HASH)
 	{
-		ASSERT_STRNE(GIT_SHORTREV_HASH, "");
+		EXPECT_STRNE(GIT_SHORTREV_HASH, "");
 	}
 }
