@@ -217,6 +217,8 @@ TEST(Str, StrCopyNum)
 	EXPECT_STREQ(aBuf, "Foobar");
 	str_utf8_truncate(aBuf, sizeof(aBuf), foo, 7);
 	EXPECT_STREQ(aBuf, "Foobaré");
+	str_utf8_truncate(aBuf, sizeof(aBuf), foo, 0);
+	EXPECT_STREQ(aBuf, "");
 
 	char aBuf2[8];
 	str_utf8_truncate(aBuf2, sizeof(aBuf2), foo, 7);
@@ -251,3 +253,65 @@ TEST(Str, StrCopyUtf8)
 	str_utf8_copy(aBuf, foo, 16);
 	EXPECT_STREQ(aBuf, "DDNet最好了");
 }
+
+TEST(Str, StrTime)
+{
+	char aBuf[32] = "foobar";
+
+	EXPECT_EQ(str_time(123456, TIME_DAYS, aBuf, 0), -1);
+	EXPECT_STREQ(aBuf, "foobar");
+
+	EXPECT_EQ(str_time(123456, TIME_MINS_CENTISECS + 1, aBuf, sizeof(aBuf)), -1);
+	EXPECT_STREQ(aBuf, "");
+
+	EXPECT_EQ(str_time(-123456, TIME_MINS_CENTISECS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "00:00.00");
+
+	EXPECT_EQ(str_time(INT64_MAX, TIME_DAYS, aBuf, sizeof(aBuf)), 23);
+	EXPECT_STREQ(aBuf, "1067519911673d 00:09:18");
+
+	EXPECT_EQ(str_time(123456, TIME_DAYS, aBuf, sizeof(aBuf)), 5);
+	EXPECT_STREQ(aBuf, "20:34");
+	EXPECT_EQ(str_time(1234567, TIME_DAYS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "03:25:45");
+	EXPECT_EQ(str_time(12345678, TIME_DAYS, aBuf, sizeof(aBuf)), 11);
+	EXPECT_STREQ(aBuf, "1d 10:17:36");
+
+	EXPECT_EQ(str_time(123456, TIME_HOURS, aBuf, sizeof(aBuf)), 5);
+	EXPECT_STREQ(aBuf, "20:34");
+	EXPECT_EQ(str_time(1234567, TIME_HOURS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "03:25:45");
+	EXPECT_EQ(str_time(12345678, TIME_HOURS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "34:17:36");
+
+	EXPECT_EQ(str_time(123456, TIME_MINS, aBuf, sizeof(aBuf)), 5);
+	EXPECT_STREQ(aBuf, "20:34");
+	EXPECT_EQ(str_time(1234567, TIME_MINS, aBuf, sizeof(aBuf)), 6);
+	EXPECT_STREQ(aBuf, "205:45");
+	EXPECT_EQ(str_time(12345678, TIME_MINS, aBuf, sizeof(aBuf)), 7);
+	EXPECT_STREQ(aBuf, "2057:36");
+
+	EXPECT_EQ(str_time(123456, TIME_HOURS_CENTISECS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "20:34.56");
+	EXPECT_EQ(str_time(1234567, TIME_HOURS_CENTISECS, aBuf, sizeof(aBuf)), 11);
+	EXPECT_STREQ(aBuf, "03:25:45.67");
+	EXPECT_EQ(str_time(12345678, TIME_HOURS_CENTISECS, aBuf, sizeof(aBuf)), 11);
+	EXPECT_STREQ(aBuf, "34:17:36.78");
+
+	EXPECT_EQ(str_time(123456, TIME_MINS_CENTISECS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "20:34.56");
+	EXPECT_EQ(str_time(1234567, TIME_MINS_CENTISECS, aBuf, sizeof(aBuf)), 9);
+	EXPECT_STREQ(aBuf, "205:45.67");
+	EXPECT_EQ(str_time(12345678, TIME_MINS_CENTISECS, aBuf, sizeof(aBuf)), 10);
+	EXPECT_STREQ(aBuf, "2057:36.78");
+}
+
+TEST(Str, StrTimeFloat)
+{
+	char aBuf[64];
+	EXPECT_EQ(str_time_float(123456.78, TIME_DAYS, aBuf, sizeof(aBuf)), 11);
+	EXPECT_STREQ(aBuf, "1d 10:17:36");
+
+	EXPECT_EQ(str_time_float(12.16, TIME_HOURS_CENTISECS, aBuf, sizeof(aBuf)), 8);
+	EXPECT_STREQ(aBuf, "00:12.16");
+}
